// interface_localization.js
//
// uses JavaScript to detect browser language
// uses Polyglot.js ( https://github.com/airbnb/polyglot.js ) to translate interface

// translators: add your language code here such as "es" for Spanish, "ru" for Russian
var knownLanguages = ["en","es","nl","it","fr","ru","de","zh-CN","ja","pt-BR","tr","uk","cs","cmn","yue"];

if ($.Options.locLang.val === 'auto') {
    // detect language with JavaScript
    preferredLanguage = window.navigator.userLanguage || window.navigator.language || 'en';
    if (knownLanguages.indexOf(preferredLanguage) > -1) {
        // en for en or similar
        preferredLanguage = preferredLanguage;
    } else if (knownLanguages.indexOf(preferredLanguage.split('-')[0]) > -1) {
        // en for en-US or similar
        preferredLanguage = preferredLanguage.split('-')[0];
    } else {
        // did not find match
        preferredLanguage = 'en';
    }
} else
    preferredLanguage = $.Options.locLang.val;

// set up Polyglot
polyglot = new Polyglot();
var wordset = {};

if(preferredLanguage == "en"){
    polyglot.locale("en");
    wordset = {
      "Actions ▼": "Actions ▼",
      "Active DHT nodes:": "Active DHT nodes: ",
      "Add DNS": "Add DNS",
      "Add peer": "Add peer",
      "ajax_error": "Ajax error: %{error}", // JavaScript error
      "All users publicly followed by": "All users publicly followed by",
      "Available": "Available", // username is available
      "Appearance": "Appearance",
      "Apply": "Apply",
      "Block chain information": "Block chain information",
      "Block chain is up-to-date, twister is ready to use!": "Block chain is up-to-date, twister is ready to use!",
      "Block generation": "Block generation ",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Cancel",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Local daemon is not connected to the network or block chain is outdated.\n" +
            "If you stay in this page your actions may not work.\n" +
            "Do you want to check [Network Status page](%{page}) instead?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Change user",
      "Checking...": "Checking...", // checking if username is available
      "Collapse": "Collapse", // smaller view of a post
      "Configure block generation": "Configure block generation",
      "Connections:": "Connections: ", // to network
      "Connection lost.": "Connection lost.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} day |||| %{smart_count} days",
      "Detailed information": "Detailed information",
      "DHT network down.": "DHT network down.",
      "Direct Messages": "Direct Messages",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Direct messages with %{username}",
      "Disable": "Disable",
      "display_mentions": "Display mentions",
      "Display retransmissions": "Display retransmissions",
      "DNS to obtain list of peers:": "DNS to obtain list of peers:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Downloading block chain, please wait before continuing (block chain is %{days} days old).",
      "download_posts_status": "Downloaded %{portion} posts", // Downloaded 10/30 posts
      "Enable": "Enable",
      "error": "Error: %{error}",
      "error_connecting_to_daemon": "Error connecting to local twister daemon.",
      "Error in 'createwalletuser' RPC.": "Error in 'createwalletuser' RPC.",
      "Error in 'importprivkey'": "Error in 'importprivkey' RPC: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Error in 'sendnewusertransaction' RPC.",
      "Expand": "Expand", // larger view of a post
      "Favorite": "Favorite",
      "File APIs not supported in this browser.": "File APIs not supported in this browser.",
      "Follow": "Follow",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Followed by",
      "followed_by": "Followed by %{username}",
      "Followers": "Followers",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Following",
      "Following users": "Following users",
      "Force connection to peer:": "Force connection to peer:",
      "General information": "General information",
      "Generate blocks (send promoted messages)": "Generate blocks (send promoted messages)",
      "Home": "Home", // homepage
      "hours": "%{smart_count} hour |||| %{smart_count} hours",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Internal error: lastPostId unknown (following yourself may fix!)",
      "Known peers:": "Known peers: ",
      "Last block is ahead of your computer time, check your clock.": "Last block is ahead of your computer time, check your clock.",
      "mentions_at": "Mentions @%{user}",
      "minutes": "%{smart_count} minute |||| %{smart_count} minutes",
      "Must be 16 characters or less.": "Must be 16 characters or less.", // username
      "Network": "Network",
      "Network config": "Network config",
      "Network status": "Network status",
      "New direct message...": "New direct message...",
      "New Post...": "New Post...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "You got",
      "in postboard": "in postboard",
      "in search result": "in search result",
      "in top trends": "in top trends",
      "new_posts": "%{smart_count} new post |||| %{smart_count} new posts",
      "new_mentions": "%{smart_count} new mention |||| %{smart_count} new mentions",
      "new_direct_messages": "%{smart_count} new direct message |||| %{smart_count} new direct messages",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "nobody", // used to promote a post without attaching the user
      "Not available": "Not available", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister cannot perform desktop notification: unknown error occured.",
      "notify_desktop_perm_denied": "Twister cannot perform desktop notification: permission denied.\n\nIf you want to get notifications, allow them for %{this_domain} in settings of your browser.",
      "notify_desktop_test": "All the twisters gonna twist.\nNow you are welcome too.",
      "notify_desktop_title": "Watch out, it's twister over here!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Number of blocks in block chain: ",
      "Number of CPUs to use": "Number of CPUs to use ",
      "Only alphanumeric and underscore allowed.": "Only alphanumeric and underscore allowed.",
      "peer address": "peer address",
      "Private": "Private",
      "Profile": "Profile",
      "Postboard": "Postboard",
      "post": "post", // verb - button to post a message
      "Post to promote:": "Post to promote: ",
      "Posts": "Posts",
      "propagating_nickname": "Propagating nickname %{username} to the network...",
      "Public": "Public",
      "Refresh": "Refresh",
      "retransmit_this": "Retransmit this post to your followers?",
      "Reply": "Reply",
      "Reply...": "Reply...",
      "reply_to": "Reply to %{fullname}",
      "Retransmit": "Retransmit",
      "Retransmits": "Retransmits",
      "Retransmitted by": "Retransmitted by",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "search",
      "seconds": "%{smart_count} second |||| %{smart_count} seconds",
      "send": "send",
      "Send post with username": "Send post with username ",
      "send_DM": "Send direct message",
      "Sent Post to @": "Sent Post to @",
      "Setup account": "Setup account",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "The File APIs are not fully supported in this browser.",
      "time_ago": "%{time} ago", // 5 minutes ago
      "Time of the last block:": "Time of the last block: ",
      "Type message here": "Type message here",
      "Unfollow": "Unfollow",
      "Update": "Update",
      "Auto updating": "Automatic updating",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Updating status...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Other peers have not yet accepted this new user.\n" +
                "Unfortunately it is not possible to save profile\n" +
                "or send any posts in this state.\n\n" +
                "Please wait a few minutes to continue.\n\n" +
                "The 'Save Changes' will be automatically enabled\n" +
                "when the process completes. (I promise this is\n"+
                "the last time you will have to wait before using\n" +
                "twister).\n\n" +
                "Tip: choose your avatar in the meantime!",
      "users_mentions": "Mentions of @%{username}",
      "users_profile": "%{username}'s Profile",
      "username_undefined": "Username undefined, login required.",
      "View": "View",
      "View All": "View All",
      "Who to Follow": "Who to Follow",
      "Your message was sent!": "Your message was sent!",
      "twister login": "twister login",
      "Existing local users": "Existing local users",
      "Or...": "Or...",
      "Create a new user": "Create a new user",
      "Login": "Login",
      "Check availability": "Check availability",
      "Create this nickname": "Create this nickname",
      "Type nickname here": "Type nickname here",
      "Import secret key": "Import secret key",
      "52-characters secret": "52-characters secret",
      "With nickname": "With nickname",
      "Import key": "Import key",
      "Client Version:": "Client Version:",
      "Mining difficulty:": "Mining difficulty:",
      "Block generation status": "Block generation status",
      "Current hash rate:": "Current hash rate:",
      "Terminate Daemon:": "Terminate Daemon:",
      "Exit": "Exit",
      "Save Changes": "Save Changes",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Secret key:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Options",
      "Switch to Promoted posts": "Switch to Promoted posts",
      "Switch to Normal posts": "Switch to Normal posts",
      "Use language": "Use language",
      "Ignore": "Ignore",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Notifications",
      "Desktop notifications": "Desktop notifications",
      "Sound notifications": "Sound notifications",
      "Volume": "Volume",
      "Test": "Test",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convert all",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Use proxy for image preview only",
      "Use external links behind a proxy": "Use external links behind a proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hour(s)",
      "second(s)": "second(s)",
      "only positive numbers!": "only positive numbers are allowed!",
      "Language filtering": "Language filtering",
      "By blacklist": "By blacklist",
      "By whitelist": "By whitelist",
      "Comma separated ISO 639-3 language codes": "Comma separated ISO 639-3 language codes",
      "Accuracy": "Accuracy",
      "Simulation mode": "Simulation mode",
      "This post is treated by language filter": "This post is %{treated} by language filter.",
      "blocked": "blocked",
      "passed": "passed",
      "not analyzed": "not analyzed",
      "Reason: this": "Reason: %{this}",
      "this doesnt contain that": "%{this} does not contain %{that}",
      "this is undefined": "%{this} is undefined",
      "blacklist": "blacklist",
      "whitelist": "whitelist",
      "language of this": "language of this",
      "its undefined language": "it is undefined language",
      "its this, blacklisted": "it is %{this}, blacklisted",
      "its this, whitelisted": "it is %{this}, whitelisted",
      "Most possible language: this": "Most possible language: %{this}",
      "Scope of usage": "Scope of usage",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Messages",
      "Edit profile": "Edit profile",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Secret Key",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Terminate Daemon": "Terminate Daemon",
      "New post": "New post",
      "Search": "Search",
      "Direct Msg": "Direct Msg",
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
      "New Users": "New Users",
      "Live tracking" : "Live tracking"
    };
}
if(preferredLanguage == "es"){
    polyglot.locale("es");
    wordset = {
      "Actions ▼": "Acciones ▼",
      "Active DHT nodes:": "Nodos DHT activos: ",
      "Add DNS": "Agregar DNS",
      "Add peer": "Agregar pares",
      "ajax_error": "Error ajax: %{error}", // JavaScript error
      "All users publicly followed by": "Todos los usuarios seguidos públicamente por",
      "Available": "Disponible", // username is available
      "Appearance": "Appearance",
      "Apply": "Aplicar",
      "Block chain information": "Información de la cadena de bloques",
      "Block chain is up-to-date, twister is ready to use!": "La cadena de bloques está puesta al día, twister está listo para usar!",
      "Block generation": "Generación de bloques ",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Cancelar",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Demonio local no está conectado a la red o la cadena de bloques no está actualizada.\n" +
            "Si te quedas en esta página tus acciones pueden no funcionar.\n" +
            "¿Quieres comprobar la [página de estado de la red](%{page}) en su lugar?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Cambiar de usuario",
      "Checking...": "Comprobando ...", // checking if username is available
      "Collapse": "Colapsar", // smaller view of a post
      "Configure block generation": "Configure la generación de bloques",
      "Connections:": "Conexiones: ", // to network
      "Connection lost.": "Conexión perdida.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} día |||| %{smart_count} días",
      "Detailed information": "Información detallada",
      "DHT network down.": "Red DHT caida.",
      "Direct Messages": "Mensajes directos",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Mensajes directos con %{username}",
      "Disable": "Inhabilitar",
      "display_mentions": "Visualización de menciones",
      "Display retransmissions": "Visualización de retransmisiones",
      "DNS to obtain list of peers:": "DNS para obtener la lista de los pares:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Descarga de la cadena de bloques, por favor espere antes de continuar (la cadena de bloques esta %{days} días).",
      "download_posts_status": "Post %{portion} descargados", // Downloaded 10/30 posts
      "Enable": "Permitir",
      "error": "Error: %{error}",
      "error_connecting_to_daemon": "Error al conectar con el demonio twister local.",
      "Error in 'createwalletuser' RPC.": "Error en RPC 'createwalletuser'.",
      "Error in 'importprivkey'": "Error en RPC 'importprivkey': %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Error en 'sendnewusertransaction' RPC.",
      "Expand": "Expandir", // larger view of a post
      "Favorite": "Favorito",
      "File APIs not supported in this browser.": "Los archivos API no compatibles con este navegador",
      "Follow": "Seguir",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Seguido por",
      "followed_by": "Seguido por %{username}",
      "Followers": "Seguidores",
      "Followers_of": "Seguidores de @%{alias}",
      "Following": "Siguiendo",
      "Following users": "Siguiendo los usuarios",
      "Force connection to peer:": "Forzar la conexión a  los pares:",
      "General information": "Información general",
      "Generate blocks (send promoted messages)": "Generar bloques (enviar mensajes promovidos)",
      "Home": "Inicio", // homepage
      "hours": "%{smart_count} hora |||| %{smart_count} horas",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Error interno: Último Mensaje Id desconocido (siguiendo a usted mismo lo puede solucionar!)",
      "Known peers:": "Pares conocidos: ",
      "Last block is ahead of your computer time, check your clock.": "El último bloque está por delante de su tiempo en la computadora, compruebe su reloj.",
      "mentions_at": "Menciones @%{user}",
      "minutes": "%{smart_count} minuto |||| %{smart_count} minutos",
      "Must be 16 characters or less.": "Debe tener 16 caracteres o menos.", // username
      "Network": "Red",
      "Network config": "Configuración de la red",
      "Network status": "Estado de la red",
      "New direct message...": "Nuevo mensaje directo...",
      "New Post...": "Nuevo Post...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "You got",
      "in postboard": "in postboard",
      "in search result": "in search result",
      "in top trends": "in top trends",
      "new_posts": "%{smart_count} nuevo post |||| %{smart_count} nuevos posts",
      "new_mentions": "%{smart_count} new mention |||| %{smart_count} new mentions",
      "new_direct_messages": "%{smart_count} new direct message |||| %{smart_count} new direct messages",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "Nadie", // used to promote a post without attaching the user
      "Not available": "No disponible", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister cannot perform desktop notification: unknown error occured.",
      "notify_desktop_perm_denied": "Twister cannot perform desktop notification: permission denied.\n\nIf you want to get notifications, allow them for %{this_domain} in settings of your browser.",
      "notify_desktop_test": "All the twisters gonna twist.\nNow you are welcome too.",
      "notify_desktop_title": "Watch out, it's twister over here!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Número de bloques en la cadena de bloques: ",
      "Number of CPUs to use": "Número de CPUs para utilizar ",
      "Only alphanumeric and underscore allowed.": "Sólo alfanuméricos y subrayados permitido.",
      "peer address": "dirección de pares",
      "Private": "Privado",
      "Profile": "Perfil",
      "Postboard": "Postboard",
      "post": "post", // verb - button to post a message
      "Post to promote:": "Post para promover: ",
      "Posts": "Posts",
      "propagating_nickname": "La propagación de seudónimo %{username} a la red...",
      "Public": "Publicar",
      "Refresh": "Recargar",
      "retransmit_this": "¿Retransmitir este mensaje a tus seguidores?",
      "Reply": "Responder",
      "Reply...": "Responder...",
      "reply_to": "Responder a %{fullname}",
      "Retransmit": "Retransmicion",
      "Retransmits": "Retransmiciones",
      "Retransmitted by": "Retransmitido por",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "search",
      "seconds": "%{smart_count} segundo |||| %{smart_count} segundos",
      "send": "send",
      "Send post with username": "Enviar post con nombre de usuario ",
      "send_DM": "Mensaje directo",
      "Sent Post to @": "El Post enviado a @",
      "Setup account": "Configuración de la cuenta",
      "shorten_URI": "Acortar URL",
      "shorten_URI_enter_link": "Introducir el enlace largo.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "Las API de archivos no son totalmente compatibles con este navegador.",
      "time_ago": "hace %{time}", // 5 minutes ago
      "Time of the last block:": "Hora del último bloque: ",
      "Type message here": "Escriba el mensaje aquí",
      "Unfollow": "Dejar de seguir",
      "Update": "Actualizar",
      "Auto updating": "Auto updating",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Actualización del estado ...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Otros pares no han aceptado este nuevo usuario.\n" +
                "Por desgracia, no es posible guardar el perfil\n" +
                "o enviar ningún mensaje en este estado.\n\n" +
                "Espera unos minutos para continuar.\n\n" +
                "El 'Guardar cambios' se activará automáticamente\n" +
                "cuando el proceso se complete. (Prometo esto es\n"+
                "la última vez que tendrá que esperar antes de usar\n" +
                "twister).\n\n" +
                "Consejo: elija su avatar mientras tanto!",
      "users_mentions": "Las menciones de @%{username}",
      "users_profile": "%{username}'s Profile",
      "username_undefined": "Usuario no definido, es necesario iniciar sesión.",
      "View": "Ver",
      "View All": "Ver todos",
      "Who to Follow": "A quién seguir",
      "Your message was sent!": "¡Tu mensaje ha sido enviado!",
      "twister login": "twister inicio de sesión",
      "Existing local users": "Los usuarios locales existentes",
      "Or...": "O...",
      "Create a new user": "Crear un nuevo usuario",
      "Login": "Iniciar sesión",
      "Check availability": "Consultar disponibilidad",
      "Create this nickname": "Crear este seudónimo",
      "Type nickname here": "Escriba seudónimo aquí",
      "Import secret key": "Importar clave secreta",
      "52-characters secret": "52-carácteres secretos",
      "With nickname": "Con seudónimo",
      "Import key": "Importar llave",
      "Client Version:": "Versión del cliente:",
      "Mining difficulty:": "Dificultad Minado:",
      "Block generation status": "Estado de generación de bloques",
      "Current hash rate:": "Tasa de hash actual:",
      "Terminate Daemon:": "Terminar Demonio:",
      "Exit": "Salir",
      "Save Changes": "Guardar cambios",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Clave secreta:",
      "You have to log in to post messages.": "Debes estar logueado para escribir mensajes.",
      "You have to log in to post replies.": "Debes estar logueado para escribir respuestas.",
      "You have to log in to retransmit messages.": "Debes estar logueado para retransmitir mensajes.",
      "You have to log in to use direct messages.": "Debes estar logueado para usar mensajes directos.",
      "You have to log in to follow users.": "Debes estar logueado para seguir usuarios.",
      "You are not following anyone because you are not logged in.": "No estás siguiendo a nadie porque no estás logueado.",
      "You don't have any followers because you are not logged in.": "No tienes ningun seguidor porque no estás logueado.",
      "No one can mention you because you are not logged in.": "Nadie te puede mencionar porque no estás logueado.",
      "You don't have any profile because you are not logged in.": "No tienes ningún perfil porque noe estás logueado.",
      "Options": "Opciones",
      "Switch to Promoted posts": "Cambiar a posts Promocionados",
      "Switch to Normal posts": "Cambiar a posts Normales",
      "Use language": "Utilizar idioma",
      "Ignore": "Ignorar",
      "Ignore and clear out": "Ignorar y limpiar",
      "Theme": "Tema",
      "Keys": "Teclas",
      "Notifications": "Notificaciones",
      "Desktop notifications": "Notificaciones de Escritorio",
      "Sound notifications": "Notificaciones de Sonido",
      "Volume": "Volumen",
      "Test": "Prueba",
      "Send key": "Send key",
      "Posts display": "Presentación de mensajes",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Vista previa de imágenes en línea",
      "Display": "Mostrar",
      "Line feeds": "Avances de línea",
      "Markout": "Markout marcado",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convertir todo",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Menciones",
      "Use proxy for image preview only": "Usa proxy para sólo para previsualización de imagenes",
      "Use external links behind a proxy": "Usa enlaces externos detrás de un proxy",
      "There aren't any posts with this hashtag.": "No hay posts con este hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hora(s)",
      "second(s)": "segundo(s)",
      "only positive numbers!": "only positive numbers are allowed!",
      "Language filtering": "Language filtering",
      "By blacklist": "By blacklist",
      "By whitelist": "By whitelist",
      "Comma separated ISO 639-3 language codes": "Comma separated ISO 639-3 language codes",
      "Accuracy": "Accuracy",
      "Simulation mode": "Simulation mode",
      "This post is treated by language filter": "This post is %{treated} by language filter.",
      "blocked": "blocked",
      "passed": "passed",
      "not analyzed": "not analyzed",
      "Reason: this": "Reason: %{this}",
      "this doesnt contain that": "%{this} does not contain %{that}",
      "this is undefined": "%{this} is undefined",
      "blacklist": "blacklist",
      "whitelist": "whitelist",
      "language of this": "language of this",
      "its undefined language": "it is undefined language",
      "its this, blacklisted": "it is %{this}, blacklisted",
      "its this, whitelisted": "it is %{this}, whitelisted",
      "Most possible language: this": "Most possible language: %{this}",
      "Scope of usage": "Scope of usage",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Mostrar conversación",
      "Mark all as read": "Marcar todos como leidos",
      "show_more_count": "%{smart_count} más...",
      "hide": "esconder",
      "Show more in this conversation...": "Mostrar más en esta conversación...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Posts normales",
      "Promoted posts": "Posts promocionados",
      'promoted': 'Promocionado',
      "Messages": "Mensajes",
      "Edit profile": "Editar perfil",
      "Top Trends": "Tendencias principales",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "deshacer",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Llave secreta",
      "Copy to clipboard": "Copiar al portapapeles",
      "Full name here": "Nombre completo aqui",
      "Describe yourself": "Describete",
      "Location": "Ubicación",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Dirección Bitmessage",
      "Language": "Idioma",
      "Sound": "Sonido",
      "Users": "Usuarios",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
      "New Users": "New Users",
      "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "uk"){
    polyglot.locale("uk");
    wordset = {
      "Actions ▼": "Дії ▼",
      "External IP:": "Публічний IP:",
      "External Port 1 (TCP):": "Перевірка порту 1 (TCP):",
      "External Port 2 (TCP+UDP):": "Перевірка порту 2 (TCP+UDP):",
      "Test open port (external site)": "Перевірити доступ до порту (зовнішній сайт)",
      "WebTorrent": "WebTorrent",
      "WebTorrent support to display shortened URL media": "Додати підтримку WebTorrent для відображення зкорочених URL",
      "WebTorrent default trackers to use (announce list)": "Список трекерів WebTorrent (список анонсу)",
      "Auto start download from users we follow": "Автоматично завантажувати від користувачів, на яких ви підписані",
      "Active DHT nodes:": "Активних вузлів DHT: ",
      "Add DNS": "Додати DNS",
      "Add peer": "Додати пір",
      "ajax_error": "Помилка Ajax: %{error}", // JavaScript error
      "All users publicly followed by": "Усі публічні користувачі яких читають",
      "Available": "Доступний", // username is available
      "Appearance": "Вигляд",
      "Apply": "Apply",
      "Block chain information": "Інформація про ланцюжок блоків",
      "Block chain is up-to-date, twister is ready to use!": "Ланцюг блоків оновлено, twister готовий до використання!",
      "Block generation": "Генерація блоку ",
      "busted_oh": "о ні!",
      "busted_avowal": "Мене зламали, коли я намагався ввести сюди цей підозрілий матеріал",
      "btn_ok": "Гаразд",
      "Cancel": "Відміна",
      'cant_get_requested_resourse': 'Неможливо отримати ресурс за посиланням %{link}\n статус: %{status}.',
      "clear_cache": "Очистити кеш",
      "Confirm": "Підтвердити",
      "сonfirm_group_leaving_header": "Підтвердити вихід із групи",
      "сonfirm_group_leaving_body": "Ви впевнені, що бажаєте вийти з групи %{alias}?",
      "confirm_switch_to_network":
            "Локальний сервер не підєднаний до мережі або ланцюг блоків не актуальний.\n" +
            "Якщо ви залишитесь на цій сторінці ваші дії можуть не спрацювати.\n" +
            "Чи бажаєте ви перевірити [сторінку зі статусом мережі](%{page})?",
      "confirm_terminate_daemon": "Ви впевнені, що бажаєте завершити роботу?\nКлієнт Twister буде зупинено допоки ви не запустите його знову.",
      "confirm_unfollow_@": "Ви впевнені, що хочете скасувати підписку на @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Ви впевнені, що бажаєте очистити кеш браузера від скорочених URI?",
      "Change user": "Змінити користувача",
      "Checking...": "Перевірка...", // checking if username is available
      "Collapse": "Згорнути", // smaller view of a post
      "Configure block generation": "Налаштувати генерацію блоку",
      "Connections:": "З’єднання: ", // to network
      "Connection lost.": "З’єднання втрачено.",
      "daemon_is_obsolete": "ваша версія ядра twister завтаріла, потрібно завантажити версію %{versionReq} або вище",
      "days": "%{smart_count} день |||| %{smart_count} днів",
      "Detailed information": "Детальна інформація",
      "DHT network down.": "Мережа DHT недоступна.",
      "Direct Messages": "Повідомлення",
      "Group Messages": "Групи",
      "Group Messages — New Group Creation": "Групові повідомлення — створення нової групи",
      "Group Messages — Join Group": "Групові повідомлення — приєднатися до групи",
      "group_key_cant_import": "Не вдається імпортувати ключ групи",
      "group_key_is_invalid_perhaps": "можливо ключ недійсний",
      "group_key_was_imported": "Ключ для групи %{alias} було імпортовано.\n"
        + "Повідомлення будуть незабаром отримані.",
      "direct_messages_with": "Особисті повідомлення з %{username}",
      "Disable": "Вимкнено",
      "display_mentions": "Показати сповіщення",
      "Display retransmissions": "Показати пересилання",
      "DNS to obtain list of peers:": "DNS для отримання пірів:",
      "do_not_show_it_again": "більше не показувати",
      "dns address": "адреса DNS",
      "downloading_block_chain": "Завантаження ланцюга блоків, будь ласка, зачекайте перед продовженням (ланцюг блоків віком %{days} днів).",
      "download_posts_status": "Завантажено %{portion} твістів", // Downloaded 10/30 posts
      "Enable": "Включено",
      "error": "Помилка: %{error}",
      "error_connecting_to_daemon": "Помилка з’єднання з локальним сервером twister.",
      "Error in 'createwalletuser' RPC.": "Помилка у 'createwalletuser' RPC.",
      "Error in 'importprivkey'": "Помилка у 'importprivkey' RPC: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Помилка у 'sendnewusertransaction' RPC.",
      "Expand": "Розгорнути", // larger view of a post
      "Favorite": "Улюблені",
      "File APIs not supported in this browser.": "File APIs не підтримуєтся цим браузером.",
      "Follow": "Читати",
      "Following config": "Налаштування читання",
      "select_way_to_follow_@": "У який спосіб ви бажаєте читати @%{alias}",
      "Followed by": "читає",
      "followed_by": "%{username} читає",
      "Followers": "Читачі",
      "Followers_of": "Читачі @%{alias}",
      "Following": "Читає",
      "Following users": "Підписані користувачі",
      "Force connection to peer:": "Примусове з’єднання з піром:",
      "General information": "Загальна інформація",
      "Generate blocks (send promoted messages)": "Відправка рекламних твістів за кожний згенерований блок",
      "Home": "Головна", // homepage
      "hours": "%{smart_count} година |||| %{smart_count} годин",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Внутрішня помилка: lastPostId невизначено (підписка на себе може виправити помилку!)",
      "Known peers:": "Відомі піри: ",
      "Last block is ahead of your computer time, check your clock.": "Останній блок датований майбутнім часом, перевірте свій годинник.",
      "mentions_at": "Сповіщення @%{user}",
      "minutes": "%{smart_count} хвилина |||| %{smart_count} хвилин",
      "Must be 16 characters or less.": "Повинно бути не більше 16 символів.", // username
      "Network": "Мережа",
      "Feedback": "Надіслати відгук",
      "Project Website": "Вебсайт проекту",
      "Donate": "Підтримати",
      "Network config": "Налаштування мережі",
      "Network status": "Статус мережі",
      "New direct message...": "Нове повідомлення...",
      "New Post...": "Що нового?",
      "New group": "Нова група",
      "Group description": "Опис групи",
      "Peers to invite": "Піри для запрошення",
      "Join group": "Долучитися до групи",
      "Select group(s)": "Обрати групу",
      "Create": "Створити",
      "Join": "Долучитись",
      "Invite": "Запросити",
      "Invite peers": "Запросити пірів",
      "Leave group": "Полишити групу",
      "You got": "Ви отримали",
      "in postboard": "в стрічці",
      "in search result": "в пошуковому запиті",
      "in top trends": "в трендах",
      "new_posts": "%{smart_count} новий твіст |||| %{smart_count} нових твістів",
      "new_mentions": "%{smart_count} нове сповіщення |||| %{smart_count} нових сповіщень",
      "new_direct_messages": "%{smart_count} нове повідомлення |||| %{smart_count} нові повідомлення",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "анонімно", // used to promote a post without attaching the user
      "Not available": "Не доступне", // username is not available
      "warn_following_not_any": "Ви ні за ким не слідкуєте!\nЗнайдіть когось.",
      "warn_followers_not_all": "Наразі, кількість підписників визначається орієнтовно.\n"
        + "Лічильник вказує лише кількість відомих однорангових користувачів, які спільно використовують ваш торрент.\n"
        + "У наведеному нижче списку включені, переважно, ті, за якими ви стежите.",
      "warn_followers_not_all_of": "Що ж, зараз немає простого способу отримати всіх підписників для будь-кого.\n"
        + "До наведеного нижче списку включено лише кілька, можливо, тих, хто підписався на @%{alias} публічно.",
      "notify_desktop_error": "Twister не може створити сповіщення: невідома помилка.",
      "notify_desktop_perm_denied": "Twister не може створити сповіщення на робочому столі: доступ обмежено.\n\nЯкщо ви бажаєте отримувати сповіщення, дозвольте їх виконання в налаштуваннях браузера для %{this_domain}.",
      "notify_desktop_test": "Є дещо новеньке у стрічці.",
      "notify_desktop_title": "Погляньте сюди!",
      "post_preview_dummy": 'це *bold*, ~italic~, -strike-trough- і _underlined_ текст.\n'
        + 'те само тільки екрановано: `*bold*, ~italic~, -strike-trough- і _underlined_`.\n'
        + 'посилання на [ваш чудовий логотип](%{logo}) і ваш чудовий сайт: %{site}.',
      "Number of blocks in block chain:": "Кількість блоків у ланцюгу: ",
      "Number of CPUs to use": "Кількість CPU до використання ",
      "Only alphanumeric and underscore allowed.": "Допустимі тільки латинські символи, числа та нижнє підкреслення.",
      "peer address": "адреса піру",
      "Private": "Особистий",
      "Profile": "Профіль",
      "Postboard": "Останні події",
      "post": "надіслати", // verb - button to post a message
      "Post to promote:": "Рекламний твіст: ",
      "Posts": "Твісти",
      "propagating_nickname": "Поширення інформації шо до %{username} у мережі...",
      "Public": "Публічний",
      "Refresh": "Оновити",
      "retransmit_this": "Переслати цей твіст читачам?",
      "Reply": "Відповісти",
      "Reply...": "Відповісти...",
      "reply_to": "Відповісти %{fullname}",
      "Retransmit": "Переслати",
      "Retransmits": "Пересилання",
      "Retransmitted by": "Переслано ",
      "Switch to Reply": "Переключіться на відповідь",
      "Switch to Retransmit": "Переключіться на пересилання",
      "search": "пошук",
      "seconds": "%{smart_count} секунда |||| %{smart_count} секунд",
      "send": "відіслати",
      "Send post with username": "Надіслати повідомлення від",
      "send_DM": "Надіслати особисте повідомлення",
      "Sent Post to @": "Надіслати твіст @",
      "Setup account": "Обліковий запис",
      "shorten_URI": "Зкоротити URL",
      "shorten_URI_enter_link": "Вкажіть довгу адресу.\n"
        + "Примітка: скорочення URL-адреси призведе до «порожнього» повідомлення від вашого імені, що містить повну URL-адресу.\n"
        + "Це спеціальне повідомлення не відображатиметься іншим користувачам, проте ваш лічильник повідомлень збільшиться.",
      "shorten_URI_its_public_is_it_ok": "Ваше посилання буде загальнодоступним! Ви з цим згодні?",
      "URI_shortener": "Зкорочення URI",
      "The File APIs are not fully supported in this browser.": "File APIs не повністю підтримується браузером.",
      "time_ago": "%{time} тому", // 5 minutes ago
      "Time of the last block:": "Час останнього блоку: ",
      "Type message here": "Напишіть повідомлення тут",
      "Unfollow": "Відписатись",
      "Update": "Оновити",
      "Auto updating": "Автоматичне оновлення",
      'updates_are_available': 'Доступні оновлення клієнту',
      'updates_not_available': 'Використовується актуальна версія',
      'updates_check_client': 'Перевірити наявність оновлень',
      'updates_repo_overview': 'Наразі ми перебуваємо на гілці %{branch} %{repo} під час фіксації\n %{commit} %{date}\n'
        + 'але HEAD його джерела вже в фіксації\n %{commitUpstream} %{dateUpstream}.',
      'updates_checkout_diff': 'Перевірте [різницю на GitHub](%{link}), щоб дізнатися, що саме змінилося.',
      'updates_checkout_diff_nfmt': 'Перегляньте різницю на GitHub, щоб дізнатися, що змінилося:\n  %{link}',
      'updates_upstream_isnt_changed': 'Відповідна гілка вихідного репозиторію, здається, не змінилася.',
      "Updating status...": "Оновлення статусу...", // status of block chain
      'new_account_briefing': 'Аккаунт успішно створено! Відбувається синхронізація з мережею.'
        + 'Будь ласка, не закривайте це вікно, це може зайняти кілька хвилин.\n\n'
        + 'Ваш секретний ключ: *%{secretKey}*\n\n'
        + 'Наполегливо рекомендуємо вам використати цей час, щоб зберегти свій секретний ключ. '
        + 'Роздрукуйте його, зробіть знімок екрана, скористайтеся камерою телефону або запишіть на аркуші паперу.\n\n'
        + 'Секретний ключ знадобиться для використання цього облікового запису на різних пристроях. '
        + 'Якщо ви втратите секретний ключ, ваш обліковий запис буде заблоковано назавжди '
        + '(примітка: ~це альфа-версія програмного забезпечення, і воно може вийти з ладу, спричинивши втрату даних~).\n\n'
        + 'Просто зачекайте. Кнопка підтвердження з’явиться швидше, ніж ви зможете сказати ~децентралізація~.',
      "user_not_yet_accepted": "Триває підтвердження нового користувача.\n" +
                "У цей час не можна редагувати профіль та надсилати повідомлення.\n\n" +
                "Будь ласка, зачекайте декілька хвилин.\n" +
                "Кнопка 'Зберегти зміни' стане доступною автоматично коли цей процес завершится.\n\n"+
                "Це востаннє, коли потрібно чекати перед використанням Twister",
      "users_mentions": "Події @%{username}",
      "users_profile": "Профіль %{username}",
      "username_undefined": "Користувач невизначений, необхідно вказати нік.",
      "View": "Дивитись",
      "View All": "Дивитись все",
      "Who to Follow": "Знайомтесь",
      "Your message was sent!": "Ваше повідомлення відправлене!",
      "twister login": "Вхід до Twister",
      "Existing local users": "Вже зареєстровані",
      "Or...": "Або...",
      "Create a new user": "Створити нового користувача",
      "Login": "Увійти",
      "Check availability": "Перевірити доступність",
      "Create this nickname": "Створити цей нік",
      "Type nickname here": "Напишіть нік тут",
      "Import secret key": "Імпортувати секретний ключ",
      "52-characters secret": "52-символьний ключ",
      "With nickname": "З ніком",
      "Import key": "Імпортувати",
      "Client Version:": "Версія клієнту:",
      "Mining difficulty:": "Складність майнінгу:",
      "Block generation status": "Статус генерації блоків",
      "Current hash rate:": "Поточна швидкість хешування:",
      "Terminate Daemon:": "Зупинити Twister:",
      "Exit": "Вихід",
      "Save Changes": "Зберегти зміни",
      "profile_saved": "Дані профілю збережено до DHT.",
      "profile_not_saved": "Дані профілю не збережено.",
      "Secret key:": "Секретний ключ:",
      "You have to log in to post messages.": "Ви повинні увійти для публікації повідомлень.",
      "You have to log in to post replies.": "Ви повинні увійти для публікації відповідей.",
      "You have to log in to retransmit messages.": "Ви повинні авторизуватися для перепублікації повідомлень.",
      "You have to log in to use direct messages.": "Ви повинні увійти для використання повідомлень.",
      "You have to log in to follow users.": "Ви повинні увійти для читання користувачів.",
      "You are not following anyone because you are not logged in.": "Ви ні за ким не слідкуєте, оскільки ще не авторизувалися.",
      "You don't have any followers because you are not logged in.": "У вас немає підписників, оскільки ви не ввійшли до системи.",
      "No one can mention you because you are not logged in.": "Ніхто не може вас згадати, тому що ви не ввійшли до системи.",
      "You don't have any profile because you are not logged in.": "У вас немає профілю, оскільки ви не ввійшли до системи.",
      "Options": "Опції",
      "Switch to Promoted posts": "Твісти від спонсорів",
      "Switch to Normal posts": "Твісти",
      "Use language": "Використати мову",
      "Ignore": "Ігнорувати",
      "Ignore and clear out": "Ігнорувати та очистити",
      "Theme": "Тема",
      "Keys": "Гарячі клавіші",
      "Notifications": "Оповіщення",
      "Desktop notifications": "Сповіщення на робочому столі",
      "Sound notifications": "Звукові сповіщення",
      "Volume": "Гучність",
      "Test": "Випроба",
      "Send key": "Надсилання твістів",
      "Posts display": "Відображення твістів",
      "Post editor": "Редактор твістів",
      "Post preview": "Post preview",
      "Inline image preview": "Контекстний перегляд зображення",
      "Display": "Відображати",
      "Line feeds": "Переведення рядка",
      "Markout": "Розмітка",
      "Supported punctuations:": "Підтримувані знаки пунктуації:",
      "Supported emotions:": "Підтримувані емоджі:",
      "Supported signs:": "Підтримувані підписи:",
      "Supported fractions:": "Підтримувані фракції:",
      "Automatic unicode conversion options": "Параметри автоматичного перетворення Unicode",
      "Convert punctuations to unicode": "Перетворити знаки пунктуації в юнікод",
      "Convert emotions codes to unicode symbols": "Конвертувати емоджи в символи Unicode",
      "Convert common signs to unicode": "Конвертувати загальні знаки в Юнікод",
      "Convert fractions to unicode": "Конвертувати фракції в Unicode",
      "Convert all": "Конвертувати усі",
      "Auto": "Автоматично",
      "Original": "Оригінал",
      "none": "немає",
      "Custom": "Вибірково",
      "Mentions": "Сповіщення",
      "Use proxy for image preview only": "Використовуати Proxy тільки для попереднього перегляду зображень",
      "Use external links behind a proxy": "Використовуйте зовнішні посилання за проксі",
      "There aren't any posts with this hashtag.": "Не знайдено публікацій за цим теґом.",
      "Split only new post": "Розділяти тільки нові публікації",
      "Split all": "Поєднати усі",
      "Don't split": "Не розділювати",
      "Split long posts": "Розділити довгі твісти",
      "Posts that begin with mention": "Твісти, що починаються з події",
      "Show all": "Відображати усі",
      "Show only if I am in": "Показувати, лише якщо я присутній",
      "Show if it's between users I follow": "Показувати, якщо це між користувачами, на яких я підписався",
      "Postboard displays": "Відображення публікацій",
      "RTs those are close to original twist": "Репости поруч з оригінальним повідомленням",
      "Show if the original is older than": "Показувати, якщо оригінал старіше за",
      "hour(s)": "годин",
      "second(s)": "секунд",
      "only positive numbers!": "Дозволені тільки позитивні числа!",
      "Language filtering": "Фільтрування за мовою",
      "By blacklist": "За чорним списком",
      "By whitelist": "За білим списком",
      "Comma separated ISO 639-3 language codes": "Розділені комою у мовному кодуванні ISO 639-3",
      "Accuracy": "Точність",
      "Simulation mode": "Режим симуляції",
      "This post is treated by language filter": "Цю публікацію %{treated} за мовним фільтром.",
      "blocked": "заблоковано",
      "passed": "пройшло",
      "not analyzed": "не проаналізовано",
      "Reason: this": "Причина: %{this}",
      "this doesnt contain that": "%{this} не містить %{that}",
      "this is undefined": "%{this} невідомо",
      "blacklist": "чорний список",
      "whitelist": "білий список",
      "language of this": "мова цього",
      "its undefined language": "це невідома мова",
      "its this, blacklisted": "%{this} занесено до чорного списку",
      "its this, whitelisted": "%{this} занесено до білого списку",
      "Most possible language: this": "Найбільш вірогідні мови: %{this}",
      "Scope of usage": "Сфера використання",
      "Show with every user name": "Відображати поряд з кожним ніком",
      "Show at profile modal only": "Відображати тільки в модальному вікні профілю",
      "Show if a user follows me": "Якщо користувач читає мене",
      "follows you": "читає вас",
      "Show conversation": "Показати бесіду",
      "Mark all as read": "Позначити усі як прочитані",
      "show_more_count": " та ще %{smart_count}...",
      "hide": "сховати",
      "Show more in this conversation...": "Показати більше у цій бесіді...",
      "conversation_title": "Розмова з @%{username}",
      "copy_to_clipboard": "Щоб скопіювати, натисніть Ctrl/Cmd+C а потім Enter щоб закрити",
      "Normal posts": "Твісти",
      "Promoted posts": "Спонсори",
      'promoted': 'Promoted',
      "Messages": "Повідомлення",
      "Edit profile": "Профіль",
      "Top Trends": "Тренди",
      "Twistday Reminder": "Нагадування про перший твіст",
      "Show upcoming in near future": "Відображати претендентів",
      "Who's celebrating Twistday": "Річницю святкує",
      "Today's luckies:": "Сьогоднішні щасливчики:",
      "Upcoming ones:": "Найближчим часом:",
      "post_rt_sign_prep": "ретвіст від",
      "post_rt_time_prep": "о",
      "undo": "відмінити",
      "Daemon exited...": "Twister заверив роботу...",
      "Secret Key": "Секретний ключ",
      "Copy to clipboard": "Скопіювати до буферу",
      "Full name here": "Повне ім’я",
      "Describe yourself": "Розкажіть про себе",
      "Location": "Розташування",
      "website": "Вебсайт",
      "Tox address": "Адреса Tox",
      "Bitmessage address": "Адреса Bitmessage",
      "Language": "Мова",
      "Sound": "Звук",
      "Users": "Користувачі",
      'dm_copy_outgoing_to_self': 'Повідомлення скопійовано самому собі',
      'dm_encrypt_local_cache': 'Зашифрувати локальне сховище браузеру',
      "Traffic information": "Статистика трафіку",
      "Direct messages with": "Співбесіда з",
      "DHT Torrents:": "Торенти DHT:",
      "Peers:": "Peers:",
      "Peer List Size:": "Розміск списку пірів:",
      "Active Requests:": "Активні запити:",
      "Download:": "Завантаження:",
      "Upload:": "Вивантаження:",
      "DHT Download:": "Завантаження DHT:",
      "DHT Upload:": "Вивантаження DHT:",
      "IP Overhead Download:": "Завантаження IP-адреси:",
      "IP Overhead Upload:": "Вивантаження IP-адреси:",
      "Payload Download:": "Завантаження корисного навантаження:",
      "Payload Upload:": "Вивантаження корисного навантаження:",
      "No favs here because you are not logged in." : "Список улюбленех повідомлень порожній, оскільки ви не авторизовані.",
      "users_favs": "Улюблене від @%{username}",
      "Favorites": "Улюблене",
      "You have to log in to favorite messages.": "Вам потрібно авторизуватися для додавання улублених повідомлень.",
      "fav_this": "Це тільки для вас?",
      "Last activity": "Остання активність",
        "New Users": "Нові користувачі",
        "Live tracking" : "Живе спостереження",
      "Translation": "Переклад",
        "Translation support to display localized twists": "Підтримка функції автоматичних перекладів повідомлень",
        "Translation API address": "Адреса API",
        "Get Instances": "Список провайдерів",
        "Translate": "Перекласти"
    };
}

if(preferredLanguage == "zh-CN"){
    polyglot.locale("zh-CN");
    wordset = {
      "Actions ▼": "操作 ▼",
      "Active DHT nodes:": "活动的DHT节点： ",
      "Add DNS": "添加DNS",
      "Add peer": "添加节点",
      "ajax_error": "AJAX错误：%{error}", // JavaScript error
      "All users publicly followed by": "公开关注的所有用户，被",
      "Available": "可用", // username is available
      "Appearance": "外观",
      "Apply": "应用",
      "Block chain information": "区块链信息",
      "Block chain is up-to-date, twister is ready to use!": "区块链已经更新，Twister 已经可以使用！",
      "Block generation": "区块生成器",
      "busted_oh": "噢，阻挡！",
      "busted_avowal": "系统检测到此用户试图在这里注入恶意代码",
      "btn_ok": "Okay",
      "Cancel": "取消",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "本地的后台进程没有连接到网络，或者区块链已经过时。\n" +
            "如果你留在此页面你的操作将不会生效。\n" +
            "你要跳转到[网络状态页](%{page})吗？",
      "confirm_terminate_daemon": "你确定要退出后台进程？\nTwister 客户端将停止工作。",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "切换用户",
      "Checking...": "检查中...", // checking if username is available
      "Collapse": "折叠", // smaller view of a post
      "Configure block generation": "配置区块生成器",
      "Connections:": "连接数：", // to network
      "Connection lost.": "连接中断。",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} 天",
      "Detailed information": "详细信息",
      "DHT network down.": "DHT网络中断。",
      "Direct Messages": "私信",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Direct messages with %{username}",
      "Disable": "关闭",
      "display_mentions": "显示",
      "Display retransmissions": "显示转发",
      "DNS to obtain list of peers:": "用DNS获取节点列表：",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "区块链下载中，请等待下载完成（区块链仍落后 %{days} 天）。",
      "download_posts_status": "已下载 %{portion} 推文", // Downloaded 10/30 posts
      "Enable": "开启",
      "error": "错误：%{error}",
      "error_connecting_to_daemon": "连接本地的 Twister 后台进程时出错",
      "Error in 'createwalletuser' RPC.": "'createwalletuser' RPC 错误",
      "Error in 'importprivkey'": "'importprivkey' 错误：%{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "'sendnewusertransaction' RPC 错误。",
      "Expand": "展开", // larger view of a post
      "Favorite": "收藏",
      "File APIs not supported in this browser.": "这个浏览器不支持 File API。",
      "Follow": "关注",
      "Following config": "关注配置",
      "select_way_to_follow_@": "你想以哪种方式关注 @%{alias}",
      "Followed by": "关注者",
      "followed_by": "被 %{username} 关注",
      "Followers": "粉丝",
      "Followers_of": "Followers of @%{alias}",
      "Following": "关注",
      "Following users": "关注的用户",
      "Force connection to peer:": "强制连接到节点：",
      "General information": "常规信息",
      "Generate blocks (send promoted messages)": "生成区块（发送推广信息）",
      "Home": "主页", // homepage
      "hours": "%{smart_count} 小时",
      "Internal error: lastPostId unknown (following yourself may fix!)": "内部错误：最后推文ID未知（关注你自己也许能修复！）",
      "Known peers:": "已知节点：",
      "Last block is ahead of your computer time, check your clock.": "最后的区块已经超前于你的系统时间，请检查你的时钟系统。",
      "mentions_at": "提及 @%{user}",
      "minutes": "%{smart_count} 分钟",
      "Must be 16 characters or less.": "必须少于16个字符", // username
      "Network": "网络",
      "Network config": "网络配置",
      "Network status": "网络状态",
      "New direct message...": "新私信...",
      "New Post...": "新推文...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "加入",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "你收到",
      "in postboard": "在时间线里",
      "in search result": "在搜索结果里",
      "in top trends": "在热词趋势里",
      "new_posts": "%{smart_count} 篇新推文",
      "new_mentions": "%{smart_count} 次新的提及",
      "new_direct_messages": "%{smart_count} 封新私信",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "无名", // used to promote a post without attaching the user
      "Not available": "用户名不可用", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister 无法发出桌面提醒：发生未知错误。",
      "notify_desktop_perm_denied": "Twister 无法发出桌面提醒：权限被拒绝。\n\n如果你想收到提醒，请在你的浏览器设置中允许 %{this_domain} 发出提醒。",
      "notify_desktop_test": "我们都在用 Twister。\n欢迎你的加入。",
      "notify_desktop_title": "注意，Twister 在这里！",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "区块链中的区块数：",
      "Number of CPUs to use": "使用CPU数目",
      "Only alphanumeric and underscore allowed.": "只允许字母和下划线。",
      "peer address": "节点地址",
      "Private": "私密",
      "Profile": "个人信息",
      "Postboard": "信息流",
      "post": "发送", // verb - button to post a message
      "Post to promote:": "要发送的推广信息：",
      "Posts": "推文",
      "propagating_nickname": "正在传播昵称 %{username} 到网络中...",
      "Public": "公开",
      "Refresh": "刷新",
      "retransmit_this": "转发推文给你的粉丝？",
      "Reply": "回复",
      "Reply...": "回复...",
      "reply_to": "回复 %{fullname}",
      "Retransmit": "转发",
      "Retransmits": "转发",
      "Retransmitted by": "转发于",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "搜索",
      "seconds": "%{smart_count} 秒",
      "send": "发送",
      "Send post with username": "发送推文的用户名",
      "send_DM": "发送私信",
      "Sent Post to @": "发送推文 @",
      "Setup account": "设置账号",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "这个浏览器不能完全支持 File API。",
      "time_ago": "%{time} 之前", // 5 minutes ago
      "Time of the last block:": "最新区块的时间：",
      "Type message here": "输入消息内容",
      "Unfollow": "取消关注",
      "Update": "更新",
      "Auto updating": "自动更新",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "状态更新中...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "其他节点还没有接受这个新用户。\n" +
        "很抱歉，现在你还不能保存你的个人简介\n" +
        "或发送新的推文。\n\n" +
        "请稍等几分钟再试试。\n\n" +
        "当相应操作完成后\n" +
        "“保存更改”按钮将会自动转换为可用。（我保证这是\n"+
        "在你使用 Twister 前的最后一次等待。\n\n" +
        "小提示：在这时你可以先选好你的头像！",
      "users_mentions": "提及 @%{username}",
      "users_profile": "%{username} 的个人简介",
      "username_undefined": "用户名未定义，请先登录！",
      "View": "查看",
      "View All": "查看全部",
      "Who to Follow": "推荐关注",
      "Your message was sent!": "你的信息已发送！",
      "twister login": "登录 Twister",
      "Existing local users": "现有本地用户",
      "Or...": "或...",
      "Create a new user": "创建新用户",
      "Login": "登录",
      "Check availability": "检查是否可用",
      "Create this nickname": "使用这个昵称",
      "Type nickname here": "输入你的昵称",
      "Import secret key": "导入私钥",
      "52-characters secret": "52位私钥字符串",
      "With nickname": "昵称",
      "Import key": "导入密钥",
      "Client Version:": "客户端版本：",
      "Mining difficulty:": "挖矿难度：",
      "Block generation status": "区块链生成器状态",
      "Current hash rate:": "当前hash速率：",
      "Terminate Daemon:": "终止后台进程：",
      "Exit": "退出",
      "Save Changes": "保存更改",
      "profile_saved": "个人信息已经保存到 DHT。",
      "profile_not_saved": "未保存个人信息。",
      "Secret key:": "私钥：",
      "You have to log in to post messages.": "你必须登录才能发送信息。",
      "You have to log in to post replies.": "你必须登录才能发送回复。",
      "You have to log in to retransmit messages.": "你必须登录才能转发信息",
      "You have to log in to use direct messages.": "你必须登录才能进行私信。",
      "You have to log in to follow users.": "你必须登录才能关注用户。",
      "You are not following anyone because you are not logged in.": "你没有关注任何人，因为你还没登录",
      "You don't have any followers because you are not logged in.": "你没有任何粉丝，因为你还没登录",
      "No one can mention you because you are not logged in.": "没人能提及你，因为你还没登录",
      "You don't have any profile because you are not logged in.": "你没有任何个人简介，因为你还没登录",
      "Options": "选项",
      "Switch to Promoted posts": "切换到推广发送",
      "Switch to Normal posts": "切换到普通发送",
      "Use language": "界面语言",
      "Ignore": "忽略",
      "Ignore and clear out": "忽略并清除",
      "Theme": "主题",
      "Keys": "按键",
      "Notifications": "通知",
      "Desktop notifications": "桌面通知",
      "Sound notifications": "声音通知",
      "Volume": "音量",
      "Test": "测试",
      "Send key": "发送键",
      "Posts display": "推文显示",
      "Post editor": "信息编辑器",
      "Post preview": "推文预览",
      "Inline image preview": "内嵌图像预览",
      "Display": "显示",
      "Line feeds": "换行",
      "Markout": "Markout 标记语言",
      "Supported punctuations:": "支持的标点符号：",
      "Supported emotions:": "支持的表情符号：",
      "Supported signs:": "支持的标记符号：",
      "Supported fractions:": "支持的分数形式：",
      "Automatic unicode conversion options": "自动转换 Unicode",
      "Convert punctuations to unicode": "转换标点符号到 Unicode 形式",
      "Convert emotions codes to unicode symbols": "转换表情码到 Unicode 符号",
      "Convert common signs to unicode": "转换常见符号到 Unicode 形式",
      "Convert fractions to unicode": "转换分数到 Unicode 形式",
      "Convert all": "全部转换",
      "Auto": "自动",
      "Original": "原版",
      "none": "无",
      "Custom": "自定义",
      "Mentions": "提及",
      "Use proxy for image preview only": "只对图像预览使用代理",
      "Use external links behind a proxy": "在代理的后方使用外部链接",
      "There aren't any posts with this hashtag.": "这个话题还没有任何推文呢。",
      "Split only new post": "只拆分新推文",
      "Split all": "全部拆分",
      "Don't split": "不拆分",
      "Split long posts": "拆分长推文",
      "Posts that begin with mention": "将推文的提及放在开头",
      "Show all": "全部显示",
      "Show only if I am in": "只显示如果我在其中",
      "Show if it's between users I follow": "显示我关注的用户",
      "Postboard displays": "信息流显示偏好",
      "RTs those are close to original twist": "靠近原推文的 RT",
      "Show if the original is older than": "显示，如果比原件晚",
      "hour(s)": "小时",
      "second(s)": "秒",
      "only positive numbers!": "只允许正数字！",
      "Language filtering": "语言过滤",
      "By blacklist": "用黑名单",
      "By whitelist": "用白名单",
      "Comma separated ISO 639-3 language codes": "英文逗号分隔的 ISO 639-3 语言代码",
      "Accuracy": "严格度",
      "Simulation mode": "模拟模式",
      "This post is treated by language filter": "这条推文被语言过滤器 %{treated}。",
      "blocked": "阻止",
      "passed": "放行",
      "not analyzed": "未分析",
      "Reason: this": "原因: %{this}",
      "this doesnt contain that": "%{this} 不包含 %{that}",
      "this is undefined": "%{this} 未定义",
      "blacklist": "黑名单",
      "whitelist": "白名单",
      "language of this": "此语言",
      "its undefined language": "它是未定义的语言",
      "its this, blacklisted": "它是 %{this}，被列在黑名单",
      "its this, whitelisted": "它是 %{this}，被列在白名单",
      "Most possible language: this": "最可能的语言：%{this}",
      "Scope of usage": "使用范围",
      "Show with every user name": "显示出每个用户名",
      "Show at profile modal only": "只显示在个人简介",
      "Show if a user follows me": "在一名用户关注我时",
      "follows you": "关注你",
      "Show conversation": "显示对话",
      "Mark all as read": "全部标记已读",
      "show_more_count": "还有 %{smart_count} 条...",
      "hide": "隐藏",
      "Show more in this conversation...": "显示更多此对话的内容...",
      "conversation_title": "@%{username} 的对话",
      "copy_to_clipboard": "按 Ctrl/Cmd+C 复制，然后按 Enter 关闭",
      "Normal posts": "普通推文",
      "Promoted posts": "推广推文",
      'promoted': 'Promoted',
      "Messages": "消息",
      "Edit profile": "编辑个人简介",
      "Top Trends": "热门趋势",
      "Twistday Reminder": "Twistday周年纪念日提醒",
      "Show upcoming in near future": "显示Twistday周年纪念日预告",
      "Who's celebrating Twistday": "谁在庆祝自己的Twistday周年纪念日",
      "Today's luckies:": "今天的幸运用户：",
      "Upcoming ones:": "即将到来的幸运用户：",
      "post_rt_sign_prep": "转推由",
      "post_rt_time_prep": "at",
      "undo": "撤销",
      "Daemon exited...": "后台进程已退出...",
      "Secret Key": "私钥",
      "Copy to clipboard": "复制到剪贴板",
      "Full name here": "输入全称",
      "Describe yourself": "介绍你自己",
      "Location": "位置",
      "website": "网站",
      "Tox address": "Tox 软件地址",
      "Bitmessage address": "比特信地址",
      "Language": "语言",
      "Sound": "声音",
      "Users": "用户",
      'dm_copy_outgoing_to_self': '将私信发一份副本给我',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Terminate Daemon": "终止后台进程",
      "New post": "新推文",
      "Search": "搜索",
      "Direct Msg": "私信",
      "Traffic information": "传输信息",
      "DHT Torrents:": "DHT种子:",
      "Peers:": "节点:",
      "Peer List Size:": "节点列表大小:",
      "Active Requests:": "活动请求:",
      "Download:": "下载:",
      "Upload:": "上传:",
      "DHT Download:": "DHT下载:",
      "DHT Upload:": "DHT上传:",
      "IP Overhead Download:": "IP层开销下载:",
      "IP Overhead Upload:": "IP层开销上传:",
      "Payload Download:": "有效载荷下载:",
      "Payload Upload:": "有效载荷上传:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "nl"){
    polyglot.locale("nl");
    wordset = {
      "Actions ▼": "Acties ▼",
      "Active DHT nodes:": "Actieve DHT nodes: ",
      "Add DNS": "DNS toevoegen",
      "Add peer": "Peer toevoegen",
      "ajax_error": "Ajax error: %{error}", // JavaScript error
      "All users publicly followed by": "Alle gebruikers openbaarlijk gevolgd door",
      "Available": "Beschikbaar", // username is available
      "Appearance": "Appearance",
      "Apply": "Apply",
      "Block chain information": "Block chain informatie",
      "Block chain is up-to-date, twister is ready to use!": "Block chain is up-to-date, twister is klaar voor gebruik!",
      "Block generation": "Block productie",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Annuleren",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Local daemon is not connected to the network or block chain is outdated.\n" +
            "If you stay in this page your actions may not work.\n" +
            "Do you want to check [Network Status page](%{page}) instead?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Gebruiker wijzigen",
      "Checking...": "Controleren...", // checking if username is available
      "Collapse": "Uitklappen", // smaller view of a post
      "Configure block generation": "Block productie configureren",
      "Connections:": "Connecties: ", // to network
      "Connection lost.": "Verbinding kwijt.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} dag |||| %{smart_count} dagen",
      "Detailed information": "Gedetailleerde informatie",
      "DHT network down.": "DHT netwerk down.",
      "Direct Messages": "Privéberichten",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Direct messages with %{username}",
      "Disable": "Uitschakelen",
      "display_mentions": "Toon vermeldingen",
      "Display retransmissions": "Toon retransmissions",
      "DNS to obtain list of peers:": "DNS om peers lijst op te halen:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Bezig met downloaden block chain, wacht a.u.b. voordat je doorgaat (block chain is %{days} dagen oud).",
      "download_posts_status": "%{portion} berichten gedownload", // Downloaded 10/30 posts
      "Enable": "Activeren",
      "error": "Error: %{error}",
      "error_connecting_to_daemon": "Error connecting to local twister daemon.",
      "Error in 'createwalletuser' RPC.": "Error in 'createwalletuser' RPC.",
      "Error in 'importprivkey'": "Error in 'importprivkey' RPC: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Error in 'sendnewusertransaction' RPC.",
      "Expand": "Uitklappen", // larger view of a post
      "Favorite": "Favoriet",
      "File APIs not supported in this browser.": "File APIs worden nie ondersteund in deze browser.",
      "Follow": "Volgen",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Gevolgd door",
      "followed_by": "Gevolgd door %{username}",
      "Followers": "Volgers",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Volgend",
      "Following users": "Volgende gebruikers",
      "Force connection to peer:": "Forceer connectie met peer:",
      "General information": "Algemene informatie",
      "Generate blocks (send promoted messages)": "Blocks genereren (verstuur promotie berichten)",
      "Home": "Home", // homepage
      "hours": "%{smart_count} uur |||| %{smart_count} uren",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Internal error: lastPostId unknown (following yourself may fix!)",
      "Known peers:": "Bekende peers: ",
      "Last block is ahead of your computer time, check your clock.": "Last block is ahead of your computer time, check your clock.",
      "mentions_at": "Vermeldingen @%{user}",
      "minutes": "%{smart_count} minuut |||| %{smart_count} minuten",
      "Must be 16 characters or less.": "Moet 16 tekens zijn, of minder.", // username
      "Network": "Netwerk",
      "Network config": "Netwerk configuratie",
      "Network status": "Netwerkstatus",
      "New direct message...": "Nieuw privébericht...",
      "New Post...": "Nieuw bericht...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "You got",
      "in postboard": "in postboard",
      "in search result": "in search result",
      "in top trends": "in top trends",
      "new_posts": "%{smart_count} nieuw bericht |||| %{smart_count} nieuwe berichten",
      "new_mentions": "%{smart_count} new mention |||| %{smart_count} new mentions",
      "new_direct_messages": "%{smart_count} new direct message |||| %{smart_count} new direct messages",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "nobody", // used to promote a post without attaching the user
      "Not available": "Niet beschikbaar", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister cannot perform desktop notification: unknown error occured.",
      "notify_desktop_perm_denied": "Twister cannot perform desktop notification: permission denied.\n\nIf you want to get notifications, allow them for %{this_domain} in settings of your browser.",
      "notify_desktop_test": "All the twisters gonna twist.\nNow you are welcome too.",
      "notify_desktop_title": "Watch out, it's twister over here!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Aantal blocks in block chain: ",
      "Number of CPUs to use": "Aantal CPUs om te gebruiken ",
      "Only alphanumeric and underscore allowed.": "Alleen alphanumeriek en underscore toegestaan.",
      "peer address": "peer adres",
      "Private": "Privé",
      "Profile": "Profiel",
      "Postboard": "Postboard",
      "post": "post", // verb - button to post a message
      "Post to promote:": "Bericht om te promoten: ",
      "Posts": "Berichten",
      "propagating_nickname": "Gebruikersnaam %{username} verspreiden op het netwerk...",
      "Public": "Publiek",
      "Refresh": "Vernieuwen",
      "retransmit_this": "Retransmit dit bericht naar je volgers?",
      "Reply": "Beantwoorden",
      "Reply...": "Beantwoord...",
      "reply_to": "Beantwoord %{fullname}",
      "Retransmit": "Retransmit",
      "Retransmits": "Retransmits",
      "Retransmitted by": "Retransmitted door",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "Zoeken",
      "seconds": "%{smart_count} seconde |||| %{smart_count} seconden",
      "send": "Verstuur",
      "Send post with username": "Verstuur bericht met gebruikersnaam ",
      "send_DM": "Verstuur privébericht",
      "Sent Post to @": "Verstuur bericht naar @",
      "Setup account": "Account instellingen",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "The File APIs are not fully supported in this browser.",
      "time_ago": "%{time} geleden", // 5 minutes ago
      "Time of the last block:": "Tijd van de laatste block: ",
      "Type message here": "Type bericht hier",
      "Unfollow": "Ontvolgen",
      "Update": "Update",
      "Auto updating": "Auto updating",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Status aan het updaten...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Other peers have not yet accepted this new user.\n" +
                "Unfortunately it is not possible to save profile\n" +
                "or send any posts in this state.\n\n" +
                "Please wait a few minutes to continue.\n\n" +
                "The 'Save Changes' will be automatically enabled\n" +
                "when the process completes. (I promise this is\n"+
                "the last time you will have to wait before using\n" +
                "twister).\n\n" +
                "Tip: choose your avatar in the meantime!",
      "users_mentions": "Vermeldingen voor @%{username}",
      "users_profile": "%{username}'s profiel",
      "username_undefined": "Gebruikersnaam niet opgegeven, inloggen verplicht.",
      "View": "Toon",
      "View All": "Toon alles",
      "Who to Follow": "Wie volgen?",
      "Your message was sent!": "Je bericht is verzonden!",
      "twister login": "twister login",
      "Existing local users": "Bestaande lokale gebruikers",
      "Or...": "Of...",
      "Create a new user": "Maak een nieuwe gebruiker aan",
      "Login": "Inloggen",
      "Check availability": "Controleer beschikbaarheid",
      "Create this nickname": "Maak deze gebruiker aan",
      "Type nickname here": "Gebruikersnaam",
      "Import secret key": "Importeer geheime sleutel",
      "52-characters secret": "52-tekens geheim",
      "With nickname": "Met gebruikersnaam",
      "Import key": "Importeer sleutel",
      "Client Version:": "Client versie:",
      "Mining difficulty:": "Mining moeilijkheid:",
      "Block generation status": "Block genereer status",
      "Current hash rate:": "Huidige hash snelheid:",
      "Terminate Daemon:": "Daemon beëindigen",
      "Exit": "Beëindigen",
      "Save Changes": "Opslaan",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Geheime sleutel:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Options",
      "Switch to Promoted posts": "Switch to Promoted posts",
      "Switch to Normal posts": "Switch to Normal posts",
      "Use language": "Use language",
      "Ignore": "Ignore",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Notifications",
      "Desktop notifications": "Desktop notifications",
      "Sound notifications": "Sound notifications",
      "Volume": "Volume",
      "Test": "Test",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convert all",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Use proxy for image preview only",
      "Use external links behind a proxy": "Use external links behind a proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hour(s)",
      "second(s)": "seconde(n)",
      "only positive numbers!": "only positive numbers are allowed!",
      "Language filtering": "Language filtering",
      "By blacklist": "By blacklist",
      "By whitelist": "By whitelist",
      "Comma separated ISO 639-3 language codes": "Comma separated ISO 639-3 language codes",
      "Accuracy": "Accuracy",
      "Simulation mode": "Simulation mode",
      "This post is treated by language filter": "This post is %{treated} by language filter.",
      "blocked": "blocked",
      "passed": "passed",
      "not analyzed": "not analyzed",
      "Reason: this": "Reason: %{this}",
      "this doesnt contain that": "%{this} does not contain %{that}",
      "this is undefined": "%{this} is undefined",
      "blacklist": "blacklist",
      "whitelist": "whitelist",
      "language of this": "language of this",
      "its undefined language": "it is undefined language",
      "its this, blacklisted": "it is %{this}, blacklisted",
      "its this, whitelisted": "it is %{this}, whitelisted",
      "Most possible language: this": "Most possible language: %{this}",
      "Scope of usage": "Scope of usage",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Messages",
      "Edit profile": "Edit profile",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Secret Key",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "it"){
    polyglot.locale("it");
    wordset = {
      "Actions ▼": "Azioni ▼",
      "Active DHT nodes:": "Nodi DHT attivi:",
      "Add DNS": "Connetti DNS",
      "Add peer": "Connetti nodo",
      "ajax_error": "Errore AJAX: %{error}", // JavaScript error
      "All users publicly followed by": "Utenti seguiti pubblicamente da",
      "Available": "Disponibile", // username is available
      "Appearance": "Appearance",
      "Apply": "Apply",
      "Block chain information": "Informazioni sulla catena di blocchi",
      "Block chain is up-to-date, twister is ready to use!": "Catena di blocchi aggiornata, Twister è pronto per l'uso!",
      "Block generation": "Generatore di blocchi:",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Cancella",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Il servizio locale non è connesso alla rete Twister o la catena di blocchi è vecchia.\n" +
            "Se rimani su questa pagina, Twister potrebbe non funzionare.\n" +
            "Vuoi controllare lo [stato della rete Twister](%{page}), invece?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Cambia utente",
      "Checking...": "Controllo in corso...", // checking if username is available
      "Collapse": "Chiudi", // smaller view of a post
      "Configure block generation": "Configura generatore di blocchi",
      "Connections:": "Connessioni: ", // to network
      "Connection lost.": "Connessione interrotta.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} giorno |||| %{smart_count} giorni",
      "Detailed information": "Informazioni dettagliate",
      "DHT network down.": "DHT network inaccessibile.",
      "Direct Messages": "Messaggi Diretti",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Messaggi Diretti come %{username}",
      "Disable": "Disabilitato",
      "display_mentions": "Mostra le menzioni",
      "Display retransmissions": "Mostra Ripubblicazioni",
      "DNS to obtain list of peers:": "DNS per la lista dei nodi:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Scaricamento della catena di blocchi in corso, attendere prego (la catena risale a %{days} giorni fa).",
      "download_posts_status": "Scaricati %{portion} messaggi", // Downloaded 10/30 posts
      "Enable": "Attivato",
      "error": "Errore: %{error}",
      "error_connecting_to_daemon": "Errore nella connessione al servizio Twister locale.",
      "Error in 'createwalletuser' RPC.": "Errore in 'createwalletuser' RPC.",
      "Error in 'importprivkey'": "Errore in 'importprivkey' RPC: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Errore in 'sendnewusertransaction' RPC.",
      "Expand": "Espandi", // larger view of a post
      "Favorite": "Preferito",
      "File APIs not supported in this browser.": "File APIs non supportati in questo browser.",
      "Follow": "Segui",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Seguito da",
      "followed_by": "Seguiti da %{username}",
      "Followers": "Lettori",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Seguiti",
      "Following users": "Utenti seguiti",
      "Force connection to peer:": "Forza connessione al nodo:",
      "General information": "Informazioni",
      "Generate blocks (send promoted messages)": "Genera blocchi (invia messaggi pubblicitari)",
      "Home": "Twister", // homepage -- no direct translation in Italian...?
      "hours": "%{smart_count} ora |||| %{smart_count} ore",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Errore interno: lastPostId sconosciuto (prova a seguire te stesso per risolvere!)",
      "Known peers:": "Nodi conosciuti: ",
      "Last block is ahead of your computer time, check your clock.": "L'ultimo blocco è più recente del tuo orario, controlla l'orologio di sistema.",
      "mentions_at": "Chi menziona @%{user}",
      "minutes": "%{smart_count} minuto |||| %{smart_count} minuti",
      "Must be 16 characters or less.": "Massimo 16 caratteri.", // username
      "Network": "Rete",
      "Network config": "Configurazione della rete",
      "Network status": "Status della rete",
      "New direct message...": "Nuovo messaggio diretto...",
      "New Post...": "Nuovo messaggio...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "You got",
      "in postboard": "in postboard",
      "in search result": "in search result",
      "in top trends": "in top trends",
      "new_posts": "%{smart_count} nuovo messaggio |||| %{smart_count} nuovi messaggi",
      "new_mentions": "%{smart_count} new mention |||| %{smart_count} new mentions",
      "new_direct_messages": "%{smart_count} new direct message |||| %{smart_count} new direct messages",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "nessuno", // used to promote a post without attaching the user
      "Not available": "Non disponibile", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister cannot perform desktop notification: unknown error occured.",
      "notify_desktop_perm_denied": "Twister cannot perform desktop notification: permission denied.\n\nIf you want to get notifications, allow them for %{this_domain} in settings of your browser.",
      "notify_desktop_test": "All the twisters gonna twist.\nNow you are welcome too.",
      "notify_desktop_title": "Watch out, it's twister over here!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Numero di blocchi nella catena: ",
      "Number of CPUs to use": "Numero di processori da usare:",
      "Only alphanumeric and underscore allowed.": "Sono permessi solo caratteri alfanumerici e '_'",
      "peer address": "Indirizzo del nodo",
      "Private": "Privato",
      "Profile": "Profilo",
      "Postboard": "Bacheca",
      "post": "Invia", // verb - button to post a message
      "Post to promote:": "Messaggio pubblicitario: ",
      "Posts": "Messaggi",
      "propagating_nickname": "Propagazione sul network del nome %{username} in corso...",
      "Public": "Pubblico",
      "Refresh": "Ricarica",
      "retransmit_this": "Ripubblica questo post ai tuoi lettori?",
      "Reply": "Rispondi",
      "Reply...": "Rispondi...",
      "reply_to": "Rispondi a %{fullname}",
      "Retransmit": "Ripubblica",
      "Retransmits": "Ripubblicati",
      "Retransmitted by": "Ripubblicato da",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "Cerca",
      "seconds": "%{smart_count} secondo |||| %{smart_count} secondi",
      "send": "Invia",
      "Send post with username": "Pubblica come utente ",
      "send_DM": "Messaggi Diretti inviati",
      "Sent Post to @": "Messaggi inviati a @",
      "Setup account": "Configurazione Utente",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "Le API File non sono interamente supportate da questo browser.",
      "time_ago": "%{time} fa", // 5 minutes ago
      "Time of the last block:": "Orario del blocco più recente: ",
      "Type message here": "Scrivi qui",
      "Unfollow": "Smetti di seguire",
      "Update": "Aggiorna",
      "Auto updating": "Auto updating",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Aggiornamento in corso...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Gli altri nodi non hanno ancora accettato il nuovo utente.\n" +
                "Al momento non puoi salvare il profilo o spedire messaggi.\n" +
                "Attendi qualche minuto prima di continuare.\n\n" +
                "Please wait a few minutes to continue.\n\n" +
                "Il pulsante 'Salva modifiche' sarà abilitato automaticamente appena il processo sarà completato.\n" +
                "(Prometto che è l'ultima attesa prima di poter usare Twister!).\n\n" +
                "Suggerimento: nel frattempo, trova un'immagine da usare come avatar!",
      "users_mentions": "Menzioni di @%{username}",
      "users_profile": "Profilo di %{username}",
      "username_undefined": "Utente non specificato, è necessario il login.",
      "View": "Vedi",
      "View All": "Mostra tutti",
      "Who to Follow": "Chi seguire?",
      "Your message was sent!": "Il messaggio è stato inviato!",
      "twister login": "twister login",
      "Existing local users": "Existing local users",
      "Or...": "Or...",
      "Create a new user": "Create a new user",
      "Login": "Login",
      "Check availability": "Check availability",
      "Create this nickname": "Create this nickname",
      "Type nickname here": "Type nickname here",
      "Import secret key": "Import secret key",
      "52-characters secret": "52-characters secret",
      "With nickname": "With nickname",
      "Import key": "Import key",
      "Client Version:": "Client Version:",
      "Mining difficulty:": "Mining difficulty:",
      "Block generation status": "Block generation status",
      "Current hash rate:": "Current hash rate:",
      "Terminate Daemon:": "Terminate Daemon:",
      "Exit": "Exit",
      "Save Changes": "Save Changes",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Secret key:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Options",
      "Switch to Promoted posts": "Switch to Promoted posts",
      "Switch to Normal posts": "Switch to Normal posts",
      "Use language": "Use language",
      "Ignore": "Ignore",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Notifiche",
      "Desktop notifications": "Notifiche desktop",
      "Sound notifications": "Notifiche sonore",
      "Volume": "Volume",
      "Test": "Prova",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convert all",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Use proxy for image preview only",
      "Use external links behind a proxy": "Use external links behind a proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hour(s)",
      "second(s)": "secondo(i)",
      "only positive numbers!": "only positive numbers are allowed!",
      "Language filtering": "Language filtering",
      "By blacklist": "By blacklist",
      "By whitelist": "By whitelist",
      "Comma separated ISO 639-3 language codes": "Comma separated ISO 639-3 language codes",
      "Accuracy": "Accuracy",
      "Simulation mode": "Simulation mode",
      "This post is treated by language filter": "This post is %{treated} by language filter.",
      "blocked": "blocked",
      "passed": "passed",
      "not analyzed": "not analyzed",
      "Reason: this": "Reason: %{this}",
      "this doesnt contain that": "%{this} does not contain %{that}",
      "this is undefined": "%{this} is undefined",
      "blacklist": "blacklist",
      "whitelist": "whitelist",
      "language of this": "language of this",
      "its undefined language": "it is undefined language",
      "its this, blacklisted": "it is %{this}, blacklisted",
      "its this, whitelisted": "it is %{this}, whitelisted",
      "Most possible language: this": "Most possible language: %{this}",
      "Scope of usage": "Scope of usage",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Messages",
      "Edit profile": "Edit profile",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Secret Key",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "fr"){
    polyglot.locale("fr");
    wordset = {
      "Actions ▼": "Actions ▼",
      "Active DHT nodes:": "Noeuds DHT actifs: ",
      "Add DNS": "Ajouter un DNS",
      "Add peer": "Ajouter un pair",
      "ajax_error": "Erreur ajax: %{error}", // JavaScript error
      "All users publicly followed by": "Tous les utilisateurs suivis publiquement par",
      "Available": "Disponible", // username is available
      "Appearance": "Apparence",
      "Apply": "Apply",
      "Block chain information": "Informations à propos de la chaîne de blocs",
      "Block chain is up-to-date, twister is ready to use!": "La chaîne de blocs est à jour, Twister est maintenant fonctionnel!",
      "Block generation": "Production de blocs",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Annuler",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Le démon local n'est pas connecté au réseau ou la chaîne de blocs n'est pas à jour.\n" +
            "Si vous restez dans cette page vos actions peuvent ne pas être prises en compte.\n" +
            "Voulez-vous consulter la [page d'état du réseau](%{page}) à la place?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Changer d'utilisateur",
      "Checking...": "Vérification...", // checking if username is available
      "Collapse": "Fermer", // smaller view of a post
      "Configure block generation": "Configuration de la production de blocs",
      "Connections:": "Connexions: ", // to network
      "Connection lost.": "Connexion perdue.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} jour |||| %{smart_count} jours",
      "Detailed information": "Informations détaillées",
      "DHT network down.": "Panne du réseau DHT.",
      "Direct Messages": "Messages privés",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Messages privés avec %{username}",
      "Disable": "Désactiver",
      "display_mentions": "Afficher les mentions",
      "Display retransmissions": "Afficher les retransmissions",
      "DNS to obtain list of peers:": "DNS où obtenir une liste des pairs:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Téléchargement de la chaîne de blocs, veuillez patienter avant de continuer (la chaîne de blocs a %{days} jours de retard).",
      "download_posts_status": "%{portion} billets téléchargés", // Downloaded 10/30 posts
      "Enable": "Activer",
      "error": "Erreur: %{error}",
      "error_connecting_to_daemon": "Erreur de connexion, impossible de se connecter au démon Twister.",
      "Error in 'createwalletuser' RPC.": "Erreur RPC dans 'createwalletuser'.",
      "Error in 'importprivkey'": "Erreur RPC dans 'importprivkey': %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Error RPC dans 'sendnewusertransaction'.",
      "Expand": "Ouvrir", // larger view of a post
      "Favorite": "Favori",
      "File APIs not supported in this browser.": "L'API de fichiers n'est pas pris en charge dans votre navigateur.",
      "Follow": "Suivre",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Suivi par",
      "followed_by": "Suivi par %{username}",
      "Followers": "Abonnés",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Abonnements",
      "Following users": "Following users",
      "Force connection to peer:": "Forcer la connection à un pair:",
      "General information": "Informations générales",
      "Generate blocks (send promoted messages)": "Produire des blocs (envoyer des messages promus)",
      "Home": "Accueil", // homepage
      "hours": "%{smart_count} heure |||| %{smart_count} heures",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Erreur interne: lastPostId inconnu",
      "Known peers:": "Pairs connus: ",
      "Last block is ahead of your computer time, check your clock.": "Le dernier bloc est en avance par rapport à l'heure de votre machine, vérifiez votre horloge.",
      "mentions_at": "Mentions @%{user}",
      "minutes": "%{smart_count} minute |||| %{smart_count} minutes",
      "Must be 16 characters or less.": "Doit contenir de 16 caractères ou moins.", // username
      "Network": "Réseau",
      "Network config": "Configuration réseau",
      "Network status": "État du réseau",
      "New direct message...": "Nouveau message privé...",
      "New Post...": "Nouveau billets...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "Tu as",
      "in postboard": "sur le postboard",
      "in search result": "dans les résultats de recherche",
      "in top trends": "dans les top trends",
      "new_posts": "%{smart_count} nouveau billet |||| %{smart_count} nouveaux billets",
      "new_mentions": "%{smart_count} nouvelle mention |||| %{smart_count} nouvelles mentions",
      "new_direct_messages": "%{smart_count} nouveau message privé |||| %{smart_count} nouveaux messages privés",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "nobody", // used to promote a post without attaching the user
      "Not available": "Non disponible", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Impossible d'afficher les notifications: une erreur inconnue est survenue.",
      "notify_desktop_perm_denied": "Impossible d'afficher les notifications: autorisation refusée.\n\nSi tu souhaites afficher les notifications, autorise le %{this_domain} dans les paramêtres de ton navigateur.",
      "notify_desktop_test": "All the twisters gonna twist.\nBienvenue à toi!",
      "notify_desktop_title": "Youhou! Ça bouge sur ton twister!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Nombre de blocs dans la chaîne de blocs: ",
      "Number of CPUs to use": "Nombre de processeurs à utiliser",
      "Only alphanumeric and underscore allowed.": "Seuls les caractères alphanumériques et la barre de soulignement sont permis.",
      "peer address": "adresse des pairs",
      "Private": "Privé",
      "Profile": "Profil",
      "Postboard": "Fil",
      "post": "envoyer", // verb - button to post a message
      "Post to promote:": "Billet à promouvoir: ",
      "Posts": "Billets",
      "propagating_nickname": "Propagation de votre pseudo %{username} sur le réseau...",
      "Public": "Public",
      "Refresh": "Actualiser",
      "retransmit_this": "Retransmettre ce billet à vos abonnés ?",
      "Reply": "Répondre",
      "Reply...": "Répondre...",
      "reply_to": "Répondre à %{fullname}",
      "Retransmit": "Retransmission",
      "Retransmits": "Retransmissions",
      "Retransmitted by": "Retransmis par",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "recherche",
      "seconds": "%{smart_count} seconde |||| %{smart_count} secondes",
      "send": "envoyer",
      "Send post with username": "Envoyer le billet avec le pseudo",
      "send_DM": "Message privé envoyé",
      "Sent Post to @": "Envoyer un billet à @",
      "Setup account": "Configuration du compte",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "L'API de fichier n'est pas entièrement pris en charge dans votre navigateur.",
      "time_ago": "Il y a %{time}", // 5 minutes ago
      "Time of the last block:": "Heure du dernier bloc: ",
      "Type message here": "Tapez votre message ici",
      "Unfollow": "Se désabonner",
      "Update": "Mettre à jour",
      "Auto updating": "Mise à jour automatique",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Mise à jour du statut...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Les autres pairs n'ont pas encore accepté ce nouvel utilisateur.\n" +
                "Malheureusement, il n'est pas possible d'enregistrer le profil\n" +
                "ou envoyer des billets pour le moment.\n\n" +
                "Veuillez attendre quelques minutes avant de continuer.\n\n" +
                "Le bouton 'enregistrer' s'affichera automatiquement\n" +
                "lorsque le processus sera terminé. (Je vous promets que\n"+
                "c'est la dernière fois que vous devrez attendre avant d'utiliser\n" +
                "Twister).\n\n" +
                "Astuce: pendant ce temps, choisissez votre avatar !",
      "users_mentions": "Mentions de @%{username}",
      "users_profile": "Profil de %{username}",
      "username_undefined": "Nom d'utilisateur indéfini, login requis.",
      "View": "Voir",
      "View All": "Voir tous",
      "Who to Follow": "Qui suivre",
      "Your message was sent!": "Votre message a été envoyé!",
      "twister login": "Nom d'utilisateur Twister",
      "Existing local users": "Utilisateurs locaux existants",
      "Or...": "Ou...",
      "Create a new user": "Créer un nouvel utilisateur",
      "Login": "Se connecter",
      "Check availability": "Vérifier la disponibilité",
      "Create this nickname": "Créer cet utilisateur",
      "Type nickname here": "Tapez votre nom d'utilisateur ici",
      "Import secret key": "Importer une clef secrète",
      "52-characters secret": "Clef secrète de 52 caractères",
      "With nickname": "Avec le nom d'utilisateur",
      "Import key": "Importer la clef secrète",
      "Client Version:": "Version du client:",
      "Mining difficulty:": "Difficulté du minage:",
      "Block generation status": "Statut de la production de blocs",
      "Current hash rate:": "Current hash rate:",
      "Terminate Daemon:": "Tuer le démon:",
      "Exit": "Quitter",
      "Save Changes": "Sauvegarder les changements",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Clef secrète:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Options",
      "Switch to Promoted posts": "Switch to Promoted posts",
      "Switch to Normal posts": "Switch to Normal posts",
      "Use language": "Use language",
      "Ignore": "Ignore",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Notifications",
      "Desktop notifications": "Notifications de bureau",
      "Sound notifications": "Notifications sonores",
      "Volume": "Volume",
      "Test": "Test",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convert all",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Use proxy for image preview only",
      "Use external links behind a proxy": "Use external links behind a proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hour(s)",
      "second(s)": "seconde(s)",
      "only positive numbers!": "Seuls les nombres >= 0 sont autorisés!",
      "Language filtering": "Filtre par langue",
      "By blacklist": "Par blacklist",
      "By whitelist": "Par liste blanche",
      "Comma separated ISO 639-3 language codes": "Codes ISO 639-3 séparés par une virgule",
      "Accuracy": "Précision",
      "Simulation mode": "Mode simulation",
      "This post is treated by language filter": "Ce post est %{treated} par le filtre de langue.",
      "blocked": "bloqué",
      "passed": "transmis",
      "not analyzed": "non analysé",
      "Reason: this": "Raison: %{this}",
      "this doesnt contain that": "%{this} ne contient pas %{that}",
      "this is undefined": "%{this} est indéfini",
      "blacklist": "blacklist",
      "whitelist": "liste blanche",
      "language of this": "Langue",
      "its undefined language": "Langue indéfinie",
      "its this, blacklisted": "C'est %{this}, sur blacklist",
      "its this, whitelisted": "C'est %{this}, sur liste blanche",
      "Most possible language: this": "Langue la plus probable: %{this}",
      "Scope of usage": "Champ d'application",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Messages",
      "Edit profile": "Edit profile",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Secret Key",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "ru"){
    polyglot.locale("ru");

    // list of the English words and translations
    wordset = {
       "Actions ▼": "Действия ▼", // , comma after each match except the last
       "Active DHT nodes:": "Активные узлы DHT: ",
       "Add DNS": "Добавить DNS",
       "Add peer": "Добавить пира",
       "ajax_error": "Ajax ошибка: %{error}", // JavaScript error
       "All users publicly followed by": "Все публичные пользователи на которых подписан",
       "Available": "Доступно",
       "Appearance": "Отображение",
       "Apply": "Применять",
       "Block chain information": "Информация цепочки блоков",
       "Block chain is up-to-date, twister is ready to use!": "Цепочка блоков обновлена, twister готов к использованию!",
       "Block generation": "Майнинг блоков ",
       "busted_oh": "о нет!",
       "busted_avowal": "я пойман на попытке вставить этот подозрительный код здесь",
       "btn_ok": "Лады́",
       "Cancel": "Отмена",
      'cant_get_requested_resourse': 'Невозможно получить %{link}\n  статус: %{status}.',
       "clear_cache": "Очистить кэш",
       "Confirm": "Несомненно",
       "сonfirm_group_leaving_header": "Подтверждение ухода из группы",
       "сonfirm_group_leaving_body": "Воистину покинуть группу %{alias}?",
       "confirm_switch_to_network":
            "Локальный демон не подключен к сети или цепочка блоков устарела.\n" +
            "Если вы останетесь на этой странице ваши действия могут быть не выполнены.\n" +
            "Не хотите перейти на [страницу настройки сети](%{page})?",
       "confirm_terminate_daemon": "Вы уверены, что хотите выключить демон?\nTwister клиент перестанет работать.",
       "confirm_unfollow_@": "Действительно отписаться от @%{alias}?",
       "confirm_uri_shortener_clear_cache": "В самом деле очистить кэш сокращённых URI в браузере?",
       "Change user": "Сменить пользователя",
       "Checking...": "Проверка...",
       "Collapse": "Свернуть",
       "Configure block generation": "Настройка майнинга",
       "Connections:": "Соединений: ",
       "Connection lost.": "Соединение с сетью было потеряно.",
       "daemon_is_obsolete": "твистер демон устарел, необходима версия %{versionReq} или выше",
       "days": "%{smart_count} день |||| %{smart_count} дней",
       "Detailed information": "Подробная информация",
       "DHT network down.": "Недоступна DHT сеть.",
       "Direct Messages": "Личные сообщения",
       "Group Messages": "Групповые сообщения",
       "Group Messages — New Group Creation": "Групповые сообщения — Создать новую группу",
       "Group Messages — Join Group": "Групповые сообщения — Присоединиться к группе",
       "group_key_cant_import": "невозможно импортировать ключ ЛС группы",
       "group_key_is_invalid_perhaps": "возможно, неправильный ключ",
       "group_key_was_imported": "Ключ для ЛС группы %{alias} импортирован.\n"
         + "Её сообщения будут подгружены в скором времени.",
       "direct_messages_with": "Личная переписка с %{username}",
       "Disable": "Отключено",
       "display_mentions": "Показать упоминания",
       "Display retransmissions": "Показать репосты",
       "DNS to obtain list of peers:": "DNS адрес для получения пиров:",
       "do_not_show_it_again": "не показывать больше",
       "downloading_block_chain": "Загрузка цепочки блоков, пожалуйста подождите, (Цепочка блоков устарела на %{days} дней).",
       "download_posts_status": "Загружено %{portion} постов", // Downloaded 10/30 posts
       "Enable": "Включено",
       "error": "Ошибка: %{error}",
       "error_connecting_to_daemon": "Ошибка подключения к локальному демону twisterd.",
       "Error in 'createwalletuser' RPC.": "Ошибка при обращении к RPC - при попытке выполнить 'createwalletuser'.",
       "Error in 'importprivkey'": "Ошибка при обращении к RPC - при попытке выполнить 'importprivkey' %{rpc}",
       "Error in 'sendnewusertransaction' RPC.": "Ошибка при обращении к RPC - при попытке выполнить 'sendnewusertransaction'",
       "Expand": "Развернуть",
       "Favorite": "Избранное",
       "File APIs not supported in this browser.": "Ваш браузер не поддерживает File APIs.",
       "Follow": "Подписаться",
       "Following config": "Настройка подписки",
       "select_way_to_follow_@": "Выбери же тип подписки на @%{alias}",
       "Followed by": "Подписчик у",
       "followed_by": "%{username} подписан",
       "Followers": "Читатели",
       "Followers_of": "Читатели @%{alias}",
       "Following": "Читаемые",
       "Following users": "Подписанные пользователи",
       "Force connection to peer:": "Принудительно подключиться к пиру:",
       "General information": "Основное",
       "Generate blocks (send promoted messages)": "Майнинг (отправка рекламных сообщений)",
       "Home": "Главная", // homepage
       "hours": "%{smart_count} час |||| %{smart_count} часов",
       "Internal error: lastPostId unknown (following yourself may fix!)": "Внутренняя ошибка: lastPostId неизвестен (Попробуйте подписаться сами на себя, это должно помочь!)",
       "Known peers:": "Известные пиры: ",
       "Last block is ahead of your computer time, check your clock.": "Последний полученный блок опережает время вашего компьютера, проверьте правильно ли работают часы.",
       "mentions_at": "Упоминания @%{user}",
       "minutes": "%{smart_count} минута |||| %{smart_count} минут",
       "Must be 16 characters or less.": "Должно быть не более 16 знаков.",
       "Network": "Сеть",
       "Network config": "Настройка сети",
       "Network status": "Состояние сети",
       "New direct message...": "Новое личное сообщение...",
       "New Post...": "Новый пост...",
       "New group": "Создать группу",
       "Group description": "Описание группы",
       "Peers to invite": "Кого пригласить",
       "Join group": "Присоединиться",
       "Select group(s)": "Выберите группу(ы)",
       "Create": "Создать",
       "Join": "Присоединиться",
       "Invite": "Пригласить",
       "Invite peers": "Пригласить кого",
       "Leave group": "Покинуть группу",
       "You got": "У вас",
       "in postboard": "в ленте",
       "in search result": "в результатах поиска",
       "in top trends": "в топе трендов",
       "new_posts": "%{smart_count} новый пост |||| %{smart_count} новых постов",
       "new_mentions": "%{smart_count} новое упоминание |||| %{smart_count} новых упоминаний",
       "new_direct_messages": "%{smart_count} новое личное сообщение |||| %{smart_count} новых личных сообщений",
       "new_group_messages": "%{smart_count} новое сообщение в группе |||| %{smart_count} новых групповых сообщений",
       "nobody": "Анонимно", // used to promote a post without attaching the user
       "Not available": "Недоступно",
       "warn_following_not_any": "Пока что таки нет читаемых!\nПоди ж найди кого-нибудь да и подпишись.",
       "warn_followers_not_all": "Пока что нет простого способа узнать список всех подписчиков.\n"
         + "Счётчик показывает лишь количество известных пиров твоего торрента.\n"
         + "В списке ниже включены те, главным образом, на кого подписан(а) ты сам(а).",
       "warn_followers_not_all_of": "Пока что нет простого способа узнать список всех подписчиков.\n"
         + "В списке ниже, возможно, лишь некоторые из тех, кто подписан на @%{alias} публично.",
       "notify_desktop_error": "Твистер не может выполнить уведомление: произошла неизвестная ошибка.",
       "notify_desktop_perm_denied": "Твистер не может выполнить уведомление: разрешение не получено.\n\nЧтобы получать уведомления, разрешите их для %{this_domain} в настройках вашего браузера.",
       "notify_desktop_test": "Одна лягушка сказала:\n'если не буду квакать — лопну'.\nВы нужны нам, берегите себя.",
       "notify_desktop_title": "Watch out, it's twister over here!",
       "post_preview_dummy": 'это *полужирный*, ~курсивный~, -зачёркнутый- и _подчёркнутый_ текст.\n'
         + 'экранированный: `*полужирный*, ~курсивный~, -зачёркнутый- и _подчёркнутый_`.\n'
         + 'ссылки на [наш потрясный логотип](%{logo}) и наш великолепный сайт: %{site}.',
       "Number of blocks in block chain:": "Количество блоков в цепочке: ",
       "Number of CPUs to use": "Сколько использовать ядер процессора",
       "Only alphanumeric and underscore allowed.": "Разрешены только латинские буквы, цифры и подчеркивания.",
       "peer address": "адрес пира",
       "Private": "Приватный",
       "Profile": "Профиль",
       "Postboard": "Лента",
       "post": "отправить", // verb - button to post a message
       "Post to promote:": "Рекламное сообщение: ",
       "Posts": "Посты",
       "propagating_nickname": "Распространяю информацию о регистрации %{username} в сеть...",
       "Public": "Публичный",
       "Refresh": "Обновить",
       "retransmit_this": "Перепостить данное сообщение своим подписчикам?",
       "Reply": "Ответить",
       "Reply...": "Ответить...",
       "reply_to": "Ответить %{fullname}",
       "Retransmit": "Перепостить",
       "Retransmits": "Репосты",
       "Retransmitted by": "Перепощено ",
       "Switch to Reply": "Переключиться на ответ",
       "Switch to Retransmit": "Переключиться на ретвист",
       "search": "найти",
       "seconds": "%{smart_count} секунда |||| %{smart_count} секунд",
       "send": "отправить",
       "Send post with username": "Отправить сообщение от имени",
       "send_DM": "Отправить личное сообщение",
       "Sent Post to @": "Отправить сообщение для @",
       "Setup account": "Настроить аккаунт",
       "shorten_URI": "Сокр. URL",
       "shorten_URI_enter_link": "Введи длинную ссылку.\n"
        + "Примечание: в результате сокращения URL получается 'пустой' твист, содержащий эту URL.\n"
        + "Этот особый твист не отображается клиентами твистера, но учитывается в числе твоих твистов.",
       "shorten_URI_its_public_is_it_ok": "Ссылка будет общедоступной! Это приемлимо для тебя?",
       "URI_shortener": "Сокращалка URI",
       "The File APIs are not fully supported in this browser.": "File APIs не полностью поддерживается этим браузером.",
       "time_ago": "%{time} назад", // 5 minutes ago
       "Time of the last block:": "Время последнего блока: ",
       "Type message here": "Сообщение...",
       "Unfollow": "Отписаться",
       "Update": "Обновить",
       "Auto updating": "Автоматическое обновление",
      'updates_are_available': 'Доступны обновления',
      'updates_not_available': 'Обновления не доступны',
      'updates_check_client': 'Проверить обновления клиента',
      'updates_repo_overview': 'Мы сейчас на ветке %{branch} в %{repo} на коммите\n  %{commit} от %{date},\n'
        + 'а вот HEAD её источника уже на коммите\n  %{commitUpstream} от %{dateUpstream}.',
      'updates_checkout_diff': 'Смотри [сравнение на GitHub](%{link}), чтобы узнать каковы изменения.',
      'updates_checkout_diff_nfmt': 'Смотри сравнение на GitHub, чтобы узнать каковы изменения:\n  %{link}',
      'updates_upstream_isnt_changed': 'Соответствующая ветка в исходном репозитории выглядит неизменённой.',
       "Updating status...": "Обновление информации...", // status of block chain
      'new_account_briefing': 'Новая учётная запись создана и сейчас распространяется по сети. '
        + 'Пожалуйста, не закрывай это окно: это может занять несколько минут.\n\n'
        + 'Твой секретный ключ: *%{secretKey}*\n\n'
        + 'Настоятельно рекомендуется провести время ожидания, сохраняя этот ключ. '
        + 'Распечатай его, сделай скриншот, сфотографируй телефоном или перепиши на бумагу.\n\n'
        + 'Секретный ключ будет необходим для использования на разных устройствах. '
        + 'В случае его утери учётная запись окажется недосягаемой навсегда '
        + '(заметь: ~это альфа-версия и возможны сбои, приводящие к утере данных~).\n\n'
        + 'Подожди. Кнопка OK станет доступна быстрее, чем ты произнесёшь ~децентрализация~.',
       "user_not_yet_accepted": "Другие участники сети еще не получили информацию о новом пользователе.\n" +
                 "К сожалению, сейчас вы не можете редактировать ваш профиль\n" +
                 "или отправлять сообщение.\n\n" +
                 "Пожалуйста подождите пару минут.\n\n" +
                 "Кнопка 'Сохранить' будет доступна автоматически после того,\n" +
                 "как процес регистрации завершится. (Я обещаю, это\n"+
                 "последний раз, когда вы ждете перед использованием\n" +
                 "twister'a).\n\n" +
                 "Хозяйке на заметку: Сейчас вы можете выбрать аватар!",
       "users_mentions": "Ответ от @%{username}",
       "users_profile": "Профиль @%{username}",
       "username_undefined": "Имя пользователя не определено, требуется войти.",
       "View": "Просмотреть",
       "View All": "Просмотреть Всё",
       "Who to Follow": "Кого почитать",
       "Your message was sent!": "Ваше сообщение было отправлено!",
       "twister login": "Вход в twister",
       "Existing local users": "Уже зарегистрированные",
       "Or...": "Или...",
       "Create a new user": "Зарегистрировать нового пользователя",
       "Login": "Войти",
       "Check availability": "Проверить на доступность",
       "Create this nickname": "Зарегистировать этот ник",
       "Type nickname here": "Введите ваш ник тут",
       "Import secret key": "Импортировать секретный ключ",
       "52-characters secret": "52-значный ключ",
       "With nickname": "С логином",
       "Import key": "Импортировать ключ",
       "Client Version:": "Версия клиента:",
       "Mining difficulty:": "Сложность майнинга:",
       "Block generation status": "Статус майнинга блоков",
       "Current hash rate:": "Текущая скорость хэширования:",
       "Terminate Daemon:": "Остановить twister:",
       "Exit": "Выход",
       "Save Changes": "Сохранить изменения",
       "profile_saved": "Данные профайла сохранены в DHT.",
       "profile_not_saved": "Данные профайла не сохранены.",
       "Secret key:": "Секретный ключ:",
       "You have to log in to post messages.": "Войдите, чтобы отправлять сообщения.",
       "You have to log in to post replies.": "Войдите, чтобы отвечать на сообщения.",
       "You have to log in to retransmit messages.": "Войдите, чтобы репостить сообщения.",
       "You have to log in to use direct messages.": "Войдите, чтобы использовать личные сообщения.",
       "You have to log in to follow users.": "Войдите, чтобы подписываться на пользователей.",
       "You are not following anyone because you are not logged in.": "Войдите, чтобы посмотреть на кого Вы подписаны.",
       "You don't have any followers because you are not logged in.": "Войдите, чтобы посмотреть кто на Вас подписан.",
       "No one can mention you because you are not logged in.": "Войдите, чтобы посмотреть упоминания о Вас.",
       "You don't have any profile because you are not logged in.": "Войдите, чтобы посмотреть профиль.",
       "Options": "Настройки",
       "Switch to Promoted posts": "Отображать только рекламные сообщения",
       "Switch to Normal posts": "Отображать сообщения пользователей",
       "Use language": "Язык",
       "Ignore": "Игнорировать",
       "Ignore and clear out": "Игнорировать и вычищать",
       "Theme": "Тема",
       "Keys": "Клавиши",
       "Notifications": "Уведомления",
       "Desktop notifications": "Уведомления рабочего стола",
       "Sound notifications": "Звуковые уведомления",
       "Volume": "Громкость",
       "Test": "Проверка",
       "Send key": "Отправка сообщения",
       "Posts display": "Отображение сообщений",
       "Post editor": "Редактор сообщения",
       "Post preview": "Предпросмотр сообщений",
       "Inline image preview": "Предпросмотр изображений",
       "Display": "Показывать",
       "Line feeds": "Переходы на новую строку",
       "Markout": "Разметка Markout",
       "Supported punctuations:": "Поддерживаемая пунктуация:",
       "Supported emotions:": "Поддерживаемые смайлы:",
       "Supported signs:": "Поддерживаемые знаки:",
       "Supported fractions:": "Поддерживаемые дроби:",
       "Automatic unicode conversion options": "Настройки автоматического преобразования в unicode",
       "Convert punctuations to unicode": "Преобразовывать пунктуацию в unicode",
       "Convert emotions codes to unicode symbols": "Преобразовывать смайлы в символы unicode",
       "Convert common signs to unicode": "Преобразовывать популярные знаки в unicode",
       "Convert fractions to unicode": "Преобразовывать дроби в unicode",
       "Convert all": "Преобразовывать все",
       "Auto": "Автоматически",
       "Original": "По умолчанию",
       "none": "нет",
       "Custom": "Выборочно",
       "Mentions": "Упоминания",
       "Use proxy for image preview only": "Использовать proxy только для просмотра изображений",
       "Use external links behind a proxy": "Использовать внешние ссылки через proxy",
       "There aren't any posts with this hashtag.": "Нет сообщений с этим хэштегом.",
       "Split only new post": "Разбивать только новые сообщения",
       "Split all": "Разбивать все",
       "Don't split": "Не разбивать",
       "Split long posts": "Разбивать длинные сообщения",
       "Posts that begin with mention": "Сообщения, которые начинаются с упоминания",
       "Show all": "Показывать все",
       "Show only if I am in": "Показывать только если я в адресатах",
       "Show if it's between users I follow": "Показывать если между пользователями, на которых я подписан",
       "Postboard displays": "Отображение ленты",
       "RTs those are close to original twist": "Перепосты сразу после исходного сообщения",
       "Show if the original is older than": "Показывать если исходное сообщение старше",
       "hour(s)": "час(ов)",
       "second(s)": "секунд(а,ы)",
       "only positive numbers!": "разрешены только положительные числа!",
       "Language filtering": "Ограничение языковым фильтром",
       "By blacklist": "С помощью чёрного списка",
       "By whitelist": "С помощью белого списка",
       "Comma separated ISO 639-3 language codes": "Соответствующие ISO 639-3 языковые коды, разделённые запятыми",
       "Accuracy": "Точность",
       "Simulation mode": "Режим симуляции",
       "This post is treated by language filter": "Этот пост %{treated} языковым фильтром.",
       "blocked": "запрещён",
       "passed": "разрешён",
       "not analyzed": "не обработан",
       "Reason: this": "Причина: %{this}",
       "this doesnt contain that": "%{this} не содержит %{that}",
       "this is undefined": "%{this} не определён",
       "blacklist": "чёрный список",
       "whitelist": "белый список",
       "language of this": "такой язык",
       "its undefined language": "язык не определён",
       "its this, blacklisted": "это %{this}, есть в чёрном списке",
       "its this, whitelisted": "это %{this}, есть в белом списке",
       "Most possible language: this": "Наиболее вероятный язык: %{this}",
       "Scope of usage": "Область применения",
       "Show with every user name": "Показывать везде около имени пользователя",
       "Show at profile modal only": "Показывать только в окне профиля",
       "Show if a user follows me": "Показывать подписан ли пользователь на меня",
       "follows you": "ваш подписчик",
       "Show conversation": "Показать беседу",
       "Mark all as read": "Отметить все как прочитанные",
       "show_more_count": "ещё %{smart_count}...",
       "hide": "скрыть",
       "Show more in this conversation...": "Показать дальше в этой беседе...",
       "conversation_title": "Беседа @%{username}",
       "copy_to_clipboard": "Нажмите Ctrl/Cmd+C для копирования и Enter для закрытия",
       "Normal posts": "Сообщения пользователей",
       "Promoted posts": "Рекламные сообщения",
      'promoted': 'Продвигаемый',
       "Messages": "Сообщения",
       "Edit profile": "Изменить профиль",
       "Top Trends": "Топ Трендов",
       "Twistday Reminder": "Твистоденный Календарь",
       "Show upcoming in near future": "Показывать намечающиеся юбиляров",
       "Who's celebrating Twistday": "Кто празднует Твистодень",
       "Today's luckies:": "Сегодняшные счастливчики", // няшные, ок?
       "Upcoming ones:": "Намечающиеся вскоре",
       "post_rt_sign_prep": "ретвистит",
       "post_rt_time_prep": "в",
       "undo": "отменить",
       "Daemon exited...": "Демон twister выключен...",
       "Secret Key": "Секретный ключ",
       "Copy to clipboard": "Копировать в буфер обмена",
       "Full name here": "Укажите Ваше имя",
       "Describe yourself": "Расскажите о себе",
       "Location": "Где вы живёте",
       "website": "Веб-сайт",
       "Tox address": "Tox адрес",
       "Bitmessage address": "Bitmessage адрес",
       "Language": "Язык",
       "Sound": "Звук",
       "Users": "Пользователи",
      'dm_copy_outgoing_to_self': 'Синхронизировать отправленные личные сообщения',
      'dm_encrypt_local_cache': 'Шифровать локальный кэш данных',
       "Terminate Daemon": "Выключить twister демон",
       "New post": "Новый пост",
       "Search": "Поиск",
       "Direct Msg": "ЛС",
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "de"){
    polyglot.locale("de");
    wordset = {
      "Actions ▼": "Actions ▼",
      "Active DHT nodes:": "Aktive DHT-Nodes: ",
      "Add DNS": "DNS hinzufügen",
      "Add peer": "Peer hinzufügen",
      "ajax_error": "Ajax error: %{error}", // JavaScript error
      "All users publicly followed by": "Alle öffentlich gefolgten Benutzer von",
      "Available": "Verfügbar", // username is available
      "Appearance": "Aussehen",
      "Apply": "Apply",
      "Block chain information": "Block-Chain Informationen",
      "Block chain is up-to-date, twister is ready to use!": "Block-Chain ist aktuell, twister ist benutzbar!",
      "Block generation": "Block-Generierung ",
      "busted_oh": "Oh nein!",
      "busted_avowal": "Ich bin ertappt, habe versucht, hier Schadcode einzugeben!",
      "btn_ok": "Okay",
      "Cancel": "Abbrechen",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Lokaler daemon ist nicht mit dem Netzwerk verbunden oder Block-Chain ist veraltet.\n" +
            "Wenn du auf dieser Seite bleibst können deine Handlungen nicht funktionieren.\n" +
            "Möchtest du stattdessen den [Netzwerkstatus](%{page}) überprüfen?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Benutzer wechseln",
      "Checking...": "Überprüfe...", // checking if username is available
      "Collapse": "Einklappen", // smaller view of a post
      "Configure block generation": "Block-Generierung einstellen",
      "Connections:": "Verbindungen: ", // to network
      "Connection lost.": "Verbindung verloren.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} Tag |||| %{smart_count} Tage",
      "Detailed information": "Detaillierte Informationen",
      "DHT network down.": "DHT-Netzwerk nicht verfügbar.",
      "Direct Messages": "Direktnachrichten",
      "Group Messages": "Gruppennachrichten",
      "Group Messages — New Group Creation": "Gruppennachrichten — Neue Gruppe erstellen",
      "Group Messages — Join Group": "Gruppennachrichten — Gruppe beitreten",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Direktnachrichten mit %{username}",
      "Disable": "Deaktivieren",
      "display_mentions": "Zeige Erwähnungen", // Ist das richtig? Ich weiß nicht, in welchem Zusammenhang das benutzt wird.
      "Display retransmissions": "Weiterleitungen anzeigen",
      "DNS to obtain list of peers:": "DNS um Peer-Liste abzurufen:",
      "do_not_show_it_again": "don't show it again",
      "dns address": "DNS-Adresse",
      "downloading_block_chain": "Block-Chain wird heruntergeladen, bitte warten (Block-Chain ist %{days} Tage alt).",
      "download_posts_status": "%{portion} Posts heruntergeladen", // Downloaded 10/30 posts
      "Enable": "Aktivieren",
      "error": "Fehler: %{error}",
      "error_connecting_to_daemon": "Fehler beim Verbinden zum lokalen twister-daemon.",
      "Error in 'createwalletuser' RPC.": "Fehler in 'createwalletuser' RPC.",
      "Error in 'importprivkey'": "Fehler in 'importprivkey' RPC: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Fehler in 'sendnewusertransaction' RPC.",
      "Expand": "Ausklappen", // larger view of a post
      "Favorite": "Favorisieren",
      "File APIs not supported in this browser.": "File APIs werden von diesem Browser nicht unterstützt.",
      "Follow": "Folgen",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Gefolgt von",
      "followed_by": "Gefolgt von %{username}",
      "Followers": "Followers",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Folgt",
      "Following users": "Folgt Benutzern",
      "Force connection to peer:": "Erzwinge Verbindung zu Peer:",
      "General information": "Generelle Informationen",
      "Generate blocks (send promoted messages)": "Blöcke generieren (Promoted-Messages senden)",
      "Home": "Home", // homepage
      "hours": "%{smart_count} Stunde |||| %{smart_count} Stunden",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Internen Fehler: lastPostId unbekannt (Dir selbst volgen kann den Fehler beheben!)",
      "Known peers:": "Bekannte Peers: ",
      "Last block is ahead of your computer time, check your clock.": "Letzter Block ist deiner Computerzeit voraus, überprüfe deine Uhrzeit.",
      "mentions_at": "Erwähnt @%{user}",
      "minutes": "%{smart_count} Minute |||| %{smart_count} Minuten",
      "Must be 16 characters or less.": "Darf höchstens 16 Zeichen lang sein.", // username
      "Network": "Netzwerk",
      "Network config": "Netzwerkeinstellungen",
      "Network status": "Netzwerkstatus",
      "New direct message...": "Neue Direktnachricht...",
      "New Post...": "Neuer Post...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "Du hast",
      "in postboard": "in der Timeline",
      "in search result": "in den Suchergebnissen",
      "in top trends": "in den Top Trends",
      "new_posts": "%{smart_count} neuer Post |||| %{smart_count} neue Posts", /* FIXME: duplicate */
      "new_mentions": "%{smart_count} neue Erwähnung |||| %{smart_count} neue Erwähnungen",
      "new_direct_messages": "%{smart_count} neue Direktnachricht |||| %{smart_count} neue Direktnachrichten",
      "new_group_messages": "%{smart_count} neue Gruppen-Nachricht |||| %{smart_count} neue Gruppen-Nachrichten",
      "nobody": "nobody", // used to promote a post without attaching the user
      "Not available": "Nicht verfügbar", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister kann keine Desktop-Meldungen anzeigen: ein unbekannter Fehler trat auf.",
      "notify_desktop_perm_denied": "Twister kann keine Desktop-Meldungen anzeigen: Keine Berechtigung.\n\nWenn Du Meldungen angezeigt haben möchtest, erlaube sie für %{this_domain} in den Einstellungen Deines Browsers.",
      "notify_desktop_test": "All die Twisterer werden twisten..\nDu bist nun auch willkommen!",
      "notify_desktop_title": "Achtung, hier ist Twister am Gange!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Anzahl der Blöcke in der Block-Chain: ",
      "Number of CPUs to use": "Anzahl der zu benutzenden CPU's ",
      "Only alphanumeric and underscore allowed.": "Nur Buchstaben, Zahlen und Unterstrich erlaubt.",
      "peer address": "Peer-Adresse",
      "Private": "Privat",
      "Profile": "Profil",
      "Postboard": "Postboard",
      "post": "senden", // verb - button to post a message
      "Post to promote:": "Post zum senden: ",
      "Posts": "Posts",
      "propagating_nickname": "Mache nickname %{username} dem Netzwerk bekannt...",
      "Public": "Öffentlich",
      "Refresh": "Erneuern",
      "retransmit_this": "Diesen Post an deine Follower weiterleiten?",
      "Reply": "Antworten",
      "Reply...": "Antworten...",
      "reply_to": "%{fullname} antworten",
      "Retransmit": "Weiterleiten",
      "Retransmits": "Weiterleitungen",
      "Retransmitted by": "Weitergeleitet von",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "suchen",
      "seconds": "%{smart_count} Sekunde |||| %{smart_count} Sekunden",
      "send": "senden",
      "Send post with username": "Sende Post mit Benutzernamen ",
      "send_DM": "Direktnachricht senden",
      "Sent Post to @": "Sende Post an @",
      "Setup account": "Accounteinstellungen",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "Die File-API's werden von diesem Browser nicht vollständig unterstützt.",
      "time_ago": "vor %{time}", // 5 minutes ago
      "Time of the last block:": "Zeit des letzten Blocks: ",
      "Type message here": "Nachricht hier eingeben",
      "Unfollow": "Nicht mehr folgen",
      "Update": "Aktualisieren",
      "Auto updating": "Automatische Aktualisierung",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Status wird aktualisiert...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Andere Peers haben diesen Benutzter noch nicht akzeptiert.\n" +
                "Leider ist es nicht möglich, das Profil zu speichern\n" +
                "oder Nachrichten zu senden.\n\n" +
                "Bitte warten ein paar Minuten, um fortzufahen.\n\n" +
                "'Änderungen speichern' wird automatisch aktiviert\n" +
                "wenn der Prozess abgeschlossen ist. (Ich verspreche,\n"+
                "das ist das letzte Mal, dass du warten musst, bevor\n" +
                "du twister benutzten kannst).\n\n" +
                "Tip: Wähle in der Zwischenzeit deinen Avatar aus!",
      "users_mentions": "Erwähnungen von @%{username}",
      "users_profile": "%{username}'s Profil",
      "username_undefined": "Benutzername nicht gesetzt, Login nötig.",
      "View": "Ansehen",
      "View All": "Alle ansehen",
      "Who to Follow": "Wem Folgen?",
      "Your message was sent!": "Deine Nachricht wurde gesendet!",
      "twister login": "twister login",
      "Existing local users": "Existierende lokale Benutzer",
      "Or...": "Oder...",
      "Create a new user": "Neuen Benutzer erstellen",
      "Login": "Einloggen",
      "Check availability": "Verfügbarkeit überprüfen",
      "Create this nickname": "Nicknamen erstellen",
      "Type nickname here": "Nicknamen hier eingeben",
      "Import secret key": "Privaten Schlüssel hier importieren",
      "52-characters secret": "52 Zeichen Geheimnis",
      "With nickname": "Mit Nicknamen",
      "Import key": "Schlüssel importieren",
      "Client Version:": "Client Version:",
      "Mining difficulty:": "Mining Schwierigkeit:",
      "Block generation status": "Status der Block-Generierung",
      "Current hash rate:": "Aktuelle Hash-Rate:",
      "Terminate Daemon:": "Daemon beenden:",
      "Exit": "Beenden",
      "Save Changes": "Änderungen speichern",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Privater Schlüssel:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Einstellungen",
      "Switch to Promoted posts": "Wechsle zu Promoted Nachrichten",
      "Switch to Normal posts": "Switch to Normal posts",
      "new_posts": "%{smart_count} neue Nachricht |||| %{smart_count} neue Nachrichten", /* FIXME: duplicate */
      "Use language": "Sprache einstellen",
      "Ignore": "Ignorieren",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Benachrichtigungen",
      "Desktop notifications": "Desktophinweise",
      "Sound notifications": "Audioeinstellungen",
      "Volume": "Lautstärke",
      "Test": "Probe",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Nachrichten-Editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatische Unicode Konvertierungseinstellungen",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Alle konvertieren",
      "Auto": "Auto",
      "Original": "Original",
      "none": "kein(e)",
      "Custom": "Benutzerdefiniert",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Benutze Proxy nur für Bildvorschau",
      "Use external links behind a proxy": "Benutze externe Links hinter einem Proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Teile nur neue Nachrichten auf",
      "Split all": "Alle aufteilen",
      "Don't split": "Nicht aufteilen",
      "Split long posts": "Teile längere Nachrichten auf",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "In der Timeline anzeigen",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Anzeigen, wenn das Original älter ist als",
      "hour(s)": "Stunde(n)",
      "second(s)": "Sekunde(n)",
      "only positive numbers!": "nur positive Zahlen erlaubt!",
      "Language filtering": "Sprachfilterung",
      "By blacklist": "Mittels schwarzer Liste",
      "By whitelist": "Mittels weißer Liste",
      "Comma separated ISO 639-3 language codes": "Komma getrennte ISO 639-3 Sprachcodes",
      "Accuracy": "Genauigkeit",
      "Simulation mode": "Simulationsmodus",
      "This post is treated by language filter": "Diese Nachricht ist mittels Sprachfilter %{treated}",
      "blocked": "blockiert",
      "passed": "weitergeleitet",
      "not analyzed": "nicht anylysiert",
      "Reason: this": "Ursache: %{this}",
      "this doesnt contain that": "%{this} dies enthält %{that} nicht",
      "this is undefined": "%{this} ist nicht definiert",
      "blacklist": "schwarze Liste",
      "whitelist": "weiße Liste",
      "language of this": "Die Sprache des Folgenden",
      "its undefined language": "es handelt sich um eine undefinierte Sprache",
      "its this, blacklisted": "es ist %{this}, aus der schwarzen Liste",
      "its this, whitelisted": "es ist %{this}, aus der weißen Liste",
      "Most possible language: this": "Die wahrscheinlichste Sprache: %{this}",
      "Scope of usage": "Anwendungsbereich",
      "Show with every user name": "Zeige mit jedem Benutzernamen",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Zeige an, wenn mir ein Benutzer folgt",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Nachrichten",
      "Edit profile": "Profil bearbeiten",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Erinnerung",
      "Show upcoming in near future": "Zeige bevorstehende in naher Zukunft",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Die Glücklichen von heute:",
      "Upcoming ones:": "Demnächst:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Secret Key",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Kopie der Direktnachricht an mich selbst',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "ja"){
    polyglot.locale("ja");
    wordset = {
      "Actions ▼": "アクション ▼",
      "Active DHT nodes:": "アクティブなDHTノード: ",
      "Add DNS": "DNSを追加",
      "Add peer": "ピアを追加",
      "ajax_error": "AJAXエラー: %{error}", // JavaScript error
      "All users publicly followed by": "このユーザがフォローを公開している",
      "Available": "Available", // username is available
      "Appearance": "Appearance",
      "Apply": "Apply",
      "Block chain information": "ブロックチェイン",
      "Block chain is up-to-date, twister is ready to use!": "ブロックチェインは最新の状態です。",
      "Block generation": "ブロック生成",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "キャンセル",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "ローカルデーモンが切断しているか、ブロックチェーンが最新状態ではない。\n" +
            "このページを使用しつづければ、あなたの変更が適用されないおそれがあります。\n" +
            "[ネットワーク状態ペー](%{page})ジへ移動して確認しますか？",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "ユーザーを変更",
      "Checking...": "チェック...", // checking if username is available
      "Collapse": "閉じる", // smaller view of a post
      "Configure block generation": "ブロック生成の設定",
      "Connections:": "接続数: ", // to network
      "Connection lost.": "切断されました。",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count}日 |||| %{smart_count}日",
      "Detailed information": "詳細",
      "DHT network down.": "DHTネットワークがダウンしています",
      "Direct Messages": "ダイレクトメッセージ",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Direct messages with %{username}",
      "Disable": "停止",
      "display_mentions": "メンションを表示する",
      "Display retransmissions": "リトランスミットを表示する",
      "DNS to obtain list of peers:": "ピア取得用のDNS:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "ブロックチェインをダウンロードしています。しばらくお待ちください。（ブロックチェーンは%{days}日送れています）",
      "download_posts_status": "ダウンロード済みの投稿 %{portion}", // Downloaded 10/30 posts
      "Enable": "有効",
      "error": "Error: %{error}",
      "error_connecting_to_daemon": "ロカールtwisterデーモンに接続したが、エラーが発生しました。",
      "Error in 'createwalletuser' RPC.": "'createwalletuser' RPCにエラー発生",
      "Error in 'importprivkey'": "'importprivkey' RPC: %{rpc}にエラー発生",
      "Error in 'sendnewusertransaction' RPC.": "'sendnewusertransaction' RPCにエラー発生",
      "Expand": "開く", // larger view of a post
      "Favorite": "お気に入り",
      "File APIs not supported in this browser.": "利用しているブラウザはファイルAPIをサポートしていません。",
      "Follow": "フォロー",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "フォローされている",
      "followed_by": "%{username}にフォローされている",
      "Followers": "フォロワー",
      "Followers_of": "Followers of @%{alias}",
      "Following": "フォロー",
      "Following users": "フォローしているユーザー",
      "Force connection to peer:": "強制的に経由するピア:",
      "General information": "概略",
      "Generate blocks (send promoted messages)": "ブロックチェインを生成する (プロモートメッセージを送信する)",
      "Home": "ホーム", // homepage
      "hours": "%{smart_count}時間 |||| %{smart_count}時間",
      "Internal error: lastPostId unknown (following yourself may fix!)": "内部エラー: lastPostId不明（自分をフォローすれば解決するかもしれません）",
      "Known peers:": "ネットワーク全体のピア: ",
      "Last block is ahead of your computer time, check your clock.": "最新ブロックはあなたのコンピューター時間より進んでいます。日付設定を確認してください。",
      "mentions_at": "@%{user}へのメンション",
      "minutes": "%{smart_count}分 |||| %{smart_count}分",
      "Must be 16 characters or less.": "16字以下を入力してください。", // username
      "Network": "ネットワーク",
      "Network config": "ネットワーク設定",
      "Network status": "ネットワークステータス",
      "New direct message...": "ダイレクトメッセージ...",
      "New Post...": "投稿する",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "You got",
      "in postboard": "in postboard",
      "in search result": "in search result",
      "in top trends": "in top trends",
      "new_posts": "%{smart_count} 新投稿 |||| %{smart_count} 新投稿",
      "new_mentions": "%{smart_count} new mention |||| %{smart_count} new mentions",
      "new_direct_messages": "%{smart_count} new direct message |||| %{smart_count} new direct messages",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "ナナシ", // used to promote a post without attaching the user
      "Not available": "使用中", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister cannot perform desktop notification: unknown error occured.",
      "notify_desktop_perm_denied": "Twister cannot perform desktop notification: permission denied.\n\nIf you want to get notifications, allow them for %{this_domain} in settings of your browser.",
      "notify_desktop_test": "All the twisters gonna twist.\nNow you are welcome too.",
      "notify_desktop_title": "Watch out, it's twister over here!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "全ブロック数: ",
      "Number of CPUs to use": "利用するCPUの数",
      "Only alphanumeric and underscore allowed.": "アルファベットとアンダースコア(_)が利用可能です",
      "peer address": "ピア・アドレス",
      "Private": "プライベート",
      "Profile": "プロフィール",
      "Postboard": "投稿ボード",
      "post": "投稿する", // verb - button to post a message
      "Post to promote:": "プロモートメッセージ: ",
      "Posts": "投稿",
      "propagating_nickname": "ユーザネーム%{username}をネットワークに配信しています…",
      "Public": "公開",
      "Refresh": "リフレッシュ",
      "retransmit_this": "この投稿をリトランスミットしますか？",
      "Reply": "返信",
      "Reply...": "返信...",
      "reply_to": "%{fullname}への返信",
      "Retransmit": "リトランスミット",
      "Retransmits": "リトランスミット",
      "Retransmitted by": "リトランスミットしたユーザー",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "検索",
      "seconds": "%{smart_count} 秒 |||| %{smart_count} 秒",
      "send": "送信",
      "Send post with username": "プロモートメッセージの送信元",
      "send_DM": "ダイレクトメッセージを送る",
      "Sent Post to @": "メンションを投稿する",
      "Setup account": "アカウント設定",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "ご使用のブラウザーは完全にファイルAPIに対応していません。",
      "time_ago": "%{time}前", // 5 minutes ago
      "Time of the last block:": "最新ブロックの生成日時: ",
      "Type message here": "ここにメッセージを書いてね",
      "Unfollow": "解除",
      "Update": "アップデート",
      "Auto updating": "Auto updating",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "ブロックチェーンを更新中…", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "この新ユーザーが他のピアーにまだ認証されていない。\n" +
                "なので、プロファイルの保存やメッセージの投稿は現在不可能です。\n" +
                "しばらくしてから続行してください。\n\n" +
                "処理が終了したら、「保存」ボタンは有効化されます。\n" +
                "twisterを使用するあいだ、待たされることはこれが最後です。\n\n"+
                "待っている間、アバターを選択したら？",
      "users_mentions": "@%{username}へのメンション",
      "users_profile": "%{username}のプロフィール",
      "username_undefined": "ログインしてください。",
      "View": "見る",
      "View All": "すべて見る",
      "Who to Follow": "おすすめユーザー",
      "Your message was sent!": "投稿が送信されました！",
      "twister login": "Twisterログイン",
      "Existing local users": "既存のローカルユーザー",
      "Or...": "もしくは...",
      "Create a new user": "新規ユーザーを作成",
      "Login": "ログイン",
      "Check availability": "取得可能かチェック",
      "Create this nickname": "作成",
      "Type nickname here": "ニックネーム",
      "Import secret key": "シークレットキーをインポートする",
      "52-characters secret": "52文字からなるシークレット",
      "With nickname": "このニックネームで",
      "Import key": "キーをインポートする",
      "Client Version:": "クライアントのヴァージョン:",
      "Mining difficulty:": "採掘の難しさ:",
      "Block generation status": "ブロック生成の状態",
      "Current hash rate:": "現在のハッシュレート:",
      "Terminate Daemon:": "デーモンを終了する:",
      "Exit": "終了",
      "Save Changes": "変更を保存",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "シークレットキー:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Options",
      "Switch to Promoted posts": "Switch to Promoted posts",
      "Switch to Normal posts": "Switch to Normal posts",
      "Use language": "Use language",
      "Ignore": "Ignore",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Notifications",
      "Desktop notifications": "Desktop notifications",
      "Sound notifications": "Sound notifications",
      "Volume": "Volume",
      "Test": "Test",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convert all",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Use proxy for image preview only",
      "Use external links behind a proxy": "Use external links behind a proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hour(s)",
      "second(s)": "second(s)",
      "only positive numbers!": "only positive numbers are allowed!",
      "Language filtering": "Language filtering",
      "By blacklist": "By blacklist",
      "By whitelist": "By whitelist",
      "Comma separated ISO 639-3 language codes": "Comma separated ISO 639-3 language codes",
      "Accuracy": "Accuracy",
      "Simulation mode": "Simulation mode",
      "This post is treated by language filter": "This post is %{treated} by language filter.",
      "blocked": "blocked",
      "passed": "passed",
      "not analyzed": "not analyzed",
      "Reason: this": "Reason: %{this}",
      "this doesnt contain that": "%{this} does not contain %{that}",
      "this is undefined": "%{this} is undefined",
      "blacklist": "blacklist",
      "whitelist": "whitelist",
      "language of this": "language of this",
      "its undefined language": "it is undefined language",
      "its this, blacklisted": "it is %{this}, blacklisted",
      "its this, whitelisted": "it is %{this}, whitelisted",
      "Most possible language: this": "Most possible language: %{this}",
      "Scope of usage": "Scope of usage",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Messages",
      "Edit profile": "Edit profile",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Secret Key": "Secret Key",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

// Brazilian Portuguese translation
if(preferredLanguage == "pt-BR"){
    polyglot.locale("pt-BR");
    wordset = {
      "Actions ▼": "Ações ▼",
      "Active DHT nodes:": "Nós DHT ativos: ",
      "Add DNS": "Adicionar DNS",
      "Add peer": "Adicionar nó",
      "ajax_error": "Erro Ajax: %{error}", // JavaScript error
      "All users publicly followed by": "Todos os usuários seguidos publicamente por",
      "Available": "Disponível", // username is available
      "Appearance": "Appearance",
      "Apply": "Apply",
      "Block chain information": "Informações da Cadeia de Blocos",
      "Block chain is up-to-date, twister is ready to use!": "A Cadeida de Blocos está sincronizada, Twister está pronto para uso!",
      "Block generation": "Geração de blocos ",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Cancelar",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "O Cliente do Twister não está conectado à rede ou a cadeia de blocos está desatualizada.\n" +
            "Se permanecer nesta página suas ações podem não ter efeito.\n" +
            "Gostaria de verificar o [Estado da Rede](%{page}) ao invés disso?",
      "confirm_terminate_daemon": "Are you sure you want to exit the daemon?\nThe Twister client will stop working.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Trocar usuário",
      "Checking...": "Verificando...", // checking if username is available
      "Collapse": "Recolher", // smaller view of a post
      "Configure block generation": "Configurar a geração de blocos",
      "Connections:": "Conexões: ", // to network
      "Connection lost.": "Conexão perdida.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} dia |||| %{smart_count} dias",
      "Detailed information": "Informações detalhadas",
      "DHT network down.": "Falha na rede DHT",
      "Direct Messages": "Mensagens Diretas",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Mensagens Diretas com %{username}",
      "Disable": "Desabilitado",
      "display_mentions": "Exibir menções",
      "Display retransmissions": "Exibir retransmissões",
      "DNS to obtain list of peers:": "DNS para obter a lista de nós:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Baixando a Cadeia de Blocos, por favor aguarde (A Cadeia de Blocos está %{days} dias desatualizada).",
      "download_posts_status": "%{portion} postagens carregadas.", // Downloaded 10/30 posts
      "Enable": "Habilitado",
      "error": "Erro: %{error}",
      "error_connecting_to_daemon": "Erro ao tentar se conectar com o Cliente do Twister.",
      "Error in 'createwalletuser' RPC.": "Erro ao executar a RPC 'createwalletuser'.",
      "Error in 'importprivkey'": "Erro ao executar a RPC 'importprivkey' : %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Erro ao executar a RPC 'sendnewusertransaction'",
      "Expand": "Expandir", // larger view of a post
      "Favorite": "Favorito",
      "File APIs not supported in this browser.": "O gerenciamento de arquivos não é suportado neste navegador.",
      "Follow": "Seguir",
      "Following config": "Following config",
      "select_way_to_follow_@": "Which way do you want to follow @%{alias}",
      "Followed by": "Seguido por",
      "followed_by": "Seguido por %{username}",
      "Followers": "Seguidores",
      "Followers_of": "Seguidores de @%{alias}",
      "Following": "Seguindo",
      "Following users": "Usuários que sigo",
      "Force connection to peer:": "Forçar conexão com o nó:",
      "General information": "Informações gerais",
      "Generate blocks (send promoted messages)": "Gerar blocos (enviar mensagens promovidas)",
      "Home": "Início", // homepage
      "hours": "%{smart_count} hora |||| %{smart_count} horas",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Erro interno: lastPostId desconhecido (seguir você mesmo pode corrigir este erro!)",
      "Known peers:": "Nós conhecidos: ",
      "Last block is ahead of your computer time, check your clock.": "O último bloco está adiantado em relação ao horário do seu computador, verifique seu relógio.",
      "mentions_at": "Menções @%{user}",
      "minutes": "%{smart_count} minuto |||| %{smart_count} minutos",
      "Must be 16 characters or less.": "Deve conter 16 caracteres ou menos.", // username
      "Network": "Rede",
      "Network config": "Configuração de rede",
      "Network status": "Estado da rede",
      "New direct message...": "Nova mensagem direta...",
      "New Post...": "Nova Postagem...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "You got",
      "in postboard": "in postboard",
      "in search result": "in search result",
      "in top trends": "in top trends",
      "new_posts": "%{smart_count} nova postagem |||| %{smart_count} novas postagens",
      "new_mentions": "%{smart_count} new mention |||| %{smart_count} new mentions",
      "new_direct_messages": "%{smart_count} new direct message |||| %{smart_count} new direct messages",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "nobody", // used to promote a post without attaching the user
      "Not available": "Indisponível", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twister cannot perform desktop notification: unknown error occured.",
      "notify_desktop_perm_denied": "Twister cannot perform desktop notification: permission denied.\n\nIf you want to get notifications, allow them for %{this_domain} in settings of your browser.",
      "notify_desktop_test": "All the twisters gonna twist.\nNow you are welcome too.",
      "notify_desktop_title": "Watch out, it's twister over here!",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Número de blocos: ",
      "Number of CPUs to use": "Número de CPUs a serem utilizados ",
      "Only alphanumeric and underscore allowed.": "Permitido apenas caracteres alfanuméricos e underscore '_'.",
      "peer address": "endereço do nó",
      "dns address": "endereço do DNS", // não existia
      "Private": "Privado",
      "Profile": "Perfil",
      "Postboard": "Postagens",
      "post": "Postar", // verb - button to post a message
      "Post to promote:": "Mensagem a ser promovida: ",
      "Posts": "Postagens",
      "propagating_nickname": "Propagando o nome de usuário %{username} para a rede...",
      "Public": "Público",
      "Refresh": "Atualizar",
      "retransmit_this": "Retransmitir esta postagem para seus seguidores?",
      "Reply": "Responder",
      "Reply...": "Responder...",
      "reply_to": "Responder à %{fullname}",
      "Retransmit": "Retransmitir",
      "Retransmits": "Retransmissões",
      "Retransmitted by": "Retransmitido por",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "procurar",
      "seconds": "%{smart_count} segundo |||| %{smart_count} segundos",
      "send": "enviar",
      "Send post with username": "Promover esta mensagem como usuário",
      "send_DM": "Mensagens Diretas trocadas",
      "Sent Post to @": "Postagens enviadas para @",
      "Setup account": "Configurar conta",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "O gerenciamento de arquivos não é completamente suportado neste navegador.",
      "time_ago": "%{time} atrás", // 5 minutes ago
      "Time of the last block:": "Horário do último bloco: ",
      "Type message here": "Escreva sua mensagem aqui",
      "Unfollow": "Deixar de seguir",
      "Update": "Atualizar",
      "Auto updating": "Auto updating",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Atualizando estado da Cadeia de Blocos...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Outros nós ainda não aceitaram este novo usuário.\n" +
                "Infelizmente não é possível salvar o perfil\n" +
                "ou realizar postagens neste estado.\n\n" +
                "Por favor, aguarde alguns minutos para continuar.\n\n" +
                "O botão 'Salvar Alterações' será habilitado automaticamente\n" +
                "qundo o processo for completado (Eu prometo que\n"+
                "está será a última vez que você terá que esperar\n" +
                "antes de utilizar o Twister).\n\n" +
                "Dica: escolha uma foto para o seu avatar enquanto espera!",
      "users_mentions": "Menções de @%{username}",
      "users_profile": "Perfil de %{username}",
      "username_undefined": "Nome de usuário indefinido, login requerido.",
      "View": "Visualizar",
      "View All": "Visualizar Todos",
      "Who to Follow": "Quem seguir",
      "Your message was sent!": "Sua mensagem foi enviada!",
      "twister login": "Login no Twister",
      "Existing local users": "Usuários recentes",
      "Or...": "Ou...",
      "Create a new user": "Crie um novo usuário",
      "Login": "Login",
      "Check availability": "Verificar disponibilidade",
      "Create this nickname": "Criar este usuário",
      "Type nickname here": "Digite o nome aqui",
      "Import secret key": "Importe uma chave secreta",
      "52-characters secret": "Digite aqui os 52 caracteres da chave",
      "With nickname": "Com o nome de usuário",
      "Import key": "Importar chave",
      "Client Version:": "Versão do Cliente:",
      "Mining difficulty:": "Dificulade de mineração:",
      "Block generation status": "Estado da geração de blocos",
      "Current hash rate:": "Taxa de 'hash' atual:",
      "Terminate Daemon:": "Encerrar a execução do Cliente do Twister",
      "Exit": "Sair",
      "Save Changes": "Salvar Alterações",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret Key": "Chave Secreta", // não existia
      "Secret key:": "Chave secreta:",
      "You have to log in to post messages.": "You have to log in to post messages.",
      "You have to log in to post replies.": "You have to log in to post replies.",
      "You have to log in to retransmit messages.": "You have to log in to retransmit messages.",
      "You have to log in to use direct messages.": "You have to log in to use direct messages.",
      "You have to log in to follow users.": "You have to log in to follow users.",
      "You are not following anyone because you are not logged in.": "You are not following anyone because you are not logged in.",
      "You don't have any followers because you are not logged in.": "You don't have any followers because you are not logged in.",
      "No one can mention you because you are not logged in.": "No one can mention you because you are not logged in.",
      "You don't have any profile because you are not logged in.": "You don't have any profile because you are not logged in.",
      "Options": "Options",
      "Switch to Promoted posts": "Switch to Promoted posts",
      "Switch to Normal posts": "Switch to Normal posts",
      "Use language": "Use language",
      "Ignore": "Ignore",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Theme",
      "Keys": "Keys",
      "Notifications": "Notificações",
      "Desktop notifications": "Notificações no desktop",
      "Sound notifications": "Notificações sonoras",
      "Volume": "Volume",
      "Test": "Teste",
      "Send key": "Send key",
      "Posts display": "Posts display",
      "Post editor": "Post editor",
      "Post preview": "Post preview",
      "Inline image preview": "Inline image preview",
      "Display": "Display",
      "Line feeds": "Line feeds",
      "Markout": "Markout markup",
      "Supported punctuations:": "Supported punctuations:",
      "Supported emotions:": "Supported emotions:",
      "Supported signs:": "Supported signs:",
      "Supported fractions:": "Supported fractions:",
      "Automatic unicode conversion options": "Automatic unicode conversion options",
      "Convert punctuations to unicode": "Convert punctuations to unicode",
      "Convert emotions codes to unicode symbols": "Convert emotions codes to unicode symbols",
      "Convert common signs to unicode": "Convert common signs to unicode",
      "Convert fractions to unicode": "Convert fractions to unicode",
      "Convert all": "Convert all",
      "Auto": "Auto",
      "Original": "Original",
      "none": "none",
      "Custom": "Custom",
      "Mentions": "Mentions",
      "Use proxy for image preview only": "Use proxy for image preview only",
      "Use external links behind a proxy": "Use external links behind a proxy",
      "There aren't any posts with this hashtag.": "There aren't any posts with this hashtag.",
      "Split only new post": "Split only new post",
      "Split all": "Split all",
      "Don't split": "Don't split",
      "Split long posts": "Split long posts",
      "Posts that begin with mention": "Posts that begin with mention",
      "Show all": "Show all",
      "Show only if I am in": "Show only if I am in",
      "Show if it's between users I follow": "Show if it's between users I follow",
      "Postboard displays": "Postboard displays",
      "RTs those are close to original twist": "RTs those are close to original twist",
      "Show if the original is older than": "Show if the original is older than",
      "hour(s)": "hour(s)",
      "second(s)": "segundo(s)",
      "only positive numbers!": "only positive numbers are allowed!",
      "Language filtering": "Language filtering",
      "By blacklist": "By blacklist",
      "By whitelist": "By whitelist",
      "Comma separated ISO 639-3 language codes": "Comma separated ISO 639-3 language codes",
      "Accuracy": "Accuracy",
      "Simulation mode": "Simulation mode",
      "This post is treated by language filter": "This post is %{treated} by language filter.",
      "blocked": "blocked",
      "passed": "passed",
      "not analyzed": "not analyzed",
      "Reason: this": "Reason: %{this}",
      "this doesnt contain that": "%{this} does not contain %{that}",
      "this is undefined": "%{this} is undefined",
      "blacklist": "blacklist",
      "whitelist": "whitelist",
      "language of this": "language of this",
      "its undefined language": "it is undefined language",
      "its this, blacklisted": "it is %{this}, blacklisted",
      "its this, whitelisted": "it is %{this}, whitelisted",
      "Most possible language: this": "Most possible language: %{this}",
      "Scope of usage": "Scope of usage",
      "Show with every user name": "Show with every user name",
      "Show at profile modal only": "Show at profile modal only",
      "Show if a user follows me": "Show if a user follows me",
      "follows you": "follows you",
      "Show conversation": "Show conversation",
      "Mark all as read": "Mark all as read",
      "show_more_count": "%{smart_count} more...",
      "hide": "hide",
      "Show more in this conversation...": "Show more in this conversation...",
      "conversation_title": "Conversation of @%{username}",
      "copy_to_clipboard": "Press Ctrl/Cmd+C to copy then Enter to close",
      "Normal posts": "Normal posts",
      "Promoted posts": "Promoted posts",
      'promoted': 'Promoted',
      "Messages": "Messages",
      "Edit profile": "Edit profile",
      "Top Trends": "Top Trends",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "twisted again by",
      "post_rt_time_prep": "at",
      "undo": "undo",
      "Daemon exited...": "Daemon exited...",
      "Copy to clipboard": "Copy to clipboard",
      "Full name here": "Full name here",
      "Describe yourself": "Describe yourself",
      "Location": "Location",
      "website": "website",
      "Tox address": "Tox address",
      "Bitmessage address": "Bitmessage address",
      "Language": "Language",
      "Sound": "Sound",
      "Users": "Users",
      'dm_copy_outgoing_to_self': 'Copy outgoing messages to self',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Traffic information",
      "DHT Torrents:": "DHT Torrents:",
      "Peers:": "Peers:",
      "Peer List Size:": "Peer List Size:",
      "Active Requests:": "Active Requests:",
      "Download:": "Download:",
      "Upload:": "Upload:",
      "DHT Download:": "DHT Download:",
      "DHT Upload:": "DHT Upload:",
      "IP Overhead Download:": "IP Overhead Download:",
      "IP Overhead Upload:": "IP Overhead Upload:",
      "Payload Download:": "Payload Download:",
      "Payload Upload:": "Payload Upload:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "tr"){
    polyglot.locale("tr");
    wordset = {
      "Actions ▼": "Eylemler ▼",
      "Active DHT nodes:": "Aktif DHT düğümleri: ",
      "Add DNS": "DNS ekle",
      "Add peer": "Eş ekle",
      "ajax_error": "Ajax hatası: %{error}", // JavaScript error
      "All users publicly followed by": "Açık olarak takip edilen tüm kullanıcılar",
      "Available": "Kullanılabilir", // username is available
      "Appearance": "Görünüm",
      "Apply": "Uygula",
      "Block chain information": "Blok zincir bilgisi",
      "Block chain is up-to-date, twister is ready to use!": "Blok zinciri güncel, twister kullanıma hazır!",
      "Block generation": "Blok üretimi ",
      "busted_oh": "haydaaa!",
      "busted_avowal": "Şüpheli bir şeyler iliştirmeye çalışırken enselendim.",
      "btn_ok": "Tamam",
      "Cancel": "İptal",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Geçmişi temizle",
      "Confirm": "Onayla",
      "сonfirm_group_leaving_header": "Gruptan ayrılmayı onayla",
      "сonfirm_group_leaving_body": "%{alias} grubundan ayrılmak istiyor musun?",
      "confirm_switch_to_network":
            "Yerel servis ağa bağlı değil ya da blok zinciri güncel değil.\n" +
            "Eğer bu sayfada kalırsanız eylemlerinizi işlemeyebilir.\n" +
            "Bunun yerine [Ağ Durumu sayfasını](%{page}) kontrol etmek ister misiniz?",
      "confirm_terminate_daemon": "Hizmeti sonlandırmak istiyor musun?\nTwister istemcisi çalışmayacak.",
      "confirm_unfollow_@": "@%{alias} kullanıcısını takip etmeyi bırakmak istiyor musun?",
      "confirm_uri_shortener_clear_cache": "Kısaltılmış adres geçimişini temizlemek istiyor musun?",
      "Change user": "Kullanıcı değiştir",
      "Checking...": "Denetleniyor...", // checking if username is available
      "Collapse": "Kapat", // smaller view of a post
      "Configure block generation": "Blok üretim ayarları",
      "Connections:": "Bağlantılar: ", // to network
      "Connection lost.": "Bağlantı koptu.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} gün |||| %{smart_count} gün",
      "Detailed information": "Detaylı bilgi",
      "DHT network down.": "DHT ağı çalışmıyor.",
      "Direct Messages": "Direk Mesajlar",
      "Group Messages": "Grup Mesajları",
      "Group Messages — New Group Creation": "Grup Mesajları — Yeni Grup Oluştur",
      "Group Messages — Join Group": "Grup Mesajları — Gruba katıl",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "%{username} ile Direk Mesajlar",
      "Disable": "Kullanılmaz",
      "display_mentions": "Bahsedenleri göster",
      "Display retransmissions": "Tekrar iletimleri göster",
      "DNS to obtain list of peers:": "Eş listesini almak için DNS:",
      "do_not_show_it_again": "bir daha gösterme",
      "downloading_block_chain": "Blok zinciri indiriliyor, devam edebilmek için lütfen bekleyiniz (blok zinciri %{days} günlük).",
      "download_posts_status": "Göderilerin indirilme oranı: %{portion}", // Downloaded 10/30 posts
      "Enable": "Kullanılabilir",
      "error": "Hata: %{error}",
      "error_connecting_to_daemon": "Yerel twister servisine bağlanma hatası.",
      "Error in 'createwalletuser' RPC.": "'createwalletuser' RPC'de (Uzak Yordam Çağrısında) hata.",
      "Error in 'importprivkey'": "'importprivkey' RPC'de (Uzak Yordam Çağrısında) hata: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "'sendnewusertransaction' RPC'de (Uzak Yordam Çağrısında) hata.",
      "Expand": "Aç", // larger view of a post
      "Favorite": "Favori",
      "File APIs not supported in this browser.": "Tarayıcınızda dosya API'si desteklenmiyor.",
      "Follow": "Takip et",
      "Following config": "Takip ayarları",
      "select_way_to_follow_@": "Nasıl takip etmek istiyorsun @%{alias}",
      "Followed by": "Takip edenler",
      "followed_by": "%{username} tarafından takip edilenler",
      "Followers": "Takipçiler",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Takip edilenler",
      "Following users": "Takip edilen kullanıcılar",
      "Force connection to peer:": "Bağlantı için zorlanacak eş:",
      "General information": "Genel bilgi",
      "Generate blocks (send promoted messages)": "Blok oluştur (destekli mesaj gönder)",
      "Home": "Anasayfa", // homepage
      "hours": "%{smart_count} saat |||| %{smart_count} saat",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Dahili hata: lastPostId bilinmiyor (kendinizi takip etmek bu sorunu giderebilir!)",
      "Known peers:": "Bilinen eşler: ",
      "Last block is ahead of your computer time, check your clock.": "Son blok zamanı sistem saatinden daha ileride, saatinizi kontrol ediniz.",
      "mentions_at": "@%{user} kullanıcısından bahsedenler",
      "minutes": "%{smart_count} dakika |||| %{smart_count} dakika",
      "Must be 16 characters or less.": "16 karakterden daha uzun olamaz.", // username
      "Network": "Ağ",
      "Network config": "Ağ ayarları",
      "Network status": "Ağ durumu",
      "New direct message...": "Yeni direk mesaj...",
      "New Post...": "Yeni gönderi...",
      "New group": "Yeni grup",
      "Group description": "Grup açıklaması",
      "Peers to invite": "Davet edilecek eşler",
      "Join group": "Gruba katıl",
      "Select group(s)": "Grup(ları) seç",
      "Create": "Oluştur",
      "Join": "Katıl",
      "Invite": "Davet et",
      "Invite peers": "Eşleri davet et",
      "Leave group": "Gruptan ayrıl",
      "You got": "Oldu",
      "in postboard": "gönederilerde",
      "in search result": "arama sonuçlarında",
      "in top trends": "gündemde",
      "new_posts": "%{smart_count} yeni gönederi |||| %{smart_count} yeni gönderi",
      "new_mentions": "%{smart_count} yeni bahseden |||| %{smart_count} yeni bahseden",
      "new_direct_messages": "%{smart_count} yeni direk mesaj |||| %{smart_count} yeni direk mesaj",
      "new_group_messages": "%{smart_count} yeni grup mesajı |||| %{smart_count} yeni grup mesajı",
      "nobody": "hiçkimse", // used to promote a post without attaching the user
      "Not available": "Kullanılamaz", // username is not available
      "warn_following_not_any": "Hiç kimseyi takip etmiyorsun!\nAra ve birilerini takip et.",
      "warn_followers_not_all": "Tüm takipçilerini belirlemenin kolay bir yolu bulunmuyor.\n"
        + "Takipçi sayacı torrent'ini paylaşan eşlerin sayısından ibaret ve\n"
        + "aşağıdaki liste çoğunlukla takip ettiklerinden seni takip edenleri içerir.",
      "warn_followers_not_all_of": "Tüm takipçilerini belirlemenin kolay bir yolu bulunmuyor.\n"
        + "Aşağıdaki liste takipçilerin bir kısmını ve çoğunlukla takip ettiklerinden @%{alias} kullanıcısını gizlemeden takip edenleri içerir.",
      "notify_desktop_error": "Twister masaüstü uyarısını gösterimiyor: bilimeyen bir hata oluştu.",
      "notify_desktop_perm_denied": "Twister masaüstü uyarısını gösteremiyor: yetkilendirme hatası.\n\nUyarıları almak istiyorsanız, tarayıcı ayarlarında %{this_domain} alan adı için izin veriniz.",
      "notify_desktop_test": "Kasırga her yanı saracak\nBu karnavala hoşgeldin!",
      "notify_desktop_title": "İyi bak, twister her yerde!",
      "post_preview_dummy": 'işte *kalın*, ~yatık~, -üstü-çizik- ve _alt çizgili_ yazı.\n'
        + 'düz hali: `*kalın*, ~yatık~, -üstü-çizik- ve _alt çizgili_`.\n'
        + '[güzel logomuz](%{logo})\'un ve harika sitemizin: %{site} bağlantıları.',
      "Number of blocks in block chain:": "Blok zincirindeki blok sayısı: ",
      "Number of CPUs to use": "Kullanılacak CPU sayısı ",
      "Only alphanumeric and underscore allowed.": "Sadece harf ve alt çizgi kullanılabilir.",
      "peer address": "eş adresi",
      "Private": "Özel",
      "Profile": "Profil",
      "Postboard": "Gönderiler",
      "post": "gönder", // verb - button to post a message
      "Post to promote:": "Desteklenecek gönderi: ",
      "Posts": "Gönderiler",
      "propagating_nickname": "%{username} kullanıcı adı ağda yayımlanıyor...",
      "Public": "Genel",
      "Refresh": "Yenile",
      "retransmit_this": "Bu gönderi, takipçilerine takrar iletilsin mi?",
      "Reply": "Cevapla",
      "Reply...": "Cevapla...",
      "reply_to": "%{fullname} kullancısını cevapla",
      "Retransmit": "Tekar ilet",
      "Retransmits": "Tekrar iletenler",
      "Retransmitted by": "Tekrar ileten",
      "Switch to Reply": "'Yanıt' ara yüzü...",
      "Switch to Retransmit": "'Yeniden ilet' ara yüzü",
      "search": "ara",
      "seconds": "%{smart_count} saniye |||| %{smart_count} saniye",
      "send": "gönder",
      "Send post with username": "İletiyi kullanıcı adıyla gönder ",
      "send_DM": "Direk Mesaj Gönder",
      "Sent Post to @": "@ Kullanıcıya Gönder",
      "Setup account": "Hesap ayarları",
      "shorten_URI": "URL kısalt",
      "shorten_URI_enter_link": "Uzun bağlantıyı yazın.\n"
        + "Not: URL kısaltma işlemi tam URL'i barındıran 'boş' bir twist oluşturacak.\n"
        + "Bu özel twist görüntülenmeyecek, fakat gönderi sayınızda hesaba katılacak.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "Adres kısalt",
      "The File APIs are not fully supported in this browser.": "Dosya API'si tarayıcınızda tam olarak desteklenmiyor.",
      "time_ago": "%{time} önce", // 5 minutes ago
      "Time of the last block:": "Son blok saati: ",
      "Type message here": "Mesajı buraya yazınız",
      "Unfollow": "Takibi bırak",
      "Update": "Güncelle",
      "Auto updating": "Otomatik güncelleme",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Durum güncelleniyor...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Diğer eşler bu yeni kullanıcıyı henüz kabul etmediler.\n" +
                "Malesef profili kaydetmek ya da bu durumda\n" +
                "ileti gönedermek mümkün değil.\n\n" +
                "Lütfen devam etmek için bir kaç dakika bekleyin.\n\n" +
                "İşlem tamamlandığında 'Değişiklikleri Kaydet'\n" +
                "kediliğinden kullanılabilir olacaktır. (Twister'ı kullanmak için\n"+
                "bu son bekleyişiniz olduğu konusunda size teminat veriyorum).\n\n" +
                "İpucu: beklerken profil resminizi seçiniz!",
      "users_mentions": "@%{username} kullanıcısından bahsedenler",
      "users_profile": "%{username} kullanıcısının Profili",
      "username_undefined": "Kullanıcı adı belirsiz, giriş gerekli.",
      "View": "Göster",
      "View All": "Hepsini Göster",
      "Who to Follow": "Kimi takip etmeli",
      "Your message was sent!": "Mesajınız gönderildi!",
      "twister login": "twister girişi",
      "Existing local users": "Var olan yerel kullanıcılar",
      "Or...": "Ya da...",
      "Create a new user": "Yeni bir kullanıcı oluştur",
      "Login": "Giriş",
      "Check availability": "Kullanılabilirliliği denetle",
      "Create this nickname": "Bu takma ismi oluştur",
      "Type nickname here": "Takma ismi buraya yazınınız",
      "Import secret key": "Gizli anahtarı içeri aktar",
      "52-characters secret": "52-karakterli gizli anahtar",
      "With nickname": "Takma ismi kullanarak",
      "Import key": "Anahtarı içe aktar",
      "Client Version:": "İstemci versiyonu:",
      "Mining difficulty:": "Madencilik zorluğu:",
      "Block generation status": "Block oluşturma durumu",
      "Current hash rate:": "Geçerli özet oranı:",
      "Terminate Daemon:": "Servisi Durdur:",
      "Exit": "Çıkış",
      "Save Changes": "Değişiklikleri Kaydet",
      "profile_saved": "Profil bilgileri DHT'ye kaydedildi.",
      "profile_not_saved": "Profil bilgileri kaydedilmedi.",
      "Secret key:": "Gizli anahtar:",
      "Options": "Ayarlar",
      "Switch to Promoted posts": "Destekli Mesajlara Geç",
      "Switch to Normal posts": "Normal Mesajlara Geç",
      "Use language": "Dil ayarla",
      "Ignore": "Görmezden gel",
      "Ignore and clear out": "Görmezden gel ve temizle",
      "Theme": "Tema",
      "Keys": "Tuşlar",
      "Notifications": "Uyarılar",
      "Desktop notifications": "Masaüstü uyarılar",
      "Sound notifications": "Sesli uyarılar",
      "Volume": "Ses düzeyi",
      "Test": "Deneme",
      "Send key": "Gönderme tuşu",
      "Posts display": "Gönderiler",
      "Post editor": "Gönderi düzenleyici",
      "Post preview": "Post preview",
      "Inline image preview": "Dahili resim ön izleme",
      "Display": "Göster",
      "Line feeds": "Satır sonları",
      "Markout": "Markout markup",
      "Supported punctuations:": "Desteklenen noktalama işaretleri:",
      "Supported emotions:": "Desteklenen duygu simgeleri:",
      "Supported signs:": "Desteklenen işaretler:",
      "Supported fractions:": "Desteklenen kesirler:",
      "Automatic unicode conversion options": "Otomatik unicode dönüştürme seçenekleri",
      "Convert punctuations to unicode": "Noktalama işaretlerini unicode'a dönüştür",
      "Convert emotions codes to unicode symbols": "Duygu simge kodlarını unicode simgelerine dönüştür",
      "Convert common signs to unicode": "Yaygın işaretleri unicode'a dönüştür",
      "Convert fractions to unicode": "Kesirleri unicode'a dönüştür",
      "Convert all": "Hepsini dönüştür",
      "Auto": "Otomatik",
      "Original": "Orjinal",
      "none": "Hiçbiri",
      "Custom": "Özel",
      "Mentions": "Bahsedenler",
      "You have to log in to post messages.": "Mesaj göndermek için giriş yapmalısınız.",
      "You have to log in to post replies.": "Cevap göndermek için giriş yapmalısınız.",
      "You have to log in to retransmit messages.": "Yeniden iletmek için giriş yapmalısınız.",
      "You have to log in to use direct messages.": "Direk masajları kullanabilmek için giriş yapmalısınız.",
      "You have to log in to follow users.": "Kullanıcı takip etmek için giriş yapmalısınız.",
      "You are not following anyone because you are not logged in.": "Giriş yapmadığınız için kimseyi takip etmiyorsunuz.",
      "You don't have any followers because you are not logged in.": "Giriş yapmadığınız için hiç takipçiniz yok.",
      "No one can mention you because you are not logged in.": "Giriş yapmadığınız için kimse adınıza mesaj gönderemiyor.",
      "You don't have any profile because you are not logged in.": "Giriş yapmadığınız için profiliniz yok.",
      "Use proxy for image preview only": "Vekil sunucuyu sadece resim ön izleme için kullan",
      "Use external links behind a proxy": "Harici bağlantılar için vekil sunucu kullan",
      "There aren't any posts with this hashtag.": "Bu etiketle ilgili gönderi yok.",
      "Split only new post": "Sadece yeni postaları böl",
      "Split all": "Hepsini böl",
      "Don't split": "Bölme",
      "Split long posts": "Uzun gönderileri böl",
      "Posts that begin with mention": "Bahsettiği kullanıcının adıyla başlayan gönderiler",
      "Show all": "Hepsini göster",
      "Show only if I am in": "Sadece ben içindeysem göster",
      "Show if it's between users I follow": "Takip ettiğim kullanıcılar arasında ise göster",
      "Postboard displays": "Zaman çizelgesinde",
      "RTs those are close to original twist": "Orjinal twist'e yakın olan RTler",
      "Show if the original is older than": "Orjinali yandaki süreden daha eskiyse göster",
      "hour(s)": "saat",
      "second(s)": "saniye",
      "only positive numbers!": "sadece rakam girilebilir!",
      "Language filtering": "Dile göre süzme",
      "By blacklist": "Kara liste ile",
      "By whitelist": "Beyaz liste ile",
      "Comma separated ISO 639-3 language codes": "Virgülle ayrılmış ISO 639-3 dil kodları",
      "Accuracy": "Hassasiyet",
      "Simulation mode": "Benzetim yap",
      "This post is treated by language filter": "Bu gönderi dil süzgeci tarafından %{treated}.",
      "blocked": "engellendi",
      "passed": "kabul edildi",
      "not analyzed": "incelenmedi",
      "Reason: this": "Sebep: %{this}",
      "this doesnt contain that": "%{that} %{this} tarafından kapsanmıyor",
      "this is undefined": "%{this} tanımsız",
      "blacklist": "beyaz liste",
      "whitelist": "kara liste",
      "language of this": "dili",
      "its undefined language": "belirsiz dil",
      "its this, blacklisted": "%{this}, engellendi",
      "its this, whitelisted": "%{this}, kabul edildi",
      "Most possible language: this": "Muhtemel dil: %{this}",
      "Scope of usage": "Kullanım alanı",
      "Show with every user name": "Tüm kullanıcı adlarının yanında göster",
      "Show at profile modal only": "Sadece profilinde göster",
      "Show if a user follows me": "Bir kullanıcının beni takip edip etmediğini göster",
      "follows you": "seni takip ediyor",
      "Show conversation": "Sohbeti göster",
      "Mark all as read": "hepsini okundu olarak işaretle",
      "show_more_count": "%{smart_count} tane daha...",
      "hide": "gizle",
      "Show more in this conversation...": "Sohbetin tamamını göster...",
      "conversation_title": "@%{username} kullanıcısının bir sohbeti",
      "copy_to_clipboard": "Kopyalamak için Ctrl/Cmd+C tuşlarına, kapatmak için Enter tuşuna basınız",
      "Normal posts": "Normal gönderiler",
      "Promoted posts": "Desteklenen gönderiler",
      'promoted': 'Promoted',
      "Messages": "İletiler",
      "Edit profile": "Profili düzenle",
      "Top Trends": "Gündem",
      "Twistday Reminder": "Twistgünü Hatırlatıcı",
      "Show upcoming in near future": "Yakın gelecektekileri göster",
      "Who's celebrating Twistday": "Twistgünü'nü kutlayanlar",
      "Today's luckies:": "Günün talihlileri:",
      "Upcoming ones:": "Yakında:",
      "post_rt_sign_prep": "yeniden ileten",
      "post_rt_time_prep": "at",
      "undo": "geri al",
      "Daemon exited...": "Hizmet sonlandırıldı...",
      "Secret Key": "Gizli Anahtar",
      "Copy to clipboard": "Panoya kopyala",
      "Full name here": "Tam isim buraya",
      "Describe yourself": "Kendini tanımla",
      "Location": "Konum",
      "website": "website",
      "Tox address": "Tox adresi",
      "Bitmessage address": "Bitmessage adresi",
      "Language": "Dil",
      "Sound": "Ses",
      "Users": "Kullanıcılar",
      'dm_copy_outgoing_to_self': 'Özel iletinin kopyasını sakla',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "Traffic information": "Trafik bilgileri",
      "DHT Torrents:": "DHT Torrentleri:",
      "Peers:": "Eşler:",
      "Peer List Size:": "Eş listesi uzunluğu:",
      "Active Requests:": "Aktif istekler:",
      "Download:": "İndirme:",
      "Upload:": "Yükleme:",
      "DHT Download:": "DHT İndirme:",
      "DHT Upload:": "DHT Yükleme:",
      "IP Overhead Download:": "IP Başlık İndirme:",
      "IP Overhead Upload:": "IP Başlık Yükleme:",
      "Payload Download:": "İçerik İndirme:",
      "Payload Upload:": "İçerik Yükleme:",
      "No favs here because you are not logged in." : "Giriş yapmadan favorileri göremezsin.",
      "users_favs": "@%{username} kullanıcısının favorileri",
      "Favorites": "Favoriler",
      "You have to log in to favorite messages.": "İletileri favorine eklemek için giriş yapmalısın.",
      "fav_this": "Sana özel mi?",
      "Last activity": "Son etkinlik",
      "New Users": "Yeni Kullanıcılar",
      "Live tracking" : "Canlı takip"
    };
}

if(preferredLanguage == "cs"){
    polyglot.locale("cs");
    wordset = {
      "Actions ▼": "Akce ▼",
      "Active DHT nodes:": "Aktivní uzly DHT: ",
      "Add DNS": "Přidat DNS",
      "Add peer": "Přidat uzel",
      "ajax_error": "Ajax chyba: %{error}", // JavaScript error
      "All users publicly followed by": "Všichni uživatelé, které veřejně sleduje",
      "Available": "Je volná", // username is available
      "Appearance": "Vzhled",
      "Apply": "Apply",
      "Block chain information": "Informace o blockchainu",
      "Block chain is up-to-date, twister is ready to use!": "Blockchain je aktuální, twister je připraven k použití!",
      "Block generation": "Generování bloků ",
      "busted_oh": "oh noes!",
      "busted_avowal": "I\'m busted trying inject this suspicious stuff here",
      "btn_ok": "Okay",
      "Cancel": "Zrušit",
      'cant_get_requested_resourse': 'Can\'t get resourse at %{link}\n  status: %{status}.',
      "clear_cache": "Clear cache",
      "Confirm": "Confirm",
      "сonfirm_group_leaving_header": "Confirm group leaving",
      "сonfirm_group_leaving_body": "Are you sure want to leave %{alias} group?",
      "confirm_switch_to_network":
            "Váš lokální server ještě není připojen k síti, nebo teprve stahuje aktuální blockchain.\n" +
            "Pokud zůstanete na této stránce, vaše akce možná nebudou fungovat.\n" +
            "Chcete se místo toho podívat na [stav sítě](%{page})?",
      "confirm_terminate_daemon": "Skutečně chcete ukončit server?\nTwister tím vypnete.",
      "confirm_unfollow_@": "Are you sure you want to unfollow @%{alias}?",
      "confirm_uri_shortener_clear_cache": "Are you sure you want to clear browser cache of shortened URIs?",
      "Change user": "Změnit uživatele",
      "Checking...": "Ověřuji...", // checking if username is available
      "Collapse": "Složit", // smaller view of a post
      "Configure block generation": "Nastavení generace bloků",
      "Connections:": "Připojení: ", // to network
      "Connection lost.": "Žádné připojení do sítě.",
      "daemon_is_obsolete": "twister daemon is obsolete, version %{versionReq} or higher is required",
      "days": "%{smart_count} dnem |||| %{smart_count} dny |||| %{smart_count} dny",
      "Detailed information": "Podrobné informace",
      "DHT network down.": "Síť DHT je nedostupná.",
      "Direct Messages": "Přímé zprávy",
      "Group Messages": "Group Messages",
      "Group Messages — New Group Creation": "Group Messages — New Group Creation",
      "Group Messages — Join Group": "Group Messages — Join Group",
      "group_key_cant_import": "Can\'t import DM group key",
      "group_key_is_invalid_perhaps": "perhaps the key is invalid",
      "group_key_was_imported": "Key for DM group %{alias} was imported.\n"
        + "Its messages will be fetched soon.",
      "direct_messages_with": "Direct messages with %{username}",
      "Disable": "Vypnuto",
      "display_mentions": "Zobrazit zmínky",
      "Display retransmissions": "Zobrazit přeposlané",
      "DNS to obtain list of peers:": "DNS pro načtení seznamu uzlů:",
      "do_not_show_it_again": "don't show it again",
      "downloading_block_chain": "Stahuji blockchain, prosím počkejte (blockchain je %{days} dnů starý).",
      "download_posts_status": "Staženo %{portion} příspěvků", // Downloaded 10/30 posts
      "Enable": "Zapnuto",
      "error": "Chyba: %{error}",
      "error_connecting_to_daemon": "Chyba: Nemohu se připojit k vašemu lokálnímu serveru.",
      "Error in 'createwalletuser' RPC.": "Chyba v RPC funkci 'createwalletuser'.",
      "Error in 'importprivkey'": "Chyba v RPC funkci 'importprivkey'. RPC: %{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "Chyba v RPC funkci 'sendnewusertransaction'.",
      "Expand": "Rozbalit", // larger view of a post
      "Favorite": "Přidat k oblíbeným",
      "File APIs not supported in this browser.": "Upozornění: váš webový prohlížeč nepodporuje File API.",
      "Follow": "Sledovat",
      "Following config": "Nastavení sledování",
      "select_way_to_follow_@": "Zvolte způsob sledování @%{alias}",
      "Followed by": "tohoto uživatele sleduje",
      "followed_by": "Uživatelé, které sleduje %{username}",
      "Followers": "Sledující",
      "Followers_of": "Followers of @%{alias}",
      "Following": "Sledovaní",
      "Following users": "Sledovaní uživatelé",
      "Force connection to peer:": "Vyžádat připojení k uzlu:",
      "General information": "Obecné informace",
      "Generate blocks (send promoted messages)": "Generovat bloky (posílat přednostní příspěvky)",
      "Home": "Domů", // homepage
      "hours": "%{smart_count} hodinou |||| %{smart_count} hodinami |||| %{smart_count} hodinami",
      "Internal error: lastPostId unknown (following yourself may fix!)": "Vnitřní chyba: neznámé lastPostId",
      "Known peers:": "Známé uzly: ",
      "Last block is ahead of your computer time, check your clock.": "Čas posledního bloku je popředu před časem vašeho počítače, zkontrolujte seřízení hodin.",
      "mentions_at": "Zmínky o @%{user}",
      "minutes": "%{smart_count} minutou |||| %{smart_count} minutami |||| %{smart_count} minutami",
      "Must be 16 characters or less.": "Příliš dlouhé, maximum je 16 znaků.", // username
      "Network": "Síť",
      "Network config": "Nastavení sítě",
      "Network status": "Stav sítě",
      "New direct message...": "Nová přímá zpráva...",
      "New Post...": "Nový příspěvek...",
      "New group": "New group",
      "Group description": "Group description",
      "Peers to invite": "Peers to invite",
      "Join group": "Join group",
      "Select group(s)": "Select group(s)",
      "Create": "Create",
      "Join": "Join",
      "Invite": "Invite",
      "Invite peers": "Invite peers",
      "Leave group": "Leave group",
      "You got": "Máte",
      "in postboard": "na nástěnce",
      "in search result": "ve výsledcích hledání",
      "in top trends": "v hlavních trendech",
      "new_posts": "%{smart_count} nový příspěvek |||| %{smart_count} nové příspěvky |||| %{smart_count} nových příspěvků",
      "new_mentions": "%{smart_count} novou zmínku |||| %{smart_count} nové zmínky |||| %{smart_count} nových zmínek",
      "new_direct_messages": "%{smart_count} novou přímou zprávu |||| %{smart_count} nové přímé zprávy |||| %{smart_count} nových přímých zpráv",
      "new_group_messages": "%{smart_count} new group message |||| %{smart_count} new group messages",
      "nobody": "nikdo", // used to promote a post without attaching the user
      "Not available": "Tuto přezdívku již někdo používá", // username is not available
      "warn_following_not_any": "Not following any twisters!\nSearch and follow someone.",
      "warn_followers_not_all": "Well, currently here's no easy way to obtain all your followers.\n"
        + "The counter indicates only a number of known peers sharing your torrent.\n"
        + "In the list below are included ones, mostly, who are followed by you.",
      "warn_followers_not_all_of": "Well, currently here's no easy way to obtain all followers of whoever.\n"
        + "In the list below are included only few, perhaps, of ones who follow @%{alias} publicly.",
      "notify_desktop_error": "Twisteru se nepodařilo zobrazit upozornění na ploše: došlo k neznámé chybě.",
      "notify_desktop_perm_denied": "Twisteru se nepodařilo zobrazit upozornění na ploše: přístup byl odepřen.\n\nPokud chcete používat upozornění na ploše, povolte je pro %{this_domain} v nastavení vašeho prohlížeče.",
      "notify_desktop_test": "Twister to umí pořádně roztočit.\nKaždý je tu vítán.",
      "notify_desktop_title": "Twister",
      "post_preview_dummy": 'here is *bold*, ~italic~, -strike-trough- and _underlined_ text.\n'
        + 'same but escaped: `*bold*, ~italic~, -strike-trough- and _underlined_`.\n'
        + 'links to [our awesome logo](%{logo}) and our gorgeous site: %{site}.',
      "Number of blocks in block chain:": "Počet bloků v blockchainu: ",
      "Number of CPUs to use": "Kolik jader procesoru použít? ",
      "Only alphanumeric and underscore allowed.": "Povolena jsou jen písmena, čísla a podtržítko.",
      "peer address": "adresa uzlu",
      "dns address": "adresa dns",
      "Private": "Soukromé",
      "Profile": "Profil",
      "Postboard": "Nástěnka",
      "post": "vložit", // verb - button to post a message
      "Post to promote:": "Váš přednostní příspěvek: ",
      "Posts": "Příspěvky",
      "propagating_nickname": "Posílám novou přezdívku %{username} do sítě...",
      "Public": "Veřejné",
      "Refresh": "Obnovit",
      "retransmit_this": "Chcete tento příspěvek přeposlat těm, kdo vás sledují?",
      "Reply": "Odpovědět",
      "Reply...": "Odpověď...",
      "reply_to": "Odpověď pro %{fullname}",
      "Retransmit": "Přeposlat",
      "Retransmits": "Přeposlání",
      "Retransmitted by": "Přeposlal(a)",
      "Switch to Reply": "Switch to Reply",
      "Switch to Retransmit": "Switch to Retransmit",
      "search": "hledat",
      "seconds": "%{smart_count} vteřinou |||| %{smart_count} vteřinami |||| %{smart_count} vteřinami",
      "send": "odeslat",
      "Send post with username": "Příspěvek vložit pod přezdívkou ",
      "send_DM": "Poslat přímou zprávu",
      "Sent Post to @": "Poslat veřejný příspěvek pro @",
      "Setup account": "Upravit profil",
      "shorten_URI": "Shorten URL",
      "shorten_URI_enter_link": "Enter the long link.\n"
        + "Note: shortening an URL will produce an 'empty' twist on your behalf containing the full URL.\n"
        + "This special twist is not displayed by twister clients, but your twist counter will increase.",
      "shorten_URI_its_public_is_it_ok": "Your link will be public available! Are you OK with that?",
      "URI_shortener": "URI Shortener",
      "The File APIs are not fully supported in this browser.": "Upozornění: váš webový prohlížeč nepodporuje File API.",
      "time_ago": "před %{time}", // 5 minutes ago
      "Time of the last block:": "Čas posledního bloku: ",
      "Type message here": "Sem napište příspěvek",
      "Unfollow": "Přestat sledovat",
      "Update": "Aktualizovat",
      "Auto updating": "Automatická aktualizace",
      'updates_are_available': 'Updates are available',
      'updates_not_available': 'No updates are available',
      'updates_check_client': 'Check for client\'s updates',
      'updates_repo_overview': 'Currently we are on the branch %{branch} of %{repo} at the commit\n  %{commit} of %{date}\n'
        + 'but HEAD of its source is already at the commit\n  %{commitUpstream} of %{dateUpstream}.',
      'updates_checkout_diff': 'Checkout [the diff on GitHub](%{link}) to learn what is changed.',
      'updates_checkout_diff_nfmt': 'Checkout the diff on GitHub to learn what is changed:\n  %{link}',
      'updates_upstream_isnt_changed': 'A corresponding branch of the source repository seems to have not changed.',
      "Updating status...": "Aktualizuji stav...", // status of block chain
      'new_account_briefing': 'A new account was created and it is being propagated to the network. '
        + 'Please do not close this window, this might take a few minutes.\n\n'
        + 'Your secret key is: *%{secretKey}*\n\n'
        + 'It is highly recommended that you take this time to save your secret key. '
        + 'Print it, do a screenshot, use the camera in your phone or write it down to a piece of paper.\n\n'
        + 'The secret key will be needed to use this account from different computers. '
        + 'If you ever lose your secret key your account will be locked forever '
        + '(note: ~this is alpha software and it may crash, causing loss of data~).\n\n'
        + 'Just wait. The OK button will be shown faster you can say ~decentralization~.',
      "user_not_yet_accepted": "Ostatní uzly ještě nepřijaly tohoto nového uživatele.\n" +
                "Proto zatím není možné uložit profil nebo vkládat příspěvky.\n" +
                "Počkejte prosím pár minut.\n\n" +
                "Tlačítko \"Uložit změny\" bude po dokončení tohoto procesu automaticky\n" +
                "povoleno. Slibujeme, že je to naposledy, co musíte čekat než budete moci\n"+
                "používat twister.\n" +
                "Tip: vyberte si mezitím svůj obrázek!",
      "users_mentions": "Zmínky o @%{username}",
      "users_profile": "Profil %{username}",
      "username_undefined": "Uživatelské jméno není definováno, je třeba se přihlásit.",
      "View": "Zobrazit",
      "View All": "Zobrazit vše",
      "Who to Follow": "Koho sledovat",
      "Your message was sent!": "Vaše zpráva byla odeslána!",
      "twister login": "přihlášení na twister",
      "Existing local users": "Existující lokální účty",
      "Or...": "Nebo...",
      "Create a new user": "Vytvořit nový účet",
      "Login": "Přihlásit",
      "Check availability": "Ověřit dostupnost",
      "Create this nickname": "Vytvořit tuto přezdívku",
      "Type nickname here": "Napište sem přezdívku",
      "Import secret key": "Importovat tajný klíč",
      "52-characters secret": "52 znaků dlouhý tajný klíč",
      "With nickname": "S přezdívkou",
      "Import key": "Importovat klíč",
      "Client Version:": "Verze klienta:",
      "Mining difficulty:": "Obtížnost pro mining:",
      "Block generation status": "Stav generování bloků",
      "Current hash rate:": "Aktuální rychlost hashování:",
      "Terminate Daemon:": "Ukončit server:",
      "Exit": "Konec",
      "Save Changes": "Uložit změny",
      "profile_saved": "Profile data has been saved to DHT.",
      "profile_not_saved": "Profile data has been not saved.",
      "Secret key:": "Tajný klíč:",
      "You have to log in to post messages.": "Pro vkládání příspěvků je třeba se přihlásit.",
      "You have to log in to post replies.": "Pro vkládání odpovědí je třeba se přihlásit.",
      "You have to log in to retransmit messages.": "Pro přeposílání příspěvků je třeba se přihlásit.",
      "You have to log in to use direct messages.": "Pro používání přímých zpráv je třeba se přihlásit.",
      "You have to log in to follow users.": "Pro sledování jiných uživatelů je třeba se přihlásit.",
      "You are not following anyone because you are not logged in.": "Nikoho nesledujete, protože nejste přihlášen(a).",
      "You don't have any followers because you are not logged in.": "Nikdo vás nesleduje, protože nejste přihlášen(a).",
      "No one can mention you because you are not logged in.": "Nikdo se o vás nemůže zmínit, protože nejste přihlášen(a).",
      "You don't have any profile because you are not logged in.": "Nemáte žádný profil, protože nejste přihlášen(a).",
      "Options": "Možnosti",
      "Switch to Promoted posts": "Přepnout na přednostní příspěvky",
      "Switch to Normal posts": "Přepnout na normální příspěvky",
      "Use language": "Jazyk",
      "Ignore": "Ignorovat",
      "Ignore and clear out": "Ignore and clear out",
      "Theme": "Styl grafiky",
      "Keys": "Klávesy",
      "Notifications": "Upozornění",
      "Desktop notifications": "Upozornění na ploše",
      "Sound notifications": "Zvuková upozornění",
      "Volume": "Hlasitost",
      "Test": "Zkouška",
      "Send key": "Klávesa pro odeslání",
      "Posts display": "Zobrazení příspěvků",
      "Post editor": "Editor příspěvků",
      "Post preview": "Post preview",
      "Inline image preview": "Náhled vložených obrázků",
      "Display": "Zobrazit",
      "Line feeds": "Nové řádky",
      "Markout": "Markout markup",
      "Supported punctuations:": "Podporovaná znaménka:",
      "Supported emotions:": "Podporovaní smajlíci:",
      "Supported signs:": "Podporované značky:",
      "Supported fractions:": "Podporované zlomky:",
      "Automatic unicode conversion options": "Automatická konverze znaků Unicode",
      "Convert punctuations to unicode": "Konvertovat interpunkční znaménka do Unicode",
      "Convert emotions codes to unicode symbols": "Konvertovat smajlíky do Unicode",
      "Convert common signs to unicode": "Konvertovat běžné značky do Unicode",
      "Convert fractions to unicode": "Konvertovat zlomky do Unicode",
      "Convert all": "Konvertovat vše",
      "Auto": "Detekovat automaticky",
      "Original": "Původní",
      "none": "žádné",
      "Custom": "Vlastní nastavení",
      "Mentions": "Zmínky",
      "Use proxy for image preview only": "Používat proxy jen pro náhledy obrázků",
      "Use external links behind a proxy": "Otevírat externí odkazy skrze proxy server",
      "There aren't any posts with this hashtag.": "Nebyly nalezeny žádné příspěvky s tímto štítkem.",
      "Split only new post": "Dělit pouze nově vkládaný příspěvek",
      "Split all": "Dělit všechny",
      "Don't split": "Nedělit",
      "Split long posts": "Dělení dlouhých příspěvků",
      "Posts that begin with mention": "Příspěvky začínající zmínkou",
      "Show all": "Ukázat všechny",
      "Show only if I am in": "Ukázat jen když zmiňují mně",
      "Show if it's between users I follow": "Ukázat pokud jsou posílány mezi uživateli, které sleduji",
      "Postboard displays": "Zobrazení na nástěnce",
      "RTs those are close to original twist": "Přeposlání blízká původnímu příspěvku",
      "Show if the original is older than": "Ukázat pokud je už originál starší než",
      "hour(s)": "hodin",
      "second(s)": "vteřin",
      "only positive numbers!": "povolena jsou jen kladná čísla!",
      "Language filtering": "Filtrování jazyků",
      "By blacklist": "Černá listina",
      "By whitelist": "Bílá listina",
      "Comma separated ISO 639-3 language codes": "Kódy jazyků ve formátu ISO 639-3, oddělené čárkami",
      "Accuracy": "Přesnost",
      "Simulation mode": "Režim simulace",
      "This post is treated by language filter": "Tento příspěvek %{treated} jazykovým filtrem.",
      "blocked": "byl zablokován",
      "passed": "prošel",
      "not analyzed": "nebyl analyzován",
      "Reason: this": "Důvod: %{this}",
      "this doesnt contain that": "%{this} neobsahuje %{that}",
      "this is undefined": "%{this} není definována",
      "blacklist": "černá listina",
      "whitelist": "bílá listina",
      "language of this": "tento jazyk",
      "its undefined language": "jazyk nelze určit",
      "its this, blacklisted": "jazyk %{this} je na černé listině",
      "its this, whitelisted": "jazyk %{this} je na bílé listině",
      "Most possible language: this": "Nejpravděpodobnější jazyk: %{this}",
      "Scope of usage": "Rozsah použití",
      "Show with every user name": "Ukázat u každé přezdívky",
      "Show at profile modal only": "Ukázat jen v okně profilu",
      "Show if a user follows me": "Ukazovat, zda mně uživatel sleduje",
      "follows you": "sleduje vás",
      "Show conversation": "Zobrazit rozhovor",
      "Mark all as read": "Označit všechny jako přečtené",
      "show_more_count": "%{smart_count} další... |||| %{smart_count} další... |||| %{smart_count} dalších...",
      "hide": "skrýt",
      "Show more in this conversation...": "Zobrazit tento rozhovor...",
      "conversation_title": "Rozhovor uživatele @%{username}",
      "copy_to_clipboard": "Stiskněte Ctrl+C pro zkopírování, a pak Enter pro zavření tohoto okénka.",
      "Normal posts": "Normální příspěvky",
      "Promoted posts": "Přednostní příspěvky",
      'promoted': 'Promoted',
      "Messages": "Zprávy",
      "Edit profile": "Upravit profil",
      "Top Trends": "Hlavní trendy",
      "Twistday Reminder": "Twistday Reminder",
      "Show upcoming in near future": "Show upcoming in near future",
      "Who's celebrating Twistday": "Who's celebrating Twistday",
      "Today's luckies:": "Today's luckies:",
      "Upcoming ones:": "Upcoming ones:",
      "post_rt_sign_prep": "přeposlal(a)",
      "post_rt_time_prep": "at",
      "undo": "zpět",
      "Daemon exited...": "Server ukončen...",
      "Secret Key": "Tajný klíč",
      "Copy to clipboard": "Kopírovat do clipboardu",
      "Full name here": "Sem patří celé jméno",
      "Describe yourself": "Napište něco o sobě",
      "Location": "Místo, kde žijete",
      "website": "Webové stránky",
      "Tox address": "Adresa pro Tox",
      "Bitmessage address": "Adresa pro Bitmessage",
      "Language": "Jazyk",
      "Sound": "Zvuky",
      "Users": "Uživatelé",
      'dm_copy_outgoing_to_self': 'Posílat kopie přímých zpráv sám sobě',
      'dm_encrypt_local_cache': 'Encrypt local data cache',
      "New post": "Nový příspěvek",
      "Search": "Hledat",
      "Direct Msg": "Přímá zpráva",
      "Traffic information": "Informace o provozu na síti",
      "DHT Torrents:": "DHT Torrenty:",
      "Peers:": "Uzly:",
      "Peer List Size:": "Velikost seznamu uzlů:",
      "Active Requests:": "Aktivní požadavky:",
      "Download:": "Stahování:",
      "Upload:": "Odesílání:",
      "DHT Download:": "DHT - stahování:",
      "DHT Upload:": "DHT - odesílání:",
      "IP Overhead Download:": "IP Overhead - stahování:",
      "IP Overhead Upload:": "IP Overhead - odesílání:",
      "Payload Download:": "Data aplikace - stahování:",
      "Payload Upload:": "Data aplikace - odesílání:",
      "No favs here because you are not logged in." : "No favs here because you are not logged in.",
      "users_favs": "Favorites of @%{username}",
      "Favorites": "Favorites",
      "You have to log in to favorite messages.": "You have to log in to favorite messages.",
      "fav_this": "Is it for you only?",
      "Last activity": "Last activity",
        "New Users": "New Users",
        "Live tracking" : "Live tracking"
    };
}

if(preferredLanguage == "cmn"){
    polyglot.locale("cmn");
    wordset = {
      "Maximum post size to show": "可顯示的最長推文",
      "Maximum post size to send": "可發送的最長推文",
      "characters": "字元",
      "WebTorrent": "網頁訊流",
      "WebTorrent support to display shortened URL media": "網頁訊流支援顯示短版網址媒體",
      "External IP:": "外部網路位址：",
      "External Port 1 (TCP):": "外部通訊埠一（TCP）：",
      "External Port 2 (TCP+UDP):": "外部通訊埠二（TCP+UDP）：",
      "Test open port (external site)": "測試打開的通訊埠（外部站臺）",
      "Actions ▼": "動作 ▼",
      "Active DHT nodes:": "分散式雜湊表活躍節點：",
      "Add DNS": "加入 DNS",
      "Add peer": "加入對等點",
      "ajax_error": "Ajax 錯誤：%{error}", // JavaScript error
      "All users publicly followed by": "所有使用者被公開跟隨自",
      "Available": "可用", // username is available
      "Appearance": "外觀",
      "Apply": "套用",
      "Block chain information": "區塊鏈資訊",
      "Block chain is up-to-date, twister is ready to use!": "區塊鏈已更新，Twister 備妥待用！",
      "Block generation": "區塊生成：",
      "busted_oh": "噢，不！",
      "busted_avowal": "檢測到有人嘗試注入惡意材料",
      "btn_ok": "沒問題",
      "Cancel": "取消",
      'cant_get_requested_resourse': "無法於 %{link} 取得資源\n  狀態：%{status}。",
      "clear_cache": "清空快取",
      "Confirm": "確認",
      "сonfirm_group_leaving_header": "確認要離開群組",
      "сonfirm_group_leaving_body": "確定要離開 %{alias} 群組？",
      "confirm_switch_to_network":
            "本地守護程式還未連線到網路，或者區塊鏈已經過期。\n" +
            "如果您待在這個頁面，您的動作可能沒有效用。\n" +
            "您要檢查 [網路] 狀態頁面(%{page}) 做為替代嗎？",
      "confirm_terminate_daemon": "確定要退離守護程式？\nTwister 客戶端將停止作用。",
      "confirm_unfollow_@": "確定要取消跟隨 @%{alias}？",
      "confirm_uri_shortener_clear_cache": "確定要清空瀏覽器的短版網址快取？",
      "Change user": "變更使用者",
      "Checking...": "檢查…", // checking if username is available
      "Collapse": "塌縮", // smaller view of a post
      "Configure block generation": "組配區塊生成",
      "Connections:": "連接：", // to network
      "Connection lost.": "連線已經中斷。",
      "daemon_is_obsolete": "Twister 守護程式已過時，必須使用 %{versionReq} 或更高版本",
      "days": "%{smart_count} 天",
      "Detailed information": "詳細資訊",
      "DHT network down.": "分散式雜湊表 網路斷線。",
      "Direct Messages": "私人訊息",
      "Group Messages": "群組訊息",
      "Group Messages — New Group Creation": "群組訊息 — 新群組建立",
      "Group Messages — Join Group": "群組訊息 — 參與群組",
      "group_key_cant_import": "無法匯入私人訊息的群組金鑰",
      "group_key_is_invalid_perhaps": "也許金鑰無效",
      "group_key_was_imported": "已匯入私人訊息群組 %{alias} 的金鑰。\n"
       +"很快就能擷取到它的訊息。",
      "direct_messages_with": "與 %{username} 的私人訊息",
      "Disable": "停用",
      "display_mentions": "顯示提及次數",
      "Display retransmissions": "顯示轉推次數",
      "DNS to obtain list of peers:": "用來取得對等點列表的 DNS：",
      "do_not_show_it_again": "再也不要顯示它",
      "downloading_block_chain": "正在下載區塊鏈，繼續之前請稍待 (區塊鏈仍落後 %{days} 天)。",
      "download_posts_status": "已下載 %{portion} 則推文", // Downloaded 10/30 posts
      "Enable": "啟用",
      "error": "錯誤：%{error}",
      "error_connecting_to_daemon": "連線到本地 Twister 守護程式時發生錯誤。",
      "Error in 'createwalletuser' RPC.": "在 createwalletuser RPC 時發生錯誤。",
      "Error in 'importprivkey'": "在 importprivkey RPC 時發生錯誤：%{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "在 sendnewusertransaction RPC 時發生錯誤。",
      "Expand": "展開", // larger view of a post
      "Favorite": "收藏",
      "File APIs not supported in this browser.": "這個瀏覽器不支援檔案應用程式介面。",
      "Follow": "跟隨",
      "Following config": "跟隨組配",
      "select_way_to_follow_@": "您想以哪種方式跟隨 @%{alias}",
      "Followed by": "跟隨者",
      "followed_by": "被 %{username} 跟隨",
      "Followers": "跟隨者",
      "Followers_of": "@%{alias} 的跟隨者",
      "Following": "跟隨中",
      "Following users": "跟隨的使用者",
      "Force connection to peer:": "強制連接到對等點：",
      "General information": "一般資訊",
      "Generate blocks (send promoted messages)": "產生區塊 (發送推廣訊息)",
      "Home": "首頁", // homepage
      "hours": "%{smart_count} 小時",
      "Internal error: lastPostId unknown (following yourself may fix!)": "內部錯誤：lastPostId 不明 (跟隨您自己也許就能修正！)",
      "Known peers:": "已知對等點：",
      "Last block is ahead of your computer time, check your clock.": "最後一段區塊已經超前了您的電腦時間，請檢查您的系統時鐘。",
      "mentions_at": "提及 @%{user}",
      "minutes": "%{smart_count} 分鐘",
      "Must be 16 characters or less.": "必須是 16 個字元或更少。", // username
      "Network": "網路",
      "Network config": "網路組配",
      "Network status": "網路狀態",
      "New direct message...": "新的私人訊息…",
      "New Post...": "新推文…",
      "New group": "新群組",
      "Group description": "群組描述",
      "Peers to invite": "可邀請的對等點",
      "Join group": "參與群組",
      "Select group(s)": "選取群組",
      "Create": "建立",
      "Join": "參與",
      "Invite": "邀請",
      "Invite peers": "邀請對等點",
      "Leave group": "離開群組",
      "You got": "您得到",
      "in postboard": "於推布欄",
      "in search result": "於搜尋結果",
      "in top trends": "於熱門趨勢",
      "new_posts": "%{smart_count} 則新推文",
      "new_mentions": "%{smart_count} 次新提及",
      "new_direct_messages": "%{smart_count} 則新的私人訊息",
      "new_group_messages": "%{smart_count} 則新的群組訊息",
      "nobody": "沒有人", // used to promote a post without attaching the user
      "Not available": "無名可用", // username is not available
      "warn_following_not_any": "沒有跟隨任何推友！\n請搜尋並跟隨某人。",
      "warn_followers_not_all": "好吧，目前還沒有容易的方式，可以知道所有跟隨您的人。\n"
       +"計數器衹有指示已知的對等點數量，他們正在共享您的推文訊流。\n"
       +"以下列表包含的推友，大多是您所跟隨的人。",
      "warn_followers_not_all_of": "好吧，目前還沒有容易的方式，可以知道某人的所有跟隨者。\n"
       +"以下列表衹包括幾個，或許是公開跟隨 @%{alias} 的人。",
      "notify_desktop_error": "Twister 無法進行桌面通知：發生不明錯誤。",
      "notify_desktop_perm_denied": "Twister 無法進行桌面通知：權限被拒。\n\n如果您要獲得通知，請在您的瀏覽器設定值中，允許它們用於 %{this_domain}。",
      "notify_desktop_test": "所有人都在推文。\n歡迎您的加入。",
      "notify_desktop_title": "注意，這裡是 Twister！",
      "post_preview_dummy": '這裡有 *粗體*、~斜體~、-刪除線- 以及 _加底線_ 的文字。\n'
        + '同樣但是例外排除的：`*粗體*、~斜體~、-刪除線- 以及 _加底線_`。\n'
        + '鏈結到 [最棒的圖標](%{logo}) 以及我們華麗的站臺：%{site}。',
      "Number of blocks in block chain:": "區塊鏈中的區塊數量：",
      "Number of CPUs to use": "使用的中央處理器數量：",
      "Only alphanumeric and underscore allowed.": "衹允許文數字與底線。",
      "peer address": "對等點位址",
      "Private": "私人的",
      "Profile": "側寫",
      "Postboard": "推布欄",
      "post": "推文", // verb - button to post a message
      "Post to promote:": "用來推廣的推文：",
      "Posts": "推文",
      "propagating_nickname": "將暱稱 %{username} 傳播到網路…",
      "Public": "公開",
      "Refresh": "重新整理",
      "retransmit_this": "轉發這則推文給您的跟隨者？",
      "Reply": "回覆",
      "Reply...": "回覆…",
      "reply_to": "回覆給 %{fullname}",
      "Retransmit": "轉發",
      "Retransmits": "轉發",
      "Retransmitted by": "轉發自",
      "Switch to Reply": "切換為回覆",
      "Switch to Retransmit": "切換為轉發",
      "search": "搜尋",
      "seconds": "%{smart_count} 秒",
      "send": "發送",
      "Send post with username": "發送推文所用的使用者名稱：",
      "send_DM": "發送私人訊息",
      "Sent Post to @": "已發送推文給 @",
      "Setup account": "設定帳號",
      "shorten_URI": "短版網址",
      "shorten_URI_enter_link": "輸入長版網址鏈結。\n"
       +"註記：短版網址將為您產生空的推文以包含完整網址。\n"
       +"這一則特殊推文不會顯示於 Twister 客戶端，但是您的推文計數將會增加。",
      "shorten_URI_its_public_is_it_ok": "您的鏈結將是公開可讀的！確定這樣可以嗎？",
      "URI_shortener": "網址縮短器",
      "The File APIs are not fully supported in this browser.": "這個瀏覽器並不完全支援檔案應用軟體介面。",
      "time_ago": "%{time} 之前", // 5 minutes ago
      "Time of the last block:": "最後一段區塊的時間：",
      "Type message here": "在這裡輸入訊息",
      "Unfollow": "取消跟隨",
      "Update": "更新",
      "Auto updating": "自動更新",
      'updates_are_available': '有可用更新',
      'updates_not_available': '沒有可用更新',
      'updates_check_client': '檢查有無客戶端軟體更新',
      'updates_repo_overview': '目前我們是基於 %{repo} 分支 %{branch} 上於\n  %{date} 的定案 %{commit}\n'
       +'但是原始碼的最前緣已經位於\n  定案 %{commitUpstream} 的 %{dateUpstream}。',
      'updates_checkout_diff': '簽出 [GitHub 上的差異](%{link}) 來瞭解變更了什麼。',
      'updates_checkout_diff_nfmt': '簽出 GitHub 上的差異來瞭解變更了什麼：\n  %{link}',
      'updates_upstream_isnt_changed': '相應分支的原始碼儲存庫似乎沒有變更。',
      "Updating status...": "更新狀態…", // status of block chain
      'new_account_briefing': '正在將新建立的帳號傳播到網路中。'
       +'請不要關閉這個視窗，也許需要幾分鐘。\n\n'
       +'您的密鑰是：*%{secretKey}*\n\n'
       +'強烈建議您利用這段時間去儲存您的密鑰。'
       +'印出來、做螢幕快照、使用您手機中的照相功能，或是將它寫在紙上。\n\n'
       +'從不同的電腦使用這個帳號時將會需要密鑰。'
       +'如果您一旦遺失密鑰，您的帳號將永遠被鎖住'
       +'(註記：~這是開發中軟體，它也許會崩潰而造成資料漏失~)。\n\n'
       +'請稍待。當您還沒唸完 ~decentralization~ 之前，就會顯示「確定」按鈕。',
      "user_not_yet_accepted": "其他對等點尚未接受這位新使用者。\n"+
                "很不幸地，無法儲存側寫\n"+
                "或在這個狀態下發送任何推文。\n\n"+
                "請稍待幾分鐘再繼續。\n\n"+
                "「儲存變更」將在處理完成時\n"+
                "自動啟用。（我保證這是您\n"+
                "使用 Twister 之前的最後一次\n"+
                "等待）。\n\n"+
                "祕訣：趁這時選擇您的頭像！",
      "users_mentions": "@%{username} 的提及次數",
      "users_profile": "%{username} 的側寫",
      "username_undefined": "未定義的使用者名稱，這是登入必要項目。",
      "View": "檢視",
      "View All": "檢視全部",
      "Who to Follow": "可以跟隨誰",
      "Your message was sent!": "您的訊息已發送！",
      "twister login": "Twister 登入",
      "Existing local users": "既有本地使用者",
      "Or...": "或…",
      "Create a new user": "建立新使用者",
      "Login": "登入",
      "Check availability": "檢查可用程度",
      "Create this nickname": "建立這個暱稱",
      "Type nickname here": "在這裡輸入暱稱",
      "Import secret key": "匯入密鑰",
      "52-characters secret": "52 個字元的密鑰",
      "With nickname": "與暱稱",
      "Import key": "匯入金鑰",
      "Client Version:": "客戶端版本：",
      "Mining difficulty:": "挖礦難度：",
      "Block generation status": "區塊生成狀態",
      "Current hash rate:": "目前雜湊比率：",
      "Terminate Daemon:": "終止守護程式：",
      "Exit": "退離",
      "Save Changes": "儲存變更",
      "profile_saved": "側寫資料已被儲存到分散式雜湊表。",
      "profile_not_saved": "無法儲存側寫資料。",
      "Secret key:": "密鑰：",
      "You have to log in to post messages.": "您必須登入才能推送訊息。",
      "You have to log in to post replies.": "您必須登入才能回覆訊息。",
      "You have to log in to retransmit messages.": "您必須登入才能轉發訊息。",
      "You have to log in to use direct messages.": "您必須登入才能使用私人訊息。",
      "You have to log in to follow users.": "您必須登入才能跟隨使用者。",
      "You are not following anyone because you are not logged in.": "您沒有跟隨任何人因為您還未登入。",
      "You don't have any followers because you are not logged in.": "您沒有任何跟隨者因為您還未登入。",
      "No one can mention you because you are not logged in.": "沒有人可以提及您因為您還未登入。",
      "You don't have any profile because you are not logged in.": "您沒有任何側寫因為您還未登入。",
      "Options": "選項",
      "Switch to Promoted posts": "切換至推廣推文",
      "Switch to Normal posts": "切換至一般推文",
      "Use language": "使用語言",
      "Ignore": "忽略",
      "Ignore and clear out": "忽略並清空",
      "Theme": "布景主題",
      "Keys": "金鑰",
      "Notifications": "通知",
      "Desktop notifications": "桌面通知",
      "Sound notifications": "聲音通知",
      "Volume": "音量",
      "Test": "測試",
      "Send key": "發送金鑰",
      "Posts display": "推文顯示",
      "Post editor": "推文編輯器",
      "Post preview": "推文預覽",
      "Inline image preview": "內聯影像預覽",
      "Display": "顯示",
      "Line feeds": "送列符號",
      "Markout": "註明標記",
      "Supported punctuations:": "支援的標點符號：",
      "Supported emotions:": "支援的表情符號：",
      "Supported signs:": "支援的符號：",
      "Supported fractions:": "支援的分數：",
      "Automatic unicode conversion options": "萬國碼自動轉換選項",
      "Convert punctuations to unicode": "轉換標點符號為萬國碼",
      "Convert emotions codes to unicode symbols": "轉換表情符號編碼為萬國碼符號",
      "Convert common signs to unicode": "轉換一般符號為萬國碼",
      "Convert fractions to unicode": "轉換分數為萬國碼",
      "Convert all": "轉換全部",
      "Auto": "自動",
      "Original": "原版",
      "none": "無",
      "Custom": "自訂",
      "Mentions": "提及",
      "Use proxy for image preview only": "代理伺服器衹用於影像預覽",
      "Use external links behind a proxy": "使用代理伺服器後方的外部鏈結",
      "There aren't any posts with this hashtag.": "沒有任何推文包含這個雜湊標籤。",
      "Split only new post": "衹分割新的推文",
      "Split all": "分割全部",
      "Don't split": "不要分割",
      "Split long posts": "分割長的推文",
      "Posts that begin with mention": "以提及做為開頭的推文",
      "Show all": "全部顯示",
      "Show only if I am in": "衹顯示有我的推文",
      "Show if it's between users I follow": "如果是我跟隨的使用者才顯示",
      "Postboard displays": "推布欄顯示",
      "RTs those are close to original twist": "與原始推文相近的轉推",
      "Show if the original is older than": "衹顯示原始推文出現超過",
      "hour(s)": "小時",
      "second(s)": "秒",
      "only positive numbers!": "衹允許正數！",
      "Language filtering": "語言篩選",
      "By blacklist": "依照黑名單",
      "By whitelist": "依照白名單",
      "Comma separated ISO 639-3 language codes": "逗號分隔的 ISO 639-3 語言編碼",
      "Accuracy": "準確度",
      "Simulation mode": "模擬模式",
      "This post is treated by language filter": "這則推文由語言篩選器所 %{treated}。",
      "blocked": "阻斷",
      "passed": "傳遞",
      "not analyzed": "無法分析",
      "Reason: this": "原因：%{this}",
      "this doesnt contain that": "%{this} 不包含 %{that}",
      "this is undefined": "%{this} 未定義",
      "blacklist": "黑名單",
      "whitelist": "白名單",
      "language of this": "語言所屬",
      "its undefined language": "它是種未定義的語言",
      "its this, blacklisted": "它是 %{this}，已列入黑名單",
      "its this, whitelisted": "它是 %{this}，已列入白名單",
      "Most possible language: this": "最可能的語言：%{this}",
      "Scope of usage": "使用的範圍",
      "Show with every user name": "每位使用者都要顯示",
      "Show at profile modal only": "衹有側寫頁面才要顯示",
      "Show if a user follows me": "跟隨我的使用者才顯示",
      "follows you": "跟隨您",
      "Show conversation": "顯示會話",
      "Mark all as read": "標記所有為已讀",
      "show_more_count": "%{smart_count} 更多…",
      "hide": "隱藏",
      "Show more in this conversation...": "顯示更多會話內容…",
      "conversation_title": "@%{username} 的會話",
      "copy_to_clipboard": "按下 Ctrl/Cmd+C 來拷貝，然後按下 Enter 來關閉",
      "Normal posts": "一般推文",
      "Promoted posts": "推廣推文",
      'promoted': 'Promoted',
      "Messages": "訊息",
      "Edit profile": "編輯側寫",
      "Top Trends": "熱門趨勢",
      "Twistday Reminder": "開推周年紀念日提醒訊息",
      "Show upcoming in near future": "顯示不久即將迎來慶祝的人",
      "Who's celebrating Twistday": "誰正在慶祝開推周年紀念日",
      "Today's luckies:": "今天的幸運兒：",
      "Upcoming ones:": "即將迎來慶祝的人：",
      "post_rt_sign_prep": "再次推文自",
      "post_rt_time_prep": "於",
      "undo": "復原",
      "Daemon exited...": "已退離守護程式…",
      "Secret Key": "密鑰",
      "Copy to clipboard": "拷貝到剪貼簿",
      "Full name here": "全名",
      "Describe yourself": "描述您自己",
      "Location": "位置",
      "website": "網站",
      "Tox address": "Tox 位址",
      "Bitmessage address": "Bitmessage 位址",
      "Language": "語言",
      "Sound": "聲音",
      "Users": "使用者",
      "Direct Message's copy to self": "私人訊息拷貝給自己",
      "Terminate Daemon": "終止守護程式",
      "New post": "新推文",
      "Search": "搜尋",
      "Direct Msg": "私人訊息",
      "Traffic information": "流量資訊",
      "DHT Torrents:": "分散式雜湊表訊流：",
      "Peers:": "對等點：",
      "Peer List Size:": "對等點列表大小：",
      "Active Requests:": "活躍請求：",
      "Download:": "下載：",
      "Upload:": "上傳：",
      "DHT Download:": "分散式雜湊表下載：",
      "DHT Upload:": "分散式雜湊表上傳：",
      "IP Overhead Download:": "網際網路協定耗費下載：",
      "IP Overhead Upload:": "網際網路協定耗費上傳：",
      "Payload Download:": "酬載下載：",
      "Payload Upload:": "酬載上傳：",
      "No favs here because you are not logged in.": "因為您還沒登入，這裡不會顯示收藏。",
      "users_favs": "@%{username} 的收藏",
      "Favorites": "收藏",
      "You have to log in to favorite messages.": "您必須登入以收藏訊息。",
      "fav_this": "它為您所獨有？",
      "Last activity": "最近一次活動",
      "New Users": "新近使用者",
      "Live tracking" : "即時追蹤"
    };
}

if(preferredLanguage == "yue"){
    polyglot.locale("yue");
    wordset = {
      "Maximum post size to show": "可顯示其至長推文",
      "Maximum post size to send": "可寄個其至長推文",
      "characters": "字符",
      "WebTorrent": "網頁訊流",
      "WebTorrent support to display shortened URL media": "網頁訊流支援顯示短版網址媒體",
      "External IP:": "外部網絡位址：",
      "External Port 1 (TCP):": "外部通訊接口一（TCP）：",
      "External Port 2 (TCP+UDP):": "外部通訊接口二（TCP+UDP）：",
      "Test open port (external site)": "測試開其通訊接口（外部站台）",
      "Actions ▼": "動作 ▼",
      "Active DHT nodes:": "分散式雜湊表生猛節點：",
      "Add DNS": "添加 DNS",
      "Add peer": "添加對等點",
      "ajax_error": "Ajax 錯誤：%{error}", // JavaScript error
      "All users publicly followed by": "所有用家畀公開發摟自",
      "Available": "可用", // username is available
      "Appearance": "外觀",
      "Apply": "改實",
      "Block chain information": "區塊捵資訊",
      "Block chain is up-to-date, twister is ready to use!": "區塊捵已更加新，Twister 備妥待用！",
      "Block generation": "區塊生成：",
      "busted_oh": "噢，毋！",
      "busted_avowal": "檢測到有人嘗試斟惡意材料",
      "btn_ok": "無問題",
      "Cancel": "毋做",
      'cant_get_requested_resourse': "無法嚮 %{link} 攎資源\n  狀態：%{status}。",
      "clear_cache": "清空快取",
      "Confirm": "落實",
      "сonfirm_group_leaving_header": "落實要行開谷",
      "сonfirm_group_leaving_body": "直煞要行開 %{alias} 谷嘛？",
      "confirm_switch_to_network":
            "本地守護程式尚未連綫到網絡，定抑區塊捵經已過期。\n" +
            "若果閣下待繫個頁面，閣下其動作似無效用。\n" +
            "閣下要檢查 [網絡] 狀態頁面(%{page}) 做為幫代啊？",
      "confirm_terminate_daemon": "直煞要退出守護程式嘛？\nTwister 客戶端將閘住作用。",
      "confirm_unfollow_@": "直煞要毋做發摟 @%{alias}嘛？",
      "confirm_uri_shortener_clear_cache": "直煞要清空瀏覽器其短版網址快取嘛？",
      "Change user": "更改用家",
      "Checking...": "檢查…", // checking if username is available
      "Collapse": "塌縮", // smaller view of a post
      "Configure block generation": "組襯區塊生成",
      "Connections:": "接通：", // to network
      "Connection lost.": "連綫經已掹開。",
      "daemon_is_obsolete": "Twister 守護程式過着時，必須用 %{versionReq} 或更加高版本",
      "days": "%{smart_count} 天",
      "Detailed information": "詳細資訊",
      "DHT network down.": "分散式雜湊表 網絡斷綫。",
      "Direct Messages": "私人口訊",
      "Group Messages": "谷口訊",
      "Group Messages — New Group Creation": "谷口訊 — 新谷建立",
      "Group Messages — Join Group": "谷口訊 — 埋份谷",
      "group_key_cant_import": "無法匯入私人口訊其谷密碼匙",
      "group_key_is_invalid_perhaps": "亦許密碼匙無效",
      "group_key_was_imported": "已匯入私人口訊谷 %{alias} 其密碼匙。\n"
       +"好快就能擷取到佢其口訊。",
      "direct_messages_with": "同 %{username} 其私人口訊",
      "Disable": "停用",
      "display_mentions": "顯示提及次數",
      "Display retransmissions": "顯示轉推次數",
      "DNS to obtain list of peers:": "用來攎對等點列表其 DNS：",
      "do_not_show_it_again": "再亦毋好顯示佢",
      "downloading_block_chain": "取得緊區塊捵，繼續之前請稍待 (區塊捵仍落後 %{days} 天)。",
      "download_posts_status": "已取得 %{portion} 則推文", // Downloaded 10/30 posts
      "Enable": "啟用",
      "error": "錯誤：%{error}",
      "error_connecting_to_daemon": "連綫到本地 Twister 守護程式時發生錯誤。",
      "Error in 'createwalletuser' RPC.": "繫 createwalletuser RPC 時發生錯誤。",
      "Error in 'importprivkey'": "繫 importprivkey RPC 時發生錯誤：%{rpc}",
      "Error in 'sendnewusertransaction' RPC.": "繫 sendnewusertransaction RPC 時發生錯誤。",
      "Expand": "展開", // larger view of a post
      "Favorite": "收匿",
      "File APIs not supported in this browser.": "個瀏覽器毋支援快勞應用程式界面。",
      "Follow": "發摟",
      "Following config": "發摟組襯",
      "select_way_to_follow_@": "閣下諗俾焉種方式發摟 @%{alias}",
      "Followed by": "發摟者",
      "followed_by": "畀 %{username} 發摟",
      "Followers": "發摟者",
      "Followers_of": "@%{alias} 其發摟者",
      "Following": "發摟中",
      "Following users": "發摟其用家",
      "Force connection to peer:": "夾硬接通到對等點：",
      "General information": "一般資訊",
      "Generate blocks (send promoted messages)": "產生區塊 (寄個推廣口訊)",
      "Home": "頭版", // homepage
      "hours": "%{smart_count} 個鐘",
      "Internal error: lastPostId unknown (following yourself may fix!)": "內部錯誤：lastPostId 未知 (發摟閣下自己亦許就能修正！)",
      "Known peers:": "已知對等點：",
      "Last block is ahead of your computer time, check your clock.": "最後一橛區塊經已超前着閣下其電腦時間，請檢查閣下其系統時鐘。",
      "mentions_at": "提及 @%{user}",
      "minutes": "%{smart_count} 分鐘",
      "Must be 16 characters or less.": "必須係 16 個字符或更加少。", // username
      "Network": "網絡",
      "Network config": "網絡組襯",
      "Network status": "網絡狀態",
      "New direct message...": "新淨私人口訊…",
      "New Post...": "新推文…",
      "New group": "新谷",
      "Group description": "谷斟",
      "Peers to invite": "可邀請其對等點",
      "Join group": "埋份谷",
      "Select group(s)": "繫度揀谷",
      "Create": "建立",
      "Join": "埋份",
      "Invite": "邀請",
      "Invite peers": "邀請對等點",
      "Leave group": "行開谷",
      "You got": "閣下得到",
      "in postboard": "嚮推布欄",
      "in search result": "嚮揾結果",
      "in top trends": "嚮熱門趨勢",
      "new_posts": "%{smart_count} 則新推文",
      "new_mentions": "%{smart_count} 次新提及",
      "new_direct_messages": "%{smart_count} 則新淨私人口訊",
      "new_group_messages": "%{smart_count} 則新淨谷口訊",
      "nobody": "無人", // used to promote a post without attaching the user
      "Not available": "無名可用", // username is not available
      "warn_following_not_any": "無發摟任何推友！\n請揾兼發摟某人。",
      "warn_followers_not_all": "好吧，而今尚未易其方式，得知所有發摟閣下其人。\n"
       +"計數器惟有指示已知其對等點數量，佢地正係共享閣下其推文訊流。\n"
       +"以下列表埋其推友，大多係閣下所發摟其人。",
      "warn_followers_not_all_of": "好吧，而今尚未易其方式，得知某人其所有發摟者。\n"
       +"以下列表衹埋幾個，一係係公開發摟 @%{alias} 其人。",
      "notify_desktop_error": "Twister 無法去枱面通知：發生未知錯誤。",
      "notify_desktop_perm_denied": "Twister 無法去枱面通知：權限畀拒。\n\n若果閣下要攎到通知，請繫閣下其瀏覽器設定值中，允許佢用嚮 %{this_domain}。",
      "notify_desktop_test": "冚不論都繫推文。\n歡迎閣下其添加。",
      "notify_desktop_title": "留意，呢度係 Twister！",
      "post_preview_dummy": '呢度有 *粗字*、~斜字~、-刪清綫- 同埋 _加底綫_ 其文字。\n'
        + '同樣但例外飛起其：`*粗字*、~斜字~、-刪清綫- 同埋 _加底綫_`。\n'
        + '連結到 [一流其圖標](%{logo}) 同埋我地華麗其站台：%{site}。',
      "Number of blocks in block chain:": "區塊捵中其區塊數量：",
      "Number of CPUs to use": "用其中央處理器數量：",
      "Only alphanumeric and underscore allowed.": "衹允許文數字同底綫。",
      "peer address": "對等點位址",
      "Private": "私人其",
      "Profile": "個人資料",
      "Postboard": "推布欄",
      "post": "推文", // verb - button to post a message
      "Post to promote:": "用來推廣其推文：",
      "Posts": "推文",
      "propagating_nickname": "將網名 %{username} 傳播到網絡…",
      "Public": "公開",
      "Refresh": "整返",
      "retransmit_this": "轉發呢則推文畀閣下其發摟者嘛？",
      "Reply": "覆返",
      "Reply...": "覆返…",
      "reply_to": "覆返畀 %{fullname}",
      "Retransmit": "轉發",
      "Retransmits": "轉發",
      "Retransmitted by": "轉發自",
      "Switch to Reply": "切換為覆返",
      "Switch to Retransmit": "切換為轉發",
      "search": "揾耶",
      "seconds": "%{smart_count} 秒",
      "send": "寄個",
      "Send post with username": "寄個推文所用其用家名稱：",
      "send_DM": "寄個私人口訊",
      "Sent Post to @": "已寄個推文畀 @",
      "Setup account": "設定戶口",
      "shorten_URI": "短版網址",
      "shorten_URI_enter_link": "輸入長版網址連結。\n"
       +"註記：短版網址將為閣下產生空其推文俾埋齊煞網址。\n"
       +"則特意推文毋會顯示嚮 Twister 客戶端，但閣下其推文計數會提返。",
      "shorten_URI_its_public_is_it_ok": "閣下其連結將係公開可讀其！直煞敢得啩？",
      "URI_shortener": "網址縮短器",
      "The File APIs are not fully supported in this browser.": "個瀏覽器兼毋完全支援快勞應用軟件界面。",
      "time_ago": "%{time} 之前", // 5 minutes ago
      "Time of the last block:": "最後一橛區塊其時間：",
      "Type message here": "繫度輸入口訊",
      "Unfollow": "毋做發摟",
      "Update": "更加新",
      "Auto updating": "自動更加新",
      'updates_are_available': '有可用更加新 ',
      'updates_not_available': '無可用更加新 ',
      'updates_check_client': '檢查有無客戶端軟件更加新 ',
      'updates_repo_overview': '而今我地係跟埋 %{repo} 分支 %{branch} 上嚮\n  %{date} 其定案 %{commit}\n'
       +'但原始碼其至前緣經已位嚮\n  定案 %{commitUpstream} 其 %{dateUpstream}。',
      'updates_checkout_diff': '簽走 [GitHub 上其差異](%{link}) 來知更改着麼耶。',
      'updates_checkout_diff_nfmt': '簽走 GitHub 上其差異來知更改着麼：\n  %{link}',
      'updates_upstream_isnt_changed': '相應分支其原始碼保存庫似乎無更改。',
      "Updating status...": "更加新狀態…", // status of block chain
      'new_account_briefing': '正係將新建立其戶口傳播到網絡中。'
       +'請毋好閂埋個視窗，亦許需要幾分鐘。\n\n'
       +'閣下其密碼匙係：*%{secretKey}*\n\n'
       +'強勍建議閣下利用呢段時間去保存閣下其密碼匙。'
       +'顯示來、做熒幕快照、用閣下手提電話中其影相功能，定係將佢寫繫紙上。\n\n'
       +'由毋同其電腦用個戶口時會需要密碼匙。'
       +'若果閣下一旦遺失密碼匙，閣下其戶口將永遠畀鎖住 '
       +'(註記：~ 呢係開發中軟件，佢亦許會冧而做成資料漏失 ~)。\n\n'
       +'請稍待。當閣下尚未讀煞 ~decentralization~ 之前，就會顯示「直煞」掣。',
      "user_not_yet_accepted": "遞滴對等點未接受呢位新用家。\n"+
                "好毋幸地，無法保存個人資料\n"+
                "或繫個狀態下寄個任何推文。\n\n"+
                "請稍待幾分鐘再繼續。\n\n"+
                "「保存更改」將繫處理搞直時\n"+
                "自動啟用。（我包呢係閣下\n"+
                "用 Twister 之前其最後一次\n"+
                "等待）。\n\n"+
                "秘訣：趁呢時揀閣下其頭像！",
      "users_mentions": "@%{username} 其提及次數",
      "users_profile": "%{username} 其個人資料",
      "username_undefined": "未定義其用家名稱，呢係登入要寫項目。",
      "View": "查看",
      "View All": "查看煞",
      "Who to Follow": "得發摟焉位",
      "Your message was sent!": "閣下其口訊已寄個！",
      "twister login": "Twister 登入",
      "Existing local users": "既有本地用家",
      "Or...": "或…",
      "Create a new user": "建立新用家",
      "Login": "登入",
      "Check availability": "檢查可用程度",
      "Create this nickname": "建立個網名",
      "Type nickname here": "繫度輸入網名",
      "Import secret key": "匯入密碼匙",
      "52-characters secret": "52 個字符其密碼匙",
      "With nickname": "同網名",
      "Import key": "匯入密碼匙",
      "Client Version:": "客戶端版本：",
      "Mining difficulty:": "掘礦難度：",
      "Block generation status": "區塊生成狀態",
      "Current hash rate:": "而今雜湊比率：",
      "Terminate Daemon:": "終止守護程式：",
      "Exit": "退出",
      "Save Changes": "保存更改",
      "profile_saved": "個人資料資料已畀保存到分散式雜湊表。",
      "profile_not_saved": "無法保存個人資料資料。",
      "Secret key:": "密碼匙：",
      "You have to log in to post messages.": "閣下必須登入先能推送口訊。",
      "You have to log in to post replies.": "閣下必須登入先能覆返口訊。",
      "You have to log in to retransmit messages.": "閣下必須登入先能轉發口訊。",
      "You have to log in to use direct messages.": "閣下必須登入先能用私人口訊。",
      "You have to log in to follow users.": "閣下必須登入先能發摟用家。",
      "You are not following anyone because you are not logged in.": "閣下無發摟任何人因為閣下尚未登入。",
      "You don't have any followers because you are not logged in.": "閣下無發摟者因為閣下尚未登入。",
      "No one can mention you because you are not logged in.": "無人得提及閣下因為閣下尚未登入。",
      "You don't have any profile because you are not logged in.": "閣下無個人資料因為閣下尚未登入。",
      "Options": "隨寫",
      "Switch to Promoted posts": "切換至推廣推文",
      "Switch to Normal posts": "切換至一般推文",
      "Use language": "用語言",
      "Ignore": "毋理",
      "Ignore and clear out": "毋理兼清空",
      "Theme": "布景主題",
      "Keys": "密碼匙",
      "Notifications": "通知",
      "Desktop notifications": "枱面通知",
      "Sound notifications": "聲音通知",
      "Volume": "音量",
      "Test": "測試",
      "Send key": "寄個密碼匙",
      "Posts display": "推文顯示",
      "Post editor": "推文編輯器",
      "Post preview": "推文預覽",
      "Inline image preview": "內聯錄像預覽",
      "Display": "顯示",
      "Line feeds": "送列符號",
      "Markout": "註明嘜頭",
      "Supported punctuations:": "支援其標點符號：",
      "Supported emotions:": "支援其表情符號：",
      "Supported signs:": "支援其符號：",
      "Supported fractions:": "支援其分數：",
      "Automatic unicode conversion options": "萬國碼自動轉換隨寫",
      "Convert punctuations to unicode": "轉換標點符號為萬國碼",
      "Convert emotions codes to unicode symbols": "轉換表情符號編碼為萬國碼符號",
      "Convert common signs to unicode": "轉換一般符號為萬國碼",
      "Convert fractions to unicode": "轉換分數為萬國碼",
      "Convert all": "轉換煞",
      "Auto": "自動",
      "Original": "原版",
      "none": "無",
      "Custom": "自訂",
      "Mentions": "提及",
      "Use proxy for image preview only": "代理伺服器衹用嚮錄像預覽",
      "Use external links behind a proxy": "用代理伺服器後方其外部連結",
      "There aren't any posts with this hashtag.": "無推文埋個雜湊標籤。",
      "Split only new post": "衹斬開新淨推文",
      "Split all": "斬開煞",
      "Don't split": "毋好斬開",
      "Split long posts": "斬開長其推文",
      "Posts that begin with mention": "俾提及做為開頭其推文",
      "Show all": "冚顯示",
      "Show only if I am in": "衹顯示有我其推文",
      "Show if it's between users I follow": "若果係我發摟其用家先顯示",
      "Postboard displays": "推布欄顯示",
      "RTs those are close to original twist": "同原始推文相近其轉推",
      "Show if the original is older than": "衹顯示原始推文浮頭超過",
      "hour(s)": "個鐘",
      "second(s)": "秒",
      "only positive numbers!": "衹允許正數！",
      "Language filtering": "語言篩揀",
      "By blacklist": "依照黑名單",
      "By whitelist": "依照白名單",
      "Comma separated ISO 639-3 language codes": "撩下號分隔其 ISO 639-3 語言編碼",
      "Accuracy": "準確度",
      "Simulation mode": "模擬模式",
      "This post is treated by language filter": "呢則推文由語言篩揀器所 %{treated}。",
      "blocked": "杯葛",
      "passed": "傳遞",
      "not analyzed": "無法分析",
      "Reason: this": "因由：%{this}",
      "this doesnt contain that": "%{this} 毋埋 %{that}",
      "this is undefined": "%{this} 未定義",
      "blacklist": "黑名單",
      "whitelist": "白名單",
      "language of this": "語言所屬",
      "its undefined language": "佢係種未定義其語言",
      "its this, blacklisted": "佢係 %{this}，已列入黑名單",
      "its this, whitelisted": "佢係 %{this}，已列入白名單",
      "Most possible language: this": "至似其語言：%{this}",
      "Scope of usage": "用其範圍",
      "Show with every user name": "每位用家都要顯示",
      "Show at profile modal only": "惟有個人資料頁面先要顯示",
      "Show if a user follows me": "發摟我其用家先顯示",
      "follows you": "發摟閣下",
      "Show conversation": "顯示會話",
      "Mark all as read": "嘜頭所有為已讀",
      "show_more_count": "%{smart_count} 更加多…",
      "hide": "匿埋",
      "Show more in this conversation...": "顯示更加多會話內容…",
      "conversation_title": "@%{username} 其會話",
      "copy_to_clipboard": "撳 Ctrl/Cmd+C 來複製，跟住撳 Enter 來閂埋",
      "Normal posts": "一般推文",
      "Promoted posts": "推廣推文",
      'promoted': 'Promoted',
      "Messages": "口訊",
      "Edit profile": "編輯個人資料",
      "Top Trends": "熱門趨勢",
      "Twistday Reminder": "開推周年紀念日提醒口訊",
      "Show upcoming in near future": "顯示毋耐來緊迎來慶祝其人",
      "Who's celebrating Twistday": "焉位正係慶祝開推周年紀念日",
      "Today's luckies:": "今天其幸逳兒：",
      "Upcoming ones:": "來緊迎來慶祝其人：",
      "post_rt_sign_prep": "再次推文自",
      "post_rt_time_prep": "嚮",
      "undo": "揾返",
      "Daemon exited...": "已退出守護程式…",
      "Secret Key": "密碼匙",
      "Copy to clipboard": "複製到剪貼簿",
      "Full name here": "冚名",
      "Describe yourself": "斟閣下自己",
      "Location": "地步",
      "website": "網站",
      "Tox address": "Tox 位址",
      "Bitmessage address": "Bitmessage 位址",
      "Language": "語言",
      "Sound": "聲音",
      "Users": "用家",
      "Direct Message's copy to self": "私人口訊複製畀自己",
      "Terminate Daemon": "終止守護程式",
      "New post": "新推文",
      "Search": "揾耶",
      "Direct Msg": "私人口訊",
      "Traffic information": "流量資訊",
      "DHT Torrents:": "分散式雜湊表訊流：",
      "Peers:": "對等點：",
      "Peer List Size:": "對等點列表大細：",
      "Active Requests:": "生猛請求：",
      "Download:": "取得：",
      "Upload:": "上傳：",
      "DHT Download:": "分散式雜湊表取得：",
      "DHT Upload:": "分散式雜湊表上傳：",
      "IP Overhead Download:": "互聯網協定耗費取得：",
      "IP Overhead Upload:": "互聯網協定耗費上傳：",
      "Payload Download:": "酬載取得：",
      "Payload Upload:": "酬載上傳：",
      "No favs here because you are not logged in.": "因為閣下尚未登入，呢度毋會顯示收匿。",
      "users_favs": "@%{username} 其收匿",
      "Favorites": "收匿",
      "You have to log in to favorite messages.": "閣下必須登入俾收匿口訊。",
      "fav_this": "佢為閣下所獨有嘛？",
      "Last activity": "最近一次活動",
      "New Users": "新近用家",
      "Live tracking" : "即時追蹤"
    };
}

// uncomment to see all translated words replaced with filler
//for(var word in wordset){
//  wordset[word] = "AAAA";
//}

polyglot.extend(wordset);

// Text from HTML and not JavaScript is selected and translated at $(document).ready
// Add selectors here to translate the text and placeholders inside new UI
var fixedLabels = [

  // common text containers
  "a",
  "span",
  "title",
  "button",
  "option",
  "textarea",
  "h1",
  "h2",
  "h3",
  "h4",
  "h5",
  "h6",

  // An easy way to include new items in translation is to add the "label" class
  ".label",

  // navbar and home
  ".postboard-news",
  ".refresh-toptrends, .refresh-users, .view-all-users",
  ".userMenu-search-field",
  "a.dropdown-menu-item, a.direct-messages",
  ".post-interactions span",
  ".post-expand",
  ".post-context span",
  ".post-stats .stat-count div",
  ".postboard span",

  // following page
  ".mini-profile-actions span, .mini-profile-actions li",

  // network page
  ".network ul li span",
  ".network ul li label",
  ".network label",
  ".network input",

  ".module span",

  // mobile version //

  ".ui-btn-text",
  ".ui-title",
  ".ui-input-text"
];

$(function () {
    $(document).on('pagecreate', function () {
    var elems,content,i,e,LenElem,elem;
    var LenLabels = fixedLabels.length;

    for( i=0; i < LenLabels; i++ ){
      elems = $(fixedLabels[i]);
      LenElem = elems.length;
      for( e=0; e < LenElem; e++ ){
        elem = $(elems[e]);
        content = $.trim(elem.text());
        if(wordset[content]){
          elem.text( polyglot.t(content) );
        }else{
          // uncomment to see translations not found
          //console.log(content);
        }
        elemplacehold = elem.attr("placeholder");
        if(typeof elemplacehold != "undefined" && wordset[elemplacehold]){
          elem.attr("placeholder", polyglot.t( elemplacehold ) );
        }
      }
    }
  });

  if(!$.mobile)
    $(document).trigger('pagecreate');
});
